@@ -48,6 +48,7 @@ typedef struct {
 			inode_t *inode;
 			struct stat buf;
 			dict_t *dict;
+                        struct stat postparent;
 		} lookup_cbk;
 
 		/* stat */
@@ -72,56 +73,6 @@ typedef struct {
 			struct stat buf;
 		} fstat_cbk;
 
-		/* chmod */
-		struct {
-			fop_chmod_t fn;
-			loc_t loc;
-			mode_t mode;
-		} chmod;
-		struct {
-			fop_chmod_cbk_t fn;
-			int32_t op_ret, op_errno;
-			struct stat buf;
-		} chmod_cbk;
-
-		/* fchmod */
-		struct {
-			fop_fchmod_t fn;
-			fd_t *fd;
-			mode_t mode;
-		} fchmod;
-		struct {
-			fop_fchmod_cbk_t fn;
-			int32_t op_ret, op_errno;
-			struct stat buf;
-		} fchmod_cbk;
-
-		/* chown */
-		struct {
-			fop_chown_t fn;
-			loc_t loc;
-			uid_t uid;
-			gid_t gid;
-		} chown;
-		struct {
-			fop_chown_cbk_t fn;
-			int32_t op_ret, op_errno;
-			struct stat buf;
-		} chown_cbk;
-
-		/* fchown */
-		struct {
-			fop_fchown_t fn;
-			fd_t *fd;
-			uid_t uid;
-			gid_t gid;
-		} fchown;
-		struct {
-			fop_fchown_cbk_t fn;
-			int32_t op_ret, op_errno;
-			struct stat buf;
-		} fchown_cbk;
-
 		/* truncate */
 		struct {
 			fop_truncate_t fn;
@@ -131,7 +82,8 @@ typedef struct {
 		struct {
 			fop_truncate_cbk_t fn;
 			int32_t op_ret, op_errno;
-			struct stat buf;
+			struct stat prebuf;
+                        struct stat postbuf;
 		} truncate_cbk;
 
 		/* ftruncate */
@@ -143,21 +95,10 @@ typedef struct {
 		struct {
 			fop_ftruncate_cbk_t fn;
 			int32_t op_ret, op_errno;
-			struct stat buf;
+			struct stat prebuf;
+                        struct stat postbuf;
 		} ftruncate_cbk;
 
-		/* utimens */
-		struct {
-			fop_utimens_t fn;
-			loc_t loc;
-			struct timespec tv[2];
-		} utimens;
-		struct {
-			fop_utimens_cbk_t fn;
-			int32_t op_ret, op_errno;
-			struct stat buf;
-		} utimens_cbk;
-
 		/* access */
 		struct {
 			fop_access_t fn;
@@ -179,6 +120,7 @@ typedef struct {
 			fop_readlink_cbk_t fn;
 			int32_t op_ret, op_errno;
 			const char *buf;
+                        struct stat sbuf;
 		} readlink_cbk;
 
 		/* mknod */
@@ -193,6 +135,8 @@ typedef struct {
 			int32_t op_ret, op_errno;
 			inode_t *inode;
 			struct stat buf;
+                        struct stat preparent;
+                        struct stat postparent;
 		} mknod_cbk;
 
 		/* mkdir */
@@ -206,6 +150,8 @@ typedef struct {
 			int32_t op_ret, op_errno;
 			inode_t *inode;
 			struct stat buf;
+                        struct stat preparent;
+                        struct stat postparent;
 		} mkdir_cbk;
 
 		/* unlink */
@@ -216,6 +162,8 @@ typedef struct {
 		struct {
 			fop_unlink_cbk_t fn;
 			int32_t op_ret, op_errno;
+                        struct stat preparent;
+                        struct stat postparent;
 		} unlink_cbk;
 
 		/* rmdir */
@@ -226,6 +174,8 @@ typedef struct {
 		struct {
 			fop_rmdir_cbk_t fn;
 			int32_t op_ret, op_errno;
+                        struct stat preparent;
+                        struct stat postparent;
 		} rmdir_cbk;
 
 		/* symlink */
@@ -239,6 +189,8 @@ typedef struct {
 			int32_t op_ret, op_errno;
 			inode_t *inode;
 			struct stat buf;
+                        struct stat preparent;
+                        struct stat postparent;
 		} symlink_cbk;
 
 		/* rename */
@@ -251,6 +203,10 @@ typedef struct {
 			fop_rename_cbk_t fn;
 			int32_t op_ret, op_errno;
 			struct stat buf;
+                        struct stat preoldparent;
+                        struct stat postoldparent;
+                        struct stat prenewparent;
+                        struct stat postnewparent;
 		} rename_cbk;
 
 		/* link */
@@ -264,6 +220,8 @@ typedef struct {
 			int32_t op_ret, op_errno;
 			inode_t *inode;
 			struct stat buf;
+                        struct stat preparent;
+                        struct stat postparent;
 		} link_cbk;
 
 		/* create */
@@ -280,6 +238,8 @@ typedef struct {
 			fd_t *fd;
 			inode_t *inode;
 			struct stat buf;
+                        struct stat preparent;
+                        struct stat postparent;
 		} create_cbk;
 
 		/* open */
@@ -288,6 +248,7 @@ typedef struct {
 			loc_t loc;
 			int32_t flags;
 			fd_t *fd;
+                        int32_t wbflags;
 		} open;
 		struct {
 			fop_open_cbk_t fn;
@@ -324,7 +285,8 @@ typedef struct {
 		struct {
 			fop_writev_cbk_t fn;
 			int32_t op_ret, op_errno;
-			struct stat stbuf;
+                        struct stat prebuf;
+			struct stat postbuf;
 		} writev_cbk;
 
 		/* flush */
@@ -346,6 +308,8 @@ typedef struct {
 		struct {
 			fop_fsync_cbk_t fn;
 			int32_t op_ret, op_errno;
+                        struct stat prebuf;
+                        struct stat postbuf;
 		} fsync_cbk;
 
 		/* opendir */
@@ -555,6 +519,19 @@ typedef struct {
 			gf_dirent_t entries;
 		} readdir_cbk;
 
+                /* readdirp */
+		struct {
+			fop_readdirp_t fn;
+			fd_t *fd;
+			size_t size;
+			off_t off;
+		} readdirp;
+		struct {
+			fop_readdirp_cbk_t fn;
+			int32_t op_ret, op_errno;
+			gf_dirent_t entries;
+		} readdirp_cbk;
+
 		/* checksum */
 		struct {
 			fop_checksum_t fn;
@@ -568,6 +545,20 @@ typedef struct {
 			uint8_t *dir_checksum;
 		} checksum_cbk;
 
+		/* rchecksum */
+		struct {
+			fop_rchecksum_t fn;
+			fd_t *fd;
+                        off_t offset;
+			int32_t len;
+		} rchecksum;
+		struct {
+			fop_rchecksum_cbk_t fn;
+			int32_t op_ret, op_errno;
+			uint32_t weak_checksum;
+			uint8_t *strong_checksum;
+		} rchecksum_cbk;
+
 		/* xattrop */
 		struct {
 			fop_xattrop_t fn;
@@ -617,6 +608,36 @@ typedef struct {
 			int32_t op_ret;
 			int32_t op_errno;
 		} lock_fnotify_cbk;
+                
+                /* setattr */
+                struct {
+                        fop_setattr_t fn;
+                        loc_t loc;
+                        struct stat stbuf;
+                        int32_t valid;
+                } setattr;
+                struct {
+                        fop_setattr_cbk_t fn;
+                        int32_t op_ret;
+                        int32_t op_errno;
+                        struct stat statpre;
+                        struct stat statpost;
+                } setattr_cbk;
+
+                /* fsetattr */
+                struct {
+                        fop_fsetattr_t fn;
+                        fd_t *fd;
+                        struct stat stbuf;
+                        int32_t valid;
+                } fsetattr;
+                struct {
+                        fop_fsetattr_cbk_t fn;
+                        int32_t op_ret;
+                        int32_t op_errno;
+                        struct stat statpre;
+                        struct stat statpost;
+                } fsetattr_cbk;
 
 	} args;
 } call_stub_t;
@@ -634,7 +655,8 @@ fop_lookup_cbk_stub (call_frame_t *frame,
 		     int32_t op_errno,
 		     inode_t *inode,
 		     struct stat *buf,
-		     dict_t *dict);
+                     dict_t *dict,
+                     struct stat *postparent);
 call_stub_t *
 fop_stat_stub (call_frame_t *frame,
 	       fop_stat_t fn,
@@ -655,55 +677,6 @@ fop_fstat_cbk_stub (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
 		    struct stat *buf);
-call_stub_t *
-fop_chmod_stub (call_frame_t *frame,
-		fop_chmod_t fn,
-		loc_t *loc,
-		mode_t mode);
-call_stub_t *
-fop_chmod_cbk_stub (call_frame_t *frame,
-		    fop_chmod_cbk_t fn,
-		    int32_t op_ret,
-		    int32_t op_errno,
-		    struct stat *buf);
-call_stub_t *
-fop_fchmod_stub (call_frame_t *frame,
-		 fop_fchmod_t fn,
-		 fd_t *fd,
-		 mode_t mode);
-call_stub_t *
-fop_fchmod_cbk_stub (call_frame_t *frame,
-		     fop_fchmod_cbk_t fn,
-		     int32_t op_ret,
-		     int32_t op_errno,
-		     struct stat *buf);
-call_stub_t *
-fop_chown_stub (call_frame_t *frame,
-		fop_chown_t fn,
-		loc_t *loc,
-		uid_t uid,
-		gid_t gid);
-
-call_stub_t *
-fop_chown_cbk_stub (call_frame_t *frame,
-		    fop_chown_cbk_t fn,
-		    int32_t op_ret,
-		    int32_t op_errno,
-		    struct stat *buf);
-
-call_stub_t *
-fop_fchown_stub (call_frame_t *frame,
-		 fop_fchown_t fn,
-		 fd_t *fd,
-		 uid_t uid,
-		 gid_t gid);
-
-call_stub_t *
-fop_fchown_cbk_stub (call_frame_t *frame,
-		     fop_fchown_cbk_t fn,
-		     int32_t op_ret,
-		     int32_t op_errno,
-		     struct stat *buf);
 
 call_stub_t *
 fop_truncate_stub (call_frame_t *frame,
@@ -716,7 +689,8 @@ fop_truncate_cbk_stub (call_frame_t *frame,
 		       fop_truncate_cbk_t fn,
 		       int32_t op_ret,
 		       int32_t op_errno,
-		       struct stat *buf);
+		       struct stat *prebuf,
+                       struct stat *postbuf);
 
 call_stub_t *
 fop_ftruncate_stub (call_frame_t *frame,
@@ -729,20 +703,8 @@ fop_ftruncate_cbk_stub (call_frame_t *frame,
 			fop_ftruncate_cbk_t fn,
 			int32_t op_ret,
 			int32_t op_errno,
-			struct stat *buf);
-
-call_stub_t *
-fop_utimens_stub (call_frame_t *frame,
-		  fop_utimens_t fn,
-		  loc_t *loc,
-		  struct timespec tv[2]);
-
-call_stub_t *
-fop_utimens_cbk_stub (call_frame_t *frame,
-		      fop_utimens_cbk_t fn,
-		      int32_t op_ret,
-		      int32_t op_errno,
-		      struct stat *buf);
+			struct stat *prebuf,
+                        struct stat *postbuf);
 
 call_stub_t *
 fop_access_stub (call_frame_t *frame,
@@ -767,7 +729,8 @@ fop_readlink_cbk_stub (call_frame_t *frame,
 		       fop_readlink_cbk_t fn,
 		       int32_t op_ret,
 		       int32_t op_errno,
-		       const char *path);
+		       const char *path,
+                       struct stat *buf);
 
 call_stub_t *
 fop_mknod_stub (call_frame_t *frame,
@@ -782,7 +745,9 @@ fop_mknod_cbk_stub (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
 		    inode_t *inode,
-		    struct stat *buf);
+                    struct stat *buf,
+                    struct stat *preparent,
+                    struct stat *postparent);
 
 call_stub_t *
 fop_mkdir_stub (call_frame_t *frame,
@@ -796,7 +761,9 @@ fop_mkdir_cbk_stub (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
 		    inode_t *inode,
-		    struct stat *buf);
+                    struct stat *buf,
+                    struct stat *preparent,
+                    struct stat *postparent);
 
 call_stub_t *
 fop_unlink_stub (call_frame_t *frame,
@@ -807,7 +774,9 @@ call_stub_t *
 fop_unlink_cbk_stub (call_frame_t *frame,
 		     fop_unlink_cbk_t fn,
 		     int32_t op_ret,
-		     int32_t op_errno);
+		     int32_t op_errno,
+                     struct stat *preparent,
+                     struct stat *postparent);
 
 call_stub_t *
 fop_rmdir_stub (call_frame_t *frame,
@@ -818,7 +787,9 @@ call_stub_t *
 fop_rmdir_cbk_stub (call_frame_t *frame,
 		    fop_rmdir_cbk_t fn,
 		    int32_t op_ret,
-		    int32_t op_errno);
+		    int32_t op_errno,
+                    struct stat *preparent,
+                    struct stat *postparent);
 
 call_stub_t *
 fop_symlink_stub (call_frame_t *frame,
@@ -832,7 +803,9 @@ fop_symlink_cbk_stub (call_frame_t *frame,
 		      int32_t op_ret,
 		      int32_t op_errno,
 		      inode_t *inode,
-		      struct stat *buf);
+                      struct stat *buf,
+                      struct stat *preparent,
+                      struct stat *postparent);
 
 call_stub_t *
 fop_rename_stub (call_frame_t *frame,
@@ -845,7 +818,11 @@ fop_rename_cbk_stub (call_frame_t *frame,
 		     fop_rename_cbk_t fn,
 		     int32_t op_ret,
 		     int32_t op_errno,
-		     struct stat *buf);
+		     struct stat *buf,
+                     struct stat *preoldparent,
+                     struct stat *postoldparent,
+                     struct stat *prenewparent,
+                     struct stat *postnewparent);
 
 call_stub_t *
 fop_link_stub (call_frame_t *frame,
@@ -859,7 +836,9 @@ fop_link_cbk_stub (call_frame_t *frame,
 		   int32_t op_ret,
 		   int32_t op_errno,
 		   inode_t *inode,
-		   struct stat *buf);
+                   struct stat *buf,
+                   struct stat *preparent,
+                   struct stat *postparent);
 
 call_stub_t *
 fop_create_stub (call_frame_t *frame,
@@ -875,14 +854,17 @@ fop_create_cbk_stub (call_frame_t *frame,
 		     int32_t op_errno,
 		     fd_t *fd,
 		     inode_t *inode,
-		     struct stat *buf);
+		     struct stat *buf,
+                     struct stat *preparent,
+                     struct stat *postparent);
 
 call_stub_t *
 fop_open_stub (call_frame_t *frame,
 	       fop_open_t fn,
 	       loc_t *loc,
 	       int32_t flags,
-	       fd_t *fd);
+	       fd_t *fd,
+               int32_t wbflags);
 
 call_stub_t *
 fop_open_cbk_stub (call_frame_t *frame,
@@ -922,7 +904,8 @@ fop_writev_cbk_stub (call_frame_t *frame,
 		     fop_writev_cbk_t fn,
 		     int32_t op_ret,
 		     int32_t op_errno,
-		     struct stat *stbuf);
+                     struct stat *prebuf,
+                     struct stat *postbuf);
 
 call_stub_t *
 fop_flush_stub (call_frame_t *frame,
@@ -945,7 +928,9 @@ call_stub_t *
 fop_fsync_cbk_stub (call_frame_t *frame,
 		    fop_fsync_cbk_t fn,
 		    int32_t op_ret,
-		    int32_t op_errno);
+		    int32_t op_errno,
+                    struct stat *prebuf,
+                    struct stat *postbuf);
 
 call_stub_t *
 fop_opendir_stub (call_frame_t *frame,
@@ -1133,6 +1118,20 @@ fop_readdir_stub (call_frame_t *frame,
 		  size_t size,
 		  off_t off);
 
+call_stub_t *
+fop_readdirp_stub (call_frame_t *frame,
+		   fop_readdir_t fn,
+		   fd_t *fd,
+		   size_t size,
+		   off_t off);
+
+call_stub_t *
+fop_readdirp_cbk_stub (call_frame_t *frame,
+		       fop_readdir_cbk_t fn,
+		       int32_t op_ret,
+		       int32_t op_errno,
+		       gf_dirent_t *entries);
+
 call_stub_t *
 fop_readdir_cbk_stub (call_frame_t *frame,
 		      fop_readdir_cbk_t fn,
@@ -1154,6 +1153,20 @@ fop_checksum_cbk_stub (call_frame_t *frame,
 		       uint8_t *file_checksum,
 		       uint8_t *dir_checksum);
 
+call_stub_t *
+fop_rchecksum_stub (call_frame_t *frame,
+                    fop_rchecksum_t fn,
+                    fd_t *fd, off_t offset,
+                    int32_t len);
+
+call_stub_t *
+fop_rchecksum_cbk_stub (call_frame_t *frame,
+                        fop_rchecksum_cbk_t fn,
+                        int32_t op_ret,
+                        int32_t op_errno,
+                        uint32_t weak_checksum,
+                        uint8_t *strong_checksum);
+
 call_stub_t *
 fop_xattrop_stub (call_frame_t *frame,
 		  fop_xattrop_t fn,
@@ -1204,6 +1217,36 @@ fop_lock_fnotify_stub (call_frame_t *frame,
 		       fd_t *fd,
 		       int32_t timeout);
 
+call_stub_t *
+fop_setattr_stub (call_frame_t *frame,
+                  fop_setattr_t fn,
+                  loc_t *loc,
+                  struct stat *stbuf,
+                  int32_t valid);
+
+call_stub_t *
+fop_setattr_cbk_stub (call_frame_t *frame,
+                      fop_setattr_cbk_t fn,
+                      int32_t op_ret,
+                      int32_t op_errno,
+                      struct stat *statpre,
+                      struct stat *statpost);
+
+call_stub_t *
+fop_fsetattr_stub (call_frame_t *frame,
+                   fop_fsetattr_t fn,
+                   fd_t *fd,
+                   struct stat *stbuf,
+                   int32_t valid);
+
+call_stub_t *
+fop_fsetattr_cbk_stub (call_frame_t *frame,
+                       fop_setattr_cbk_t fn,
+                       int32_t op_ret,
+                       int32_t op_errno,
+                       struct stat *statpre,
+                       struct stat *statpost);
+
 void call_resume (call_stub_t *stub);
 void call_stub_destroy (call_stub_t *stub);
 #endif
