@@ -56,6 +56,21 @@ struct dnscache6 {
 	struct addrinfo *next;
 };
 
+
+int
+gf_log2 (unsigned long x)
+{
+        int val = 0;
+
+        while (x != 1) {
+                x /= 2;
+                val++;
+        }
+
+        return val;
+}
+
+
 int32_t
 gf_resolve_ip6 (const char *hostname, 
 		uint16_t port, 
@@ -96,7 +111,11 @@ gf_resolve_ip6 (const char *hostname,
 		hints.ai_socktype = SOCK_STREAM;
 		hints.ai_flags    = AI_ADDRCONFIG;
 
-		asprintf (&port_str, "%d", port);
+		ret = asprintf (&port_str, "%d", port);
+                if (-1 == ret) {
+                        gf_log ("resolver", GF_LOG_ERROR, "asprintf failed");
+                        return -1;
+                }
 		if ((ret = getaddrinfo(hostname, port_str, &hints, &cache->first)) != 0) {
 			gf_log ("resolver", GF_LOG_ERROR,
 				"getaddrinfo failed (%s)", gai_strerror (ret));
@@ -175,49 +194,49 @@ gf_global_variable_init()
 	gf_fop_list[GF_FOP_SYMLINK]     = "SYMLINK";
 	gf_fop_list[GF_FOP_RENAME]      = "RENAME";
 	gf_fop_list[GF_FOP_LINK]        = "LINK";
-	gf_fop_list[GF_FOP_CHMOD]       = "CHMOD";
-	gf_fop_list[GF_FOP_CHOWN]       = "CHOWN";      /* 10 */
 	gf_fop_list[GF_FOP_TRUNCATE]    = "TRUNCATE";
-	gf_fop_list[GF_FOP_OPEN]        = "OPEN";
+	gf_fop_list[GF_FOP_OPEN]        = "OPEN";       /* 10 */
 	gf_fop_list[GF_FOP_READ]        = "READ";
 	gf_fop_list[GF_FOP_WRITE]       = "WRITE";
-	gf_fop_list[GF_FOP_STATFS]      = "STATFS";     /* 15 */
+	gf_fop_list[GF_FOP_STATFS]      = "STATFS";
 	gf_fop_list[GF_FOP_FLUSH]       = "FLUSH";
-	gf_fop_list[GF_FOP_FSYNC]       = "FSYNC";
+	gf_fop_list[GF_FOP_FSYNC]       = "FSYNC";      /* 15 */
 	gf_fop_list[GF_FOP_SETXATTR]    = "SETXATTR";
-	gf_fop_list[GF_FOP_GETXATTR]    = "GETXATTR";   /* 20 */
+	gf_fop_list[GF_FOP_GETXATTR]    = "GETXATTR";
 	gf_fop_list[GF_FOP_REMOVEXATTR] = "REMOVEXATTR";
 	gf_fop_list[GF_FOP_OPENDIR]     = "OPENDIR";
-	gf_fop_list[GF_FOP_GETDENTS]    = "GETDENTS";
-	gf_fop_list[GF_FOP_FSYNCDIR]    = "FSYNCDIR";   /* 25 */
+	gf_fop_list[GF_FOP_GETDENTS]    = "GETDENTS";   /* 20 */
+	gf_fop_list[GF_FOP_FSYNCDIR]    = "FSYNCDIR";
 	gf_fop_list[GF_FOP_ACCESS]      = "ACCESS";
 	gf_fop_list[GF_FOP_CREATE]      = "CREATE";
 	gf_fop_list[GF_FOP_FTRUNCATE]   = "FTRUNCATE";
-	gf_fop_list[GF_FOP_FSTAT]       = "FSTAT";
-	gf_fop_list[GF_FOP_LK]          = "LK";         /* 30 */
-	gf_fop_list[GF_FOP_UTIMENS]     = "UTIMENS";
-	gf_fop_list[GF_FOP_FCHMOD]      = "FCHMOD";
-	gf_fop_list[GF_FOP_FCHOWN]      = "FCHOWN";
+	gf_fop_list[GF_FOP_FSTAT]       = "FSTAT";      /* 25 */
+	gf_fop_list[GF_FOP_LK]          = "LK";
 	gf_fop_list[GF_FOP_LOOKUP]      = "LOOKUP";
-	gf_fop_list[GF_FOP_SETDENTS]    = "SETDENTS";   /* 35 */
+	gf_fop_list[GF_FOP_SETDENTS]    = "SETDENTS";
 	gf_fop_list[GF_FOP_READDIR]     = "READDIR";
-	gf_fop_list[GF_FOP_INODELK]     = "INODELK";
+	gf_fop_list[GF_FOP_INODELK]     = "INODELK";    /* 30 */
 	gf_fop_list[GF_FOP_FINODELK]    = "FINODELK";
 	gf_fop_list[GF_FOP_ENTRYLK]     = "ENTRYLK";
-	gf_fop_list[GF_FOP_FENTRYLK]    = "FENTRYLK";   /* 40 */
-	gf_fop_list[GF_FOP_CHECKSUM]    = "CHECKSUM";   /* 41 */
+	gf_fop_list[GF_FOP_FENTRYLK]    = "FENTRYLK";   /* 35 */
+	gf_fop_list[GF_FOP_CHECKSUM]    = "CHECKSUM";   /* 36 */
 	gf_fop_list[GF_FOP_XATTROP]     = "XATTROP";
 	gf_fop_list[GF_FOP_FXATTROP]    = "FXATTROP";
 	gf_fop_list[GF_FOP_LOCK_NOTIFY] = "LOCK_NOTIFY";
 	gf_fop_list[GF_FOP_LOCK_FNOTIFY]= "LOCK_FNOTIFY";
 	gf_fop_list[GF_FOP_FSETXATTR]   = "FSETXATTR";
 	gf_fop_list[GF_FOP_FGETXATTR]   = "FGETXATTR";
+        gf_fop_list[GF_FOP_RCHECKSUM]   = "RCHECKSUM";
+        gf_fop_list[GF_FOP_SETATTR]     = "SETATTR";
+        gf_fop_list[GF_FOP_FSETATTR]    = "FSETATTR";
+	gf_fop_list[GF_FOP_READDIRP]    = "READDIRP";
 
 	gf_mop_list[GF_MOP_SETVOLUME]   = "SETVOLUME"; /* 0 */
 	gf_mop_list[GF_MOP_GETVOLUME]   = "GETVOLUME"; /* 1 */
 	gf_mop_list[GF_MOP_STATS]       = "STATS";
 	gf_mop_list[GF_MOP_SETSPEC]     = "SETSPEC";
 	gf_mop_list[GF_MOP_GETSPEC]     = "GETSPEC";
+        gf_mop_list[GF_MOP_LOG]         = "LOG";
 	gf_mop_list[GF_MOP_PING]        = "PING";
 
 	gf_cbk_list[GF_CBK_FORGET]      = "FORGET";
@@ -275,82 +294,84 @@ gf_log_volume_file (FILE *specfp)
 static void 
 gf_dump_config_flags (int fd)
 {
+        int ret = 0;
+        /* TODO: 'ret' is not checked properly, add this later */
 
-	write (fd, "configuration details:\n", 23);
+	ret = write (fd, "configuration details:\n", 23);
 
 /* have argp */
 #ifdef HAVE_ARGP
-	write (fd, "argp 1\n", 7);
+	ret = write (fd, "argp 1\n", 7);
 #endif
 
 /* ifdef if found backtrace */
 #ifdef HAVE_BACKTRACE 
-	write (fd, "backtrace 1\n", 12);
+	ret = write (fd, "backtrace 1\n", 12);
 #endif
 
 /* Berkeley-DB version has cursor->get() */
 #ifdef HAVE_BDB_CURSOR_GET 
-	write (fd, "bdb->cursor->get 1\n", 19);
+	ret = write (fd, "bdb->cursor->get 1\n", 19);
 #endif
 
 /* Define to 1 if you have the <db.h> header file. */
 #ifdef HAVE_DB_H 
-	write (fd, "db.h 1\n", 7);
+	ret = write (fd, "db.h 1\n", 7);
 #endif
 
 /* Define to 1 if you have the <dlfcn.h> header file. */
 #ifdef HAVE_DLFCN_H 
-	write (fd, "dlfcn 1\n", 8);
+	ret = write (fd, "dlfcn 1\n", 8);
 #endif
 
 /* define if fdatasync exists */
 #ifdef HAVE_FDATASYNC 
-	write (fd, "fdatasync 1\n", 12);
+	ret = write (fd, "fdatasync 1\n", 12);
 #endif
 
 /* Define to 1 if you have the `pthread' library (-lpthread). */
 #ifdef HAVE_LIBPTHREAD 
-	write (fd, "libpthread 1\n", 13);
+	ret = write (fd, "libpthread 1\n", 13);
 #endif
 
 /* define if llistxattr exists */
 #ifdef HAVE_LLISTXATTR 
-	write (fd, "llistxattr 1\n", 13);
+	ret = write (fd, "llistxattr 1\n", 13);
 #endif
 
 /* define if found setfsuid setfsgid */
 #ifdef HAVE_SET_FSID 
-	write (fd, "setfsid 1\n", 10);
+	ret = write (fd, "setfsid 1\n", 10);
 #endif
 
 /* define if found spinlock */
 #ifdef HAVE_SPINLOCK 
-	write (fd, "spinlock 1\n", 11);
+	ret = write (fd, "spinlock 1\n", 11);
 #endif
 
 /* Define to 1 if you have the <sys/epoll.h> header file. */
 #ifdef HAVE_SYS_EPOLL_H 
-	write (fd, "epoll.h 1\n", 10);
+	ret = write (fd, "epoll.h 1\n", 10);
 #endif
 
 /* Define to 1 if you have the <sys/extattr.h> header file. */
 #ifdef HAVE_SYS_EXTATTR_H 
-	write (fd, "extattr.h 1\n", 12);
+	ret = write (fd, "extattr.h 1\n", 12);
 #endif
 
 /* Define to 1 if you have the <sys/xattr.h> header file. */
 #ifdef HAVE_SYS_XATTR_H 
-	write (fd, "xattr.h 1\n", 10);
+	ret = write (fd, "xattr.h 1\n", 10);
 #endif
 
 /* define if found st_atim.tv_nsec */
 #ifdef HAVE_STRUCT_STAT_ST_ATIM_TV_NSEC
-	write (fd, "st_atim.tv_nsec 1\n", 18);
+	ret = write (fd, "st_atim.tv_nsec 1\n", 18);
 #endif
 
 /* define if found st_atimespec.tv_nsec */
 #ifdef HAVE_STRUCT_STAT_ST_ATIMESPEC_TV_NSEC
-	write (fd, "st_atimespec.tv_nsec 1\n",23);
+	ret = write (fd, "st_atimespec.tv_nsec 1\n",23);
 #endif
 
 /* Define to the full name and version of this package. */
@@ -358,7 +379,7 @@ gf_dump_config_flags (int fd)
 	{
 		char msg[128];
 		sprintf (msg, "package-string: %s\n", PACKAGE_STRING); 
-		write (fd, msg, strlen (msg));
+		ret = write (fd, msg, strlen (msg));
 	}
 #endif
 
@@ -371,18 +392,18 @@ gf_dump_config_flags (int fd)
 void
 gf_print_trace (int32_t signum)
 {
-	extern FILE *gf_log_logfile; /* don't reset/initialize it, its 
-                                        an extern */
-	int          fd = 0;
-	char         msg[1024] = {0,};
-	time_t       utime = 0;
-	struct tm   *tm = NULL;
-	char         timestr[256] = {0,};
+	extern FILE *gf_log_logfile;
+        struct tm   *tm = NULL;
+        char         msg[1024] = {0,};
+        char         timestr[256] = {0,};
+        time_t       utime = 0;
+        int          ret = 0;
+        int          fd = 0;
 
         fd = fileno (gf_log_logfile);
 
 	/* Pending frames, (if any), list them in order */
-	write (fd, "pending frames:\n", 16);
+	ret = write (fd, "pending frames:\n", 16);
 	{
 		extern glusterfs_ctx_t *gf_global_ctx;
 		glusterfs_ctx_t *ctx = gf_global_ctx;
@@ -405,17 +426,17 @@ gf_print_trace (int32_t signum)
 					 tmp->root->type, 
 					 gf_cbk_list[tmp->root->op]);
 			
-			write (fd, msg, strlen (msg));
+			ret = write (fd, msg, strlen (msg));
 			trav = trav->next;
 		}
-		write (fd, "\n", 1);
+		ret = write (fd, "\n", 1);
 	}
 
 	sprintf (msg, "patchset: %s\n", GLUSTERFS_REPOSITORY_REVISION); 
-	write (fd, msg, strlen (msg));
+	ret = write (fd, msg, strlen (msg));
 
 	sprintf (msg, "signal received: %d\n", signum); 
-	write (fd, msg, strlen (msg));
+	ret = write (fd, msg, strlen (msg));
 
         {
                 /* Dump the timestamp of the crash too, so the previous logs 
@@ -423,8 +444,8 @@ gf_print_trace (int32_t signum)
                 utime = time (NULL);
                 tm    = localtime (&utime);
                 strftime (timestr, 256, "%Y-%m-%d %H:%M:%S\n", tm); 
-                write (fd, "time of crash: ", 15);
-                write (fd, timestr, strlen (timestr));
+                ret = write (fd, "time of crash: ", 15);
+                ret = write (fd, timestr, strlen (timestr));
         }
 
 	gf_dump_config_flags (fd);
@@ -437,7 +458,7 @@ gf_print_trace (int32_t signum)
 		size = backtrace (array, 200);
 		backtrace_symbols_fd (&array[1], size-1, fd);
 		sprintf (msg, "---------\n");
-		write (fd, msg, strlen (msg));
+		ret = write (fd, msg, strlen (msg));
 	}
 #endif /* HAVE_BACKTRACE */
   
