@@ -21,6 +21,7 @@
 #include "glusterfs.h"
 #include "inode.h"
 #include "dict.h"
+#include "statedump.h"
 
 
 #ifndef _CONFIG_H
@@ -382,9 +383,9 @@ _fd_unref (fd_t *fd)
 static void
 fd_destroy (fd_t *fd)
 {
-        data_pair_t *pair = NULL;
         xlator_t    *xl = NULL;
 	int i = 0;
+        xlator_t    *old_THIS = NULL;
 
         if (fd == NULL){
                 gf_log ("xlator", GF_LOG_ERROR, "invalid arugument");
@@ -399,58 +400,25 @@ fd_destroy (fd_t *fd)
 		goto out;
 
         if (S_ISDIR (fd->inode->st_mode)) {
-                for (pair = fd->ctx->members_list; pair; pair = pair->next) {
-                        /* notify all xlators which have a context */
-                        xl = xlator_search_by_name (fd->inode->table->xl, 
-						    pair->key);
-          
-                        if (!xl) {
-                                gf_log ("fd", GF_LOG_CRITICAL,
-                                        "fd(%p)->ctx has invalid key(%s)",
-                                        fd, pair->key);
-                                continue;
-                        }
-                        if (xl->cbks->releasedir) {
-                                xl->cbks->releasedir (xl, fd);
-                        } else {
-                                gf_log ("fd", GF_LOG_CRITICAL,
-                                        "xlator(%s) in fd(%p) no RELEASE cbk",
-                                        xl->name, fd);
-                        }
-
-                }
 		for (i = 0; i < fd->inode->table->xl->ctx->xl_count; i++) {
 			if (fd->_ctx[i].key) {
 				xl = (xlator_t *)(long)fd->_ctx[i].key;
+                                old_THIS = THIS;
+                                THIS = xl;
 				if (xl->cbks->releasedir)
 					xl->cbks->releasedir (xl, fd);
+                                THIS = old_THIS;
 			}
 		}
         } else {
-                for (pair = fd->ctx->members_list; pair; pair = pair->next) {
-                        /* notify all xlators which have a context */
-                        xl = xlator_search_by_name (fd->inode->table->xl, 
-						    pair->key);
-          
-                        if (!xl) {
-                                gf_log ("fd", GF_LOG_CRITICAL,
-                                        "fd(%p)->ctx has invalid key(%s)",
-                                        fd, pair->key);
-                                continue;
-                        }
-                        if (xl->cbks->release) {
-                                xl->cbks->release (xl, fd);
-                        } else {
-                                gf_log ("fd", GF_LOG_CRITICAL,
-                                        "xlator(%s) in fd(%p) no RELEASE cbk",
-                                        xl->name, fd);
-                        }
-                }
 		for (i = 0; i < fd->inode->table->xl->ctx->xl_count; i++) {
 			if (fd->_ctx[i].key) {
 				xl = (xlator_t *)(long)fd->_ctx[i].key;
+                                old_THIS = THIS;
+                                THIS = xl;
 				if (xl->cbks->release)
 					xl->cbks->release (xl, fd);
+                                THIS = old_THIS;
 			}
 		}
         }
@@ -460,7 +428,6 @@ fd_destroy (fd_t *fd)
 	FREE (fd->_ctx);
         inode_unref (fd->inode);
         fd->inode = (inode_t *)0xaaaaaaaa;
-        dict_destroy (fd->ctx);
         FREE (fd);
         
 out:
@@ -525,7 +492,6 @@ fd_create (inode_t *inode, pid_t pid)
   
 	fd->_ctx = CALLOC (1, (sizeof (struct _fd_ctx) * 
 			       inode->table->xl->ctx->xl_count));
-        fd->ctx = get_new_dict ();
         fd->inode = inode_ref (inode);
         fd->pid = pid;
         INIT_LIST_HEAD (&fd->inode_list);
@@ -727,3 +693,77 @@ fd_ctx_del (fd_t *fd, xlator_t *xlator, uint64_t *value)
 
         return ret;
 }
+
+
+void
+fd_dump (fd_t *fd, char *prefix)
+{
+        char        key[GF_DUMP_MAX_BUF_LEN];
+
+        if (!fd)
+                return;
+    
+        memset(key, 0, sizeof(key));
+        gf_proc_dump_build_key(key, prefix, "pid");
+        gf_proc_dump_write(key, "%d", fd->pid);
+        gf_proc_dump_build_key(key, prefix, "refcount");
+        gf_proc_dump_write(key, "%d", fd->refcount);
+        gf_proc_dump_build_key(key, prefix, "flags");
+        gf_proc_dump_write(key, "%d", fd->flags);
+        if (fd->inode) {
+                gf_proc_dump_build_key(key, prefix, "inode");
+                gf_proc_dump_write(key, "%ld", fd->inode->ino);
+        }
+}
+
+
+void
+fdentry_dump (fdentry_t *fdentry, char *prefix)
+{
+        if (!fdentry)
+                return;
+
+        if (GF_FDENTRY_ALLOCATED != fdentry->next_free)
+                return;
+
+        if (fdentry->fd) 
+                fd_dump(fdentry->fd, prefix);
+}
+
+void
+fdtable_dump (fdtable_t *fdtable, char *prefix)
+{
+        char    key[GF_DUMP_MAX_BUF_LEN];
+        int     i = 0;
+        int     ret = -1;
+
+        if (!fdtable)
+                return;
+    
+	ret = pthread_mutex_trylock (&fdtable->lock);
+
+        if (ret) {
+		gf_log ("fd", GF_LOG_WARNING, "Unable to acquire lock");
+                return;
+        }
+
+        memset(key, 0, sizeof(key));
+        gf_proc_dump_build_key(key, prefix, "refcount");
+        gf_proc_dump_write(key, "%d", fdtable->refcount);
+        gf_proc_dump_build_key(key, prefix, "maxfds");
+        gf_proc_dump_write(key, "%d", fdtable->max_fds);
+        gf_proc_dump_build_key(key, prefix, "first_free");
+        gf_proc_dump_write(key, "%d", fdtable->first_free);
+
+        for ( i = 0 ; i < fdtable->max_fds; i++) {
+                if (GF_FDENTRY_ALLOCATED == 
+                                fdtable->fdentries[i].next_free) {
+                        gf_proc_dump_build_key(key, prefix, "fdentry[%d]", i);
+                        gf_proc_dump_add_section(key);
+                        fdentry_dump(&fdtable->fdentries[i], key);
+                }
+        }
+
+        pthread_mutex_unlock(&fdtable->lock);
+}
+
