@@ -71,18 +71,12 @@
 #define GLUSTERFS_ENTRYLK_COUNT "glusterfs.entrylk-count"
 #define GLUSTERFS_POSIXLK_COUNT "glusterfs.posixlk-count"
 
-
 #define ZR_FILE_CONTENT_REQUEST(key) (!strncmp(key, ZR_FILE_CONTENT_STR, \
 					       ZR_FILE_CONTENT_STRLEN))
 
 /* TODO: Should we use PATH-MAX? On some systems it may save space */
 #define ZR_PATH_MAX 4096    
 
-/* This is used as the maximum permitted filename length over FS. 
- * If the backend FS supports higher than this, it should be changed. 
- */
-#define ZR_FILENAME_MAX 256 
-
 
 /* NOTE: add members ONLY at the end (just before _MAXVALUE) */
 typedef enum {
@@ -95,10 +89,8 @@ typedef enum {
         GF_FOP_SYMLINK,
         GF_FOP_RENAME,
         GF_FOP_LINK,
-        GF_FOP_CHMOD,
-        GF_FOP_CHOWN,      /* 10 */
         GF_FOP_TRUNCATE,
-        GF_FOP_OPEN,
+        GF_FOP_OPEN,       /* 10 */
         GF_FOP_READ,
         GF_FOP_WRITE,
         GF_FOP_STATFS,     /* 15 */
@@ -115,9 +107,6 @@ typedef enum {
         GF_FOP_FTRUNCATE,
         GF_FOP_FSTAT,
         GF_FOP_LK,
-        GF_FOP_UTIMENS,
-        GF_FOP_FCHMOD,     /* 30 */
-        GF_FOP_FCHOWN,
         GF_FOP_LOOKUP,
         GF_FOP_SETDENTS,
         GF_FOP_READDIR,
@@ -131,7 +120,11 @@ typedef enum {
         GF_FOP_LOCK_NOTIFY,
         GF_FOP_LOCK_FNOTIFY,
         GF_FOP_FGETXATTR,
-        GF_FOP_FSETXATTR,
+        GF_FOP_FSETXATTR, /* 45 */
+        GF_FOP_RCHECKSUM,
+        GF_FOP_SETATTR,
+        GF_FOP_FSETATTR,
+        GF_FOP_READDIRP,
         GF_FOP_MAXVALUE,
 } glusterfs_fop_t;
 
@@ -142,8 +135,10 @@ typedef enum {
         GF_MOP_STATS,
         GF_MOP_SETSPEC,
         GF_MOP_GETSPEC,
-	GF_MOP_PING,
-        GF_MOP_MAXVALUE   /* 5 */
+	GF_MOP_PING,      /* 5 */
+        GF_MOP_LOG,
+        GF_MOP_NOTIFY,
+        GF_MOP_MAXVALUE   /* 8 */
 } glusterfs_mop_t;
 
 typedef enum {
@@ -207,6 +202,11 @@ typedef enum {
 #define GF_SET_DIR_ONLY       0x4
 #define GF_SET_EPOCH_TIME     0x8 /* used by afr dir lookup selfheal */
 
+/* Directory into which replicate self-heal will move deleted files and
+   directories into. The storage/posix janitor thread will periodically
+   clean up this directory */
+
+#define GF_REPLICATE_TRASH_DIR          ".landfill"
 
 struct _xlator_cmdline_option {
 	struct list_head cmd_args;
@@ -220,12 +220,14 @@ struct _cmd_args {
 	/* basic options */
 	char            *volfile_server;
 	char            *volume_file;
+        char            *log_server;
 	gf_loglevel_t    log_level;
 	char            *log_file;
         int32_t          max_connect_attempts;
 	/* advanced options */
 	uint32_t         volfile_server_port;
 	char            *volfile_server_transport;
+        uint32_t         log_server_port;
 	char            *pid_file;
 	int              no_daemon_mode;
 	char            *run_id;
