@@ -38,14 +38,14 @@
 /* Any changes in the protocol structure or adding new '[f,m]ops' needs to 
  * bump the protocol version by "0.1" 
  */
-/* Protocol version 1.0 was ASCII based dictionary protocol */
-#define GF_PROTOCOL_VERSION "2.0"
+
+#define GF_PROTOCOL_VERSION "3.0"
 
 struct gf_stat {
 	uint64_t ino;
 	uint64_t size;
 	uint64_t blocks;
-	uint32_t dev;
+	uint64_t dev;
 	uint32_t rdev;
 	uint32_t mode;
 	uint32_t nlink;
@@ -54,7 +54,7 @@ struct gf_stat {
 	uint32_t blksize;
 	uint32_t atime;
 	uint32_t atime_nsec;
-	uint32_t mtime;
+	uint32_t mtime ;
 	uint32_t mtime_nsec;
 	uint32_t ctime;
 	uint32_t ctime_nsec;
@@ -64,7 +64,7 @@ struct gf_stat {
 static inline void
 gf_stat_to_stat (struct gf_stat *gf_stat, struct stat *stat)
 {
-	stat->st_dev          = ntoh32 (gf_stat->dev);
+	stat->st_dev          = ntoh64 (gf_stat->dev);
 	stat->st_ino          = ntoh64 (gf_stat->ino);
 	stat->st_mode         = ntoh32 (gf_stat->mode);
 	stat->st_nlink        = ntoh32 (gf_stat->nlink);
@@ -84,7 +84,7 @@ gf_stat_to_stat (struct gf_stat *gf_stat, struct stat *stat)
 static inline void
 gf_stat_from_stat (struct gf_stat *gf_stat, struct stat *stat)
 {
-	gf_stat->dev         = hton32 (stat->st_dev);
+	gf_stat->dev         = hton64 (stat->st_dev);
 	gf_stat->ino         = hton64 (stat->st_ino);
 	gf_stat->mode        = hton32 (stat->st_mode);
 	gf_stat->nlink       = hton32 (stat->st_nlink);
@@ -219,10 +219,13 @@ gf_timespec_from_timespec (struct gf_timespec *gf_ts, struct timespec *ts)
 #define GF_O_APPEND          02000
 #define GF_O_NONBLOCK        04000
 #define GF_O_SYNC           010000
+#define GF_O_ASYNC          020000
 
 #define GF_O_DIRECT         040000
 #define GF_O_DIRECTORY     0200000
 #define GF_O_NOFOLLOW      0400000
+#define GF_O_NOATIME      01000000
+#define GF_O_CLOEXEC      02000000
 
 #define GF_O_LARGEFILE     0100000
 
@@ -272,11 +275,17 @@ gf_flags_from_flags (uint32_t flags)
         XLATE_BIT (flags, gf_flags, O_APPEND);
         XLATE_BIT (flags, gf_flags, O_NONBLOCK);
         XLATE_BIT (flags, gf_flags, O_SYNC);
+        XLATE_BIT (flags, gf_flags, O_ASYNC);
 
         XLATE_BIT (flags, gf_flags, O_DIRECT);
         XLATE_BIT (flags, gf_flags, O_DIRECTORY);
         XLATE_BIT (flags, gf_flags, O_NOFOLLOW);
-
+#ifdef O_NOATIME
+        XLATE_BIT (flags, gf_flags, O_NOATIME);
+#endif
+#ifdef O_CLOEXEC
+        XLATE_BIT (flags, gf_flags, O_CLOEXEC);
+#endif
         XLATE_BIT (flags, gf_flags, O_LARGEFILE);
 
         return gf_flags;
@@ -296,11 +305,17 @@ gf_flags_to_flags (uint32_t gf_flags)
         UNXLATE_BIT (gf_flags, flags, O_APPEND);
         UNXLATE_BIT (gf_flags, flags, O_NONBLOCK);
         UNXLATE_BIT (gf_flags, flags, O_SYNC);
+        UNXLATE_BIT (gf_flags, flags, O_ASYNC);
 
         UNXLATE_BIT (gf_flags, flags, O_DIRECT);
         UNXLATE_BIT (gf_flags, flags, O_DIRECTORY);
         UNXLATE_BIT (gf_flags, flags, O_NOFOLLOW);
-
+#ifdef O_NOATIME
+        UNXLATE_BIT (gf_flags, flags, O_NOATIME);
+#endif
+#ifdef O_CLOEXEC
+        UNXLATE_BIT (gf_flags, flags, O_CLOEXEC);
+#endif
         UNXLATE_BIT (gf_flags, flags, O_LARGEFILE);
 
         return flags;
@@ -309,6 +324,7 @@ gf_flags_to_flags (uint32_t gf_flags)
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	char     path[0];     /* NULL terminated */
 } __attribute__((packed)) gf_fop_stat_req_t;;
 typedef struct {
@@ -318,16 +334,19 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	uint32_t size;
 	char     path[0];     /* NULL terminated */
 } __attribute__((packed)) gf_fop_readlink_req_t;
 typedef struct {
+        struct gf_stat buf;
 	char     path[0]; /* NULL terminated */
 } __attribute__((packed)) gf_fop_readlink_rsp_t;
 
 
 typedef struct {
 	uint64_t par;
+        uint64_t gen;
 	uint64_t dev;
 	uint32_t mode;
 	char     path[0];     /* NULL terminated */
@@ -335,52 +354,68 @@ typedef struct {
 } __attribute__((packed)) gf_fop_mknod_req_t;
 typedef struct {
 	struct gf_stat stat;
+        struct gf_stat preparent;
+        struct gf_stat postparent;
 } __attribute__((packed)) gf_fop_mknod_rsp_t;
 
 
 typedef struct {
 	uint64_t par;
+        uint64_t gen;
 	uint32_t mode;
 	char     path[0];     /* NULL terminated */
 	char     bname[0]; /* NULL terminated */
 } __attribute__((packed)) gf_fop_mkdir_req_t;
 typedef struct {
 	struct gf_stat stat;
+        struct gf_stat preparent;
+        struct gf_stat postparent;
 } __attribute__((packed)) gf_fop_mkdir_rsp_t;
 
 
 typedef struct {
 	uint64_t par;
+        uint64_t gen;
 	char     path[0];     /* NULL terminated */
 	char     bname[0]; /* NULL terminated */
 } __attribute__((packed)) gf_fop_unlink_req_t;
 typedef struct {
+        struct gf_stat preparent;
+        struct gf_stat postparent;
 } __attribute__((packed)) gf_fop_unlink_rsp_t;
 
 
 typedef struct {
 	uint64_t par;
+        uint64_t gen;
 	char     path[0];
 	char     bname[0]; /* NULL terminated */
 } __attribute__((packed)) gf_fop_rmdir_req_t;
 typedef struct {
+        struct gf_stat preparent;
+        struct gf_stat postparent;
 } __attribute__((packed)) gf_fop_rmdir_rsp_t;
 
 
 typedef struct {
 	uint64_t par;
+        uint64_t gen;
 	char     path[0];
 	char     bname[0];
 	char     linkname[0];
 } __attribute__((packed)) gf_fop_symlink_req_t;
 typedef struct {
 	struct gf_stat stat;
+        struct gf_stat preparent;
+        struct gf_stat postparent;
 }__attribute__((packed)) gf_fop_symlink_rsp_t;
 
 
 typedef struct {
 	uint64_t   oldpar;
+        uint64_t   oldgen;
 	uint64_t   newpar;
+        uint64_t   newgen;
 	char       oldpath[0];
 	char       oldbname[0]; /* NULL terminated */
 	char       newpath[0];
@@ -388,55 +423,45 @@ typedef struct {
 } __attribute__((packed)) gf_fop_rename_req_t;
 typedef struct {
 	struct gf_stat stat;
+        struct gf_stat preoldparent;
+        struct gf_stat postoldparent;
+        struct gf_stat prenewparent;
+        struct gf_stat postnewparent;
 } __attribute__((packed)) gf_fop_rename_rsp_t;
 
 
 typedef struct {
 	uint64_t   oldino;
+        uint64_t   oldgen;
 	uint64_t   newpar;
+        uint64_t   newgen;
 	char       oldpath[0];
 	char       newpath[0];
 	char       newbname[0];
 }__attribute__((packed)) gf_fop_link_req_t;
 typedef struct {
 	struct gf_stat stat;
+        struct gf_stat preparent;
+        struct gf_stat postparent;
 } __attribute__((packed)) gf_fop_link_rsp_t;
 
-
-typedef struct {
-	uint64_t  ino;
-	uint32_t  mode;
-	char      path[0];
-} __attribute__((packed)) gf_fop_chmod_req_t;
-typedef struct {
-	struct gf_stat stat;
-} __attribute__((packed)) gf_fop_chmod_rsp_t;
-
-
-typedef struct {
-	uint64_t ino;
-	uint32_t uid;
-	uint32_t gid;
-	char     path[0];
-} __attribute__((packed)) gf_fop_chown_req_t;
-typedef struct {
-	struct gf_stat stat;
-} __attribute__((packed)) gf_fop_chown_rsp_t;
-
-
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	uint64_t offset;
 	char     path[0];
 } __attribute__((packed)) gf_fop_truncate_req_t;
 typedef struct {
-	struct gf_stat stat;
+	struct gf_stat prestat;
+        struct gf_stat poststat;
 } __attribute__((packed)) gf_fop_truncate_rsp_t;
 
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	uint32_t flags;
+        uint32_t wbflags;
 	char     path[0];
 } __attribute__((packed)) gf_fop_open_req_t;
 typedef struct {
@@ -446,6 +471,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	uint64_t offset;
 	uint32_t size;
@@ -458,17 +484,20 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	uint64_t offset;
 	uint32_t size;
 } __attribute__((packed)) gf_fop_write_req_t;
 typedef struct {
-	struct gf_stat stat;
+	struct gf_stat prestat;
+        struct gf_stat poststat;
 } __attribute__((packed)) gf_fop_write_rsp_t;
 
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	char     path[0];
 } __attribute__((packed)) gf_fop_statfs_req_t;
 typedef struct {
@@ -478,6 +507,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 } __attribute__((packed)) gf_fop_flush_req_t;
 typedef struct { } __attribute__((packed)) gf_fop_flush_rsp_t;
@@ -485,15 +515,19 @@ typedef struct { } __attribute__((packed)) gf_fop_flush_rsp_t;
 
 typedef struct fsync_req {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	uint32_t data;
 } __attribute__((packed)) gf_fop_fsync_req_t;
 typedef struct {
+        struct gf_stat prestat;
+        struct gf_stat poststat;
 } __attribute__((packed)) gf_fop_fsync_rsp_t;
 
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	uint32_t flags;
 	uint32_t dict_len;
 	char     dict[0];
@@ -504,6 +538,7 @@ typedef struct { } __attribute__((packed)) gf_fop_setxattr_rsp_t;
 
 typedef struct {
         uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	uint32_t flags;
 	uint32_t dict_len;
@@ -514,6 +549,7 @@ typedef struct { } __attribute__((packed)) gf_fop_fsetxattr_rsp_t;
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	uint32_t flags;
 	uint32_t dict_len;
 	char     dict[0];
@@ -528,6 +564,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	uint32_t flags;
 	uint32_t dict_len;
@@ -542,6 +579,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	uint32_t namelen;
 	char     path[0];
 	char     name[0];
@@ -554,6 +592,7 @@ typedef struct {
 
 typedef struct {
         uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
         uint32_t namelen;
 	char     name[0];
@@ -566,6 +605,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	char     path[0];
 	char     name[0];
 } __attribute__((packed)) gf_fop_removexattr_req_t;
@@ -574,6 +614,7 @@ typedef struct { } __attribute__((packed)) gf_fop_removexattr_rsp_t;
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	char     path[0];
 } __attribute__((packed)) gf_fop_opendir_req_t;
 typedef struct {
@@ -583,6 +624,7 @@ typedef struct {
 
 typedef struct fsyncdir_req {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	int32_t  data;
 } __attribute__((packed)) gf_fop_fsyncdir_req_t;
@@ -592,6 +634,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	uint64_t offset;
 	uint32_t size;
@@ -602,8 +645,22 @@ typedef struct {
 } __attribute__((packed)) gf_fop_readdir_rsp_t;
 
 
+typedef struct {
+	uint64_t ino;
+        uint64_t gen;
+	int64_t  fd;
+	uint64_t offset;
+	uint32_t size;
+} __attribute__((packed)) gf_fop_readdirp_req_t;
+typedef struct {
+	uint32_t size;
+	char     buf[0];
+} __attribute__((packed)) gf_fop_readdirp_rsp_t;
+
+
 typedef struct  {
 	uint64_t ino;
+        uint64_t gen;
 	uint32_t mask;
 	char     path[0];
 } __attribute__((packed)) gf_fop_access_req_t;
@@ -613,6 +670,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t par;
+        uint64_t gen;
 	uint32_t flags;
 	uint32_t mode;
 	char     path[0];
@@ -621,22 +679,27 @@ typedef struct {
 typedef struct {
 	struct gf_stat stat;
 	uint64_t       fd;
+        struct gf_stat preparent;
+        struct gf_stat postparent;
 } __attribute__((packed)) gf_fop_create_rsp_t;
 
 
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	uint64_t offset;
 } __attribute__((packed)) gf_fop_ftruncate_req_t;
 typedef struct {
-	struct gf_stat stat;
+	struct gf_stat prestat;
+        struct gf_stat poststat;
 } __attribute__((packed)) gf_fop_ftruncate_rsp_t;
 
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 } __attribute__((packed)) gf_fop_fstat_req_t;
 typedef struct {
@@ -646,6 +709,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t        ino;
+        uint64_t        gen;
 	int64_t         fd;
 	uint32_t        cmd;
 	uint32_t        type;
@@ -657,6 +721,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	uint32_t cmd;
 	uint32_t type;
 	struct gf_flock flock;
@@ -668,6 +733,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	uint32_t cmd;
 	uint32_t type;
@@ -679,6 +745,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t  ino;
+        uint64_t  gen;
 	uint32_t  cmd;
 	uint32_t  type;
 	uint64_t  namelen;
@@ -691,6 +758,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t  ino;
+        uint64_t  gen;
 	int64_t   fd;
 	uint32_t  cmd;
 	uint32_t  type;
@@ -701,39 +769,10 @@ typedef struct {
 typedef struct {
 } __attribute__((packed)) gf_fop_fentrylk_rsp_t;
 
-typedef struct {
-	uint64_t           ino;
-	struct gf_timespec tv[2];
-	char               path[0];
-} __attribute__((packed)) gf_fop_utimens_req_t;
-typedef struct {
-	struct gf_stat stat;
-} __attribute__((packed)) gf_fop_utimens_rsp_t;
-
-typedef struct {
-	uint64_t  ino;
-	uint64_t  fd;
-	uint32_t  mode;
-} __attribute__((packed)) gf_fop_fchmod_req_t;
-typedef struct {
-	struct gf_stat stat;
-} __attribute__((packed)) gf_fop_fchmod_rsp_t;
-
-
-typedef struct {
-	uint64_t ino;
-	int64_t  fd;
-	uint32_t uid;
-	uint32_t gid;
-} __attribute__((packed)) gf_fop_fchown_req_t;
-typedef struct {
-	struct gf_stat stat;
-} __attribute__((packed)) gf_fop_fchown_rsp_t;
-
-
 typedef struct {
 	uint64_t ino; /* NOTE: used only in case of 'root' lookup */
 	uint64_t par;
+        uint64_t gen;
 	uint32_t flags;
 	uint32_t dictlen;
 	char     path[0];
@@ -742,6 +781,7 @@ typedef struct {
 } __attribute__((packed)) gf_fop_lookup_req_t;
 typedef struct {
 	struct gf_stat stat;
+        struct gf_stat postparent;
 	uint32_t       dict_len;
 	char           dict[0];
 } __attribute__((packed)) gf_fop_lookup_rsp_t;
@@ -749,6 +789,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	uint32_t flags;
 	uint32_t count;
@@ -756,9 +797,9 @@ typedef struct {
 } __attribute__((packed)) gf_fop_setdents_req_t;
 typedef struct { } __attribute__((packed)) gf_fop_setdents_rsp_t;
 
-
 typedef struct {
 	uint64_t ino;
+        uint64_t gen;
 	int64_t  fd;
 	uint64_t offset;
 	uint32_t size;
@@ -772,6 +813,7 @@ typedef struct {
 
 typedef struct {
 	uint64_t  ino;
+        uint64_t  gen;
 	uint32_t  flag;
 	char      path[0];
 } __attribute__((packed)) gf_fop_checksum_req_t;
@@ -780,9 +822,42 @@ typedef struct {
 	unsigned char dchecksum[0];
 } __attribute__((packed)) gf_fop_checksum_rsp_t;
 
+typedef struct {
+        uint64_t       ino;
+        uint64_t       gen;
+        struct gf_stat stbuf;
+        int32_t        valid;
+        char           path[0];
+} __attribute__((packed)) gf_fop_setattr_req_t;
+typedef struct {
+        struct gf_stat statpre;
+        struct gf_stat statpost;
+} __attribute__((packed)) gf_fop_setattr_rsp_t;
+
+typedef struct {
+        int64_t        fd;
+        struct gf_stat stbuf;
+        int32_t        valid;
+} __attribute__((packed)) gf_fop_fsetattr_req_t;
+typedef struct {
+        struct gf_stat statpre;
+        struct gf_stat statpost;
+} __attribute__((packed)) gf_fop_fsetattr_rsp_t;
+
+typedef struct {
+        int64_t   fd;
+        uint64_t  offset;
+        uint32_t  len;
+} __attribute__((packed)) gf_fop_rchecksum_req_t;
+typedef struct {
+        uint32_t weak_checksum;
+        unsigned char strong_checksum[0];
+} __attribute__((packed)) gf_fop_rchecksum_rsp_t;
+
 
 typedef struct {
 	uint64_t  ino;
+        uint64_t  gen;
 	int32_t   timeout;
 } __attribute__((packed)) gf_fop_lock_notify_req_t;
 typedef struct { } __attribute__((packed)) gf_fop_lock_notify_rsp_t;
@@ -827,6 +902,14 @@ typedef struct {
 } __attribute__((packed)) gf_mop_getspec_rsp_t;
 
 
+typedef struct {
+        uint32_t msglen;
+	char     msg[0];
+} __attribute__((packed)) gf_mop_log_req_t;
+typedef struct {
+} __attribute__((packed)) gf_mop_log_rsp_t;
+
+
 typedef struct {
 	uint32_t dict_len;
 	char buf[0];
@@ -842,10 +925,19 @@ typedef struct {
 typedef struct {
 } __attribute__((packed)) gf_mop_ping_rsp_t;
 
+typedef struct {
+	uint32_t  flags;
+        char buf[0];
+} __attribute__((packed)) gf_mop_notify_req_t;
+typedef struct {
+	uint32_t  flags;
+        char buf[0];
+} __attribute__((packed)) gf_mop_notify_rsp_t;
 
 typedef struct {
 	uint64_t ino;
-	int64_t fd;
+        uint64_t gen;
+	int64_t  fd;
 } __attribute__((packed)) gf_cbk_releasedir_req_t;
 typedef struct {
 } __attribute__((packed)) gf_cbk_releasedir_rsp_t;
@@ -853,7 +945,8 @@ typedef struct {
 
 typedef struct {
 	uint64_t ino;
-	int64_t fd;
+        uint64_t gen;
+	int64_t  fd;
 } __attribute__((packed)) gf_cbk_release_req_t;
 typedef struct {
 } __attribute__((packed)) gf_cbk_release_rsp_t;
@@ -866,10 +959,26 @@ typedef struct {
 typedef struct { } __attribute__((packed)) gf_cbk_forget_rsp_t;
 
 
+/* This corresponds to the max 16 number of group IDs that are sent through an
+ * RPC request. Since NFS is the only one going to set this, we can be safe
+ * in keeping this size hardcoded.
+ */
+#define GF_REQUEST_MAXGROUPS    16
+
 typedef struct {
 	uint32_t pid;
 	uint32_t uid;
 	uint32_t gid;
+
+        /* Number of groups being sent through the array above. */
+        uint32_t ngrps;
+
+        /* Array of groups to which the uid belongs apart from the primary group
+         * in gid.
+         */
+        uint32_t groups[GF_REQUEST_MAXGROUPS];
+
+        uint64_t lk_owner;
 } __attribute__ ((packed)) gf_hdr_req_t;
 
 
