@@ -66,10 +66,7 @@ fill_defaults (xlator_t *xl)
 	SET_DEFAULT_FOP (symlink);
 	SET_DEFAULT_FOP (rename);
 	SET_DEFAULT_FOP (link);
-	SET_DEFAULT_FOP (chmod);
-	SET_DEFAULT_FOP (chown);
 	SET_DEFAULT_FOP (truncate);
-	SET_DEFAULT_FOP (utimens);
 	SET_DEFAULT_FOP (readv);
 	SET_DEFAULT_FOP (writev);
 	SET_DEFAULT_FOP (statfs);
@@ -82,6 +79,7 @@ fill_defaults (xlator_t *xl)
 	SET_DEFAULT_FOP (removexattr);
 	SET_DEFAULT_FOP (opendir);
 	SET_DEFAULT_FOP (readdir);
+	SET_DEFAULT_FOP (readdirp);
 	SET_DEFAULT_FOP (fsyncdir);
 	SET_DEFAULT_FOP (access);
 	SET_DEFAULT_FOP (ftruncate);
@@ -92,16 +90,18 @@ fill_defaults (xlator_t *xl)
 	SET_DEFAULT_FOP (entrylk);
 	SET_DEFAULT_FOP (fentrylk);
 	SET_DEFAULT_FOP (lookup);
-	SET_DEFAULT_FOP (fchown);
-	SET_DEFAULT_FOP (fchmod);
 	SET_DEFAULT_FOP (setdents);
 	SET_DEFAULT_FOP (getdents);
 	SET_DEFAULT_FOP (checksum);
+	SET_DEFAULT_FOP (rchecksum);
 	SET_DEFAULT_FOP (xattrop);
 	SET_DEFAULT_FOP (fxattrop);
 	SET_DEFAULT_FOP (lock_notify);
 	SET_DEFAULT_FOP (lock_fnotify);
-	
+        SET_DEFAULT_FOP (setattr);
+        SET_DEFAULT_FOP (fsetattr);
+
+        SET_DEFAULT_MOP (log);
 	SET_DEFAULT_MOP (stats);
 
 	SET_DEFAULT_CBK (release);
@@ -246,7 +246,15 @@ _volume_option_value_validate (xlator_t *xl,
 	switch (opt->type) {
 	case GF_OPTION_TYPE_PATH:
 	{
-  			/* Make sure the given path is valid */
+                if (strstr (pair->value->data, "../")) {
+                        gf_log (xl->name, GF_LOG_ERROR,
+                                "invalid path given '%s'",
+                                pair->value->data);
+                        ret = -1;
+                        goto out;
+                }
+
+                /* Make sure the given path is valid */
 		if (pair->value->data[0] != '/') {
 			gf_log (xl->name, GF_LOG_WARNING,
 				"option %s %s: '%s' is not an "
@@ -384,10 +392,12 @@ _volume_option_value_validate (xlator_t *xl,
 		    || ((i < ZR_OPTION_MAX_ARRAY_SIZE) 
 			&& (!opt->value[i]))) {
 			/* enter here only if
-			 * 1. reached end of opt->value array and haven't validated input
+			 * 1. reached end of opt->value array and haven't 
+                         *    validated input
 			 *                      OR
-			 * 2. valid input list is less than ZR_OPTION_MAX_ARRAY_SIZE and
-			 *    input has not matched all possible input values.
+			 * 2. valid input list is less than 
+                         *    ZR_OPTION_MAX_ARRAY_SIZE and input has not 
+                         *    matched all possible input values.
 			 */
 			char given_array[4096] = {0,};
 			for (i = 0; (i < ZR_OPTION_MAX_ARRAY_SIZE) &&
@@ -431,6 +441,96 @@ _volume_option_value_validate (xlator_t *xl,
 		ret = 0;
 	}
 	break;
+	case GF_OPTION_TYPE_PERCENT_OR_SIZET:
+	{
+		uint32_t percent = 0;
+		uint64_t input_size = 0;
+		
+		/* Check if the value is valid percentage */
+		if (gf_string2percent (pair->value->data,
+				       &percent) == 0) {
+			if (percent > 100) {
+				gf_log (xl->name, GF_LOG_DEBUG,
+					"value given was greater than 100, "
+					"assuming this is actually a size");
+        		        if (gf_string2bytesize (pair->value->data,
+      	        			                &input_size) == 0) {
+			       	        /* Check the range */
+	        			if ((opt->min == 0) && 
+                                            (opt->max == 0)) {
+	       	        		        gf_log (xl->name, GF_LOG_DEBUG,
+        	      	        		        "no range check "
+                                                        "required for "
+                	      		       		"'option %s %s'",
+	               	        	        	pair->key, 
+                                                        pair->value->data);
+						// It is a size
+			                        ret = 0;
+       				                goto out;
+              				}
+	        	        	if ((input_size < opt->min) ||
+	      			            (input_size > opt->max)) {
+        	       			        gf_log (xl->name, GF_LOG_ERROR,
+                	      			        "'%"PRId64"' in "
+                                                        "'option %s %s' is out"
+                       	       				" of range [%"PRId64""
+                                                        "- %"PRId64"]",
+               	        	        		input_size, pair->key,
+	       		                        	pair->value->data,
+	       		        	                opt->min, opt->max);
+		       	        	}
+					// It is a size
+		       			ret = 0;
+					goto out;
+				} else {
+					// It's not a percent or size
+					gf_log (xl->name, GF_LOG_ERROR,
+					"invalid number format \"%s\" "
+					"in \"option %s\"",
+					pair->value->data, pair->key);
+				}
+
+			}
+			// It is a percent
+			ret = 0;
+			goto out;
+		} else {
+       		        if (gf_string2bytesize (pair->value->data,
+     	        			        &input_size) == 0) {
+		       	        /* Check the range */
+        			if ((opt->min == 0) && (opt->max == 0)) {
+       	        		        gf_log (xl->name, GF_LOG_DEBUG,
+       	      	        		        "no range check required for "
+               	      		      		"'option %s %s'",
+               	        	        	pair->key, pair->value->data);
+					// It is a size
+		                        ret = 0;
+      				        goto out;
+             			}
+	        	        if ((input_size < opt->min) ||
+      			            (input_size > opt->max)) {
+       	       				gf_log (xl->name, GF_LOG_ERROR,
+               	      			        "'%"PRId64"' in 'option %s %s'"
+                                                " is out of range [%"PRId64" -"
+                                                " %"PRId64"]",
+              	        	        	input_size, pair->key,
+       		                        	pair->value->data,
+       		        	                opt->min, opt->max);
+				}
+			} else {
+				// It's not a percent or size
+				gf_log (xl->name, GF_LOG_ERROR,
+					"invalid number format \"%s\" "
+					"in \"option %s\"",
+					pair->value->data, pair->key);
+			}
+			//It is a size
+                        ret = 0;
+ 		        goto out;
+		}
+
+	}
+	break;
 	case GF_OPTION_TYPE_TIME:
 	{
 		uint32_t input_time = 0;
@@ -578,6 +678,7 @@ int32_t
 xlator_set_type (xlator_t *xl,
 		 const char *type)
 {
+        int   ret = 0;
 	char *name = NULL;
 	void *handle = NULL;
 	volume_opt_list_t *vol_opt = NULL;
@@ -589,7 +690,11 @@ xlator_set_type (xlator_t *xl,
 
 	xl->type = strdup (type);
 
-	asprintf (&name, "%s/%s.so", XLATORDIR, type);
+	ret = asprintf (&name, "%s/%s.so", XLATORDIR, type);
+        if (-1 == ret) {
+                gf_log ("xlator", GF_LOG_ERROR, "asprintf failed");
+                return -1;
+        }
 
 	gf_log ("xlator", GF_LOG_TRACE, "attempt to load file %s", name);
 
@@ -634,6 +739,11 @@ xlator_set_type (xlator_t *xl,
 			"dlsym(notify) on %s -- neglecting", dlerror ());
 	}
 
+	if (!(xl->dumpops = dlsym (handle, "dumpops"))) {
+		gf_log ("xlator", GF_LOG_DEBUG,
+			"dlsym(dumpops) on %s -- neglecting", dlerror ());
+	}
+
 	INIT_LIST_HEAD (&xl->volume_options);
 
 	vol_opt = CALLOC (1, sizeof (volume_opt_list_t));
@@ -705,7 +815,7 @@ xlator_search_by_name (xlator_t *any, const char *name)
 static int32_t
 xlator_init_rec (xlator_t *xl)
 {
-	xlator_list_t *trav = NULL;
+	xlator_t *trav = NULL;
 	int32_t ret = 0;
 
 	if (xl == NULL)	{
@@ -713,35 +823,29 @@ xlator_init_rec (xlator_t *xl)
 		return 0;
 	}
 
-	trav = xl->children;
+	trav = xl;
+        while (trav->prev)
+                trav = trav->prev;
 
-	while (trav) {
-		ret = 0;
-		ret = xlator_init_rec (trav->xlator);
-		if (ret != 0)
-			break;
-		gf_log (trav->xlator->name, GF_LOG_TRACE,
-			"Initialization done");
-		trav = trav->next;
-	}
-
-	if (!ret && !xl->ready) {
+        while (trav) {
 		ret = -1;
-		if (xl->init) {
-			ret = xl->init (xl);
+		if (trav->init && !trav->ready) {
+			ret = xlator_init (trav);
 			if (ret) {
-				gf_log ("xlator", GF_LOG_ERROR,
+				gf_log (trav->name, GF_LOG_ERROR,
 					"Initialization of volume '%s' failed,"
 					" review your volfile again",
-					xl->name);
+					trav->name);
+                                break;
 			} else {
-				xl->init_succeeded = 1;
+				trav->init_succeeded = 1;
 			}
 		} else {
-			gf_log (xl->name, GF_LOG_DEBUG, "No init() found");
+			gf_log (trav->name, GF_LOG_DEBUG, "No init() found");
 		}
 		/* This 'xl' is checked */
-		xl->ready = 1;
+		trav->ready = 1;
+                trav = trav->next;
 	}
 
 	return ret;
@@ -807,6 +911,40 @@ xlator_fini_rec (xlator_t *xl)
 }
 
 
+int
+xlator_notify (xlator_t *xl, int event, void *data, ...)
+{
+        xlator_t *old_THIS = NULL;
+        int       ret = 0;
+
+        old_THIS = THIS;
+        THIS = xl;
+
+        ret = xl->notify (xl, event, data);
+
+        THIS = old_THIS;
+
+        return ret;
+}
+
+
+int
+xlator_init (xlator_t *xl)
+{
+        xlator_t *old_THIS = NULL;
+        int       ret = 0;
+
+        old_THIS = THIS;
+        THIS = xl;
+
+        ret = xl->init (xl);
+
+        THIS = old_THIS;
+
+        return ret;
+}
+
+
 void
 xlator_tree_fini (xlator_t *xl)
 {
