@@ -397,7 +397,7 @@ ib_verbs_writev (transport_t *this,
                 peer = &priv->peer;
                 if (list_empty (&peer->ioq)) {
                         ret = __ib_verbs_ioq_churn_entry (peer, entry);
-                        if (ret > 0) {
+                        if (ret != 0) {
                                 need_append = 0;
                         }
                 }
@@ -1189,8 +1189,8 @@ ib_verbs_recv_completion_proc (void *data)
                                 }
                                 pthread_mutex_unlock (&priv->recv_mutex);
         
-                                if ((ret = peer->trans->xl->notify (peer->trans->xl, GF_EVENT_POLLIN, 
-                                                                    peer->trans, NULL)) == -1) {
+                                if ((ret = xlator_notify (peer->trans->xl, GF_EVENT_POLLIN, 
+                                                          peer->trans, NULL)) == -1) {
                                         gf_log ("transport/ib-verbs",
                                                 GF_LOG_DEBUG, 
                                                 "pollin notification to %s "
@@ -1555,8 +1555,6 @@ ib_verbs_init (transport_t *this)
         struct ibv_device **dev_list;
         struct ibv_device *ib_dev = NULL;
         int32_t i;
-        uint64_t windowsize = GF_DEFAULT_IBV_WINDOW_SIZE;
-        char *wsizestr = NULL;
 
         ib_verbs_options_init (this);
 
@@ -1609,16 +1607,6 @@ ib_verbs_init (transport_t *this)
                         return -1;
                 }
                 ibv_free_device_list (dev_list);
-
-                if (dict_get_str (this->xl->options, "window-size",
-                                  &wsizestr) == 0) {
-                        if (gf_string2bytesize (wsizestr, &windowsize) != 0) {
-                                gf_log (this->xl->name, GF_LOG_ERROR,
-                                        "invalid number format: %s", wsizestr);
-                                return -1;
-                        }
-                }
-                priv->windowsize = windowsize;
         }
 
         priv->peer.trans = this;
@@ -1846,7 +1834,7 @@ unlock:
         }
 
         if (!ret && priv->connected) {
-                ret = this->xl->notify (this->xl, GF_EVENT_CHILD_UP, this);
+                ret = xlator_notify (this->xl, GF_EVENT_CHILD_UP, this);
         }
 
         return ret;
@@ -1986,7 +1974,7 @@ ib_verbs_handshake_pollerr (transport_t *this)
         }
         pthread_mutex_unlock (&priv->write_mutex);
 
-        this->xl->notify (this->xl, GF_EVENT_POLLERR, this, NULL);
+        xlator_notify (this->xl, GF_EVENT_POLLERR, this, NULL);
 
         if (need_unref)
                 transport_unref (this);
@@ -2164,24 +2152,6 @@ ib_verbs_connect (struct transport *this)
                 gf_log (this->xl->name, GF_LOG_TRACE,
                         "socket fd = %d", priv->sock);
 
-                if (setsockopt (priv->sock, SOL_SOCKET, SO_RCVBUF,
-                                &priv->windowsize,
-                                sizeof (priv->windowsize)) < 0) {
-                        gf_log (this->xl->name, GF_LOG_ERROR,
-                                "setting receive window size failed: %d: %d: "
-                                "%s", priv->sock, priv->windowsize,
-                                strerror (errno));
-                }
-
-                if (setsockopt (priv->sock, SOL_SOCKET, SO_SNDBUF,
-                                &priv->windowsize,
-                                sizeof (priv->windowsize)) < 0) {
-                        gf_log (this->xl->name, GF_LOG_ERROR,
-                                "setting send window size failed: %d: %d: "
-                                "%s", priv->sock, priv->windowsize,
-                                strerror (errno));
-                }
-
                 memcpy (&this->peerinfo.sockaddr, &sockaddr, sockaddr_len);
                 this->peerinfo.sockaddr_len = sockaddr_len;
 
@@ -2383,20 +2353,6 @@ ib_verbs_listen (transport_t *this)
                 goto err;
         }
 
-        if (setsockopt (priv->sock, SOL_SOCKET, SO_RCVBUF, &priv->windowsize,
-                        sizeof (priv->windowsize)) < 0) {
-                gf_log (this->xl->name, GF_LOG_ERROR,
-                        "setting receive window size failed: %d: %d: %s",
-                        priv->sock, priv->windowsize, strerror (errno));
-        }
-
-        if (setsockopt (priv->sock, SOL_SOCKET, SO_SNDBUF, &priv->windowsize,
-                        sizeof (priv->windowsize)) < 0) {
-                gf_log (this->xl->name, GF_LOG_ERROR,
-                        "setting send window size failed: %d: %d: %s",
-                        priv->sock, priv->windowsize, strerror (errno));
-        }
-
         if (listen (priv->sock, 10) != 0) {
                 gf_log ("ib-verbs/server", GF_LOG_ERROR,
                         "init: listen () failed on socket for %s (%s)",
@@ -2506,10 +2462,5 @@ struct volume_options options[] = {
                     "unix", "inet-sdp" },
           .type  = GF_OPTION_TYPE_STR 
         },
-        { .key   = {"transport.window-size"},
-          .type  = GF_OPTION_TYPE_SIZET,
-          .min   = GF_MIN_IBV_WINDOW_SIZE,
-          .max   = GF_MAX_IBV_WINDOW_SIZE,
-        },
         { .key = {NULL} }
 };
