@@ -28,6 +28,12 @@ client_bind (transport_t *this,
              socklen_t *sockaddr_len, 
              int sock);
 
+int32_t
+client_fill_address_family (transport_t *this, sa_family_t *sa_family);
+
+int32_t
+server_fill_address_family (transport_t *this, sa_family_t *sa_family);
+
 int32_t
 socket_client_get_remote_sockaddr (transport_t *this, 
                                    struct sockaddr *sockaddr, 
