@@ -34,8 +34,8 @@
 #include "compat-errno.h"
 
 #include <fcntl.h>
-#include <netinet/tcp.h>
 #include <errno.h>
+#include <netinet/tcp.h>
 
 
 #define GF_LOG_ERRNO(errno) ((errno == ENOTCONN) ? GF_LOG_DEBUG : GF_LOG_ERROR)
@@ -230,6 +230,21 @@ __socket_nonblock (int fd)
 }
 
 
+int
+__socket_nodelay (int fd)
+{
+        int     on = 1;
+        int     ret = -1;
+
+        ret = setsockopt (fd, IPPROTO_TCP, TCP_NODELAY,
+			  &on, sizeof (on));
+        if (!ret)
+                gf_log ("", GF_LOG_TRACE,
+                        "NODELAY enabled for socket %d", fd);
+
+        return ret;
+}
+
 int
 __socket_connect_finish (int fd)
 {
@@ -417,7 +432,7 @@ socket_event_poll_err (transport_t *this)
         }
         pthread_mutex_unlock (&priv->lock);
 
-        this->xl->notify (this->xl, GF_EVENT_POLLERR, this);
+        xlator_notify (this->xl, GF_EVENT_POLLERR, this);
 
         return ret;
 }
@@ -443,7 +458,7 @@ socket_event_poll_out (transport_t *this)
         }
         pthread_mutex_unlock (&priv->lock);
 
-        this->xl->notify (this->xl, GF_EVENT_POLLOUT, this);
+        xlator_notify (this->xl, GF_EVENT_POLLOUT, this);
 
         return ret;
 }
@@ -711,7 +726,7 @@ socket_event_poll_in (transport_t *this)
            just to keep the last_received timestamp ticking */
 
         if (ret == 0)
-                ret = this->xl->notify (this->xl, GF_EVENT_POLLIN, this);
+                ret = xlator_notify (this->xl, GF_EVENT_POLLIN, this);
 
         return ret;
 }
@@ -779,7 +794,7 @@ unlock:
         pthread_mutex_unlock (&priv->lock);
 
         if (notify_xlator)
-                this->xl->notify (this->xl, event, this);
+                xlator_notify (this->xl, event, this);
 
         return 0;
 }
@@ -864,6 +879,16 @@ socket_server_event_handler (int fd, int idx, void *data,
                                 }
                         }
 
+                        if (priv->nodelay) {
+                                ret = __socket_nodelay (new_sock);
+                                if (ret == -1) {
+                                        gf_log (this->xl->name, GF_LOG_ERROR,
+                                                "setsockopt() failed for "
+                                                "NODELAY (%s)",
+                                                strerror (errno));
+                                }
+                        }
+
                         new_trans = CALLOC (1, sizeof (*new_trans));
                         new_trans->xl = this->xl;
                         new_trans->fini = this->fini;
@@ -946,7 +971,7 @@ socket_connect (transport_t *this)
         struct sockaddr_storage  sockaddr = {0, };
         socklen_t                sockaddr_len = 0;
 	glusterfs_ctx_t         *ctx = NULL;
-        int                      on = 1;
+        sa_family_t              sa_family = {0, };
 
         priv = this->private;
 	ctx = this->xl->ctx;
@@ -970,6 +995,11 @@ socket_connect (transport_t *this)
                 goto err;
         }
 
+        ret = client_fill_address_family (this, &sa_family);
+        if (ret == -1) {
+                goto err;
+        }
+
         ret = socket_client_get_remote_sockaddr (this, SA (&sockaddr),
                                                  &sockaddr_len);
         if (ret == -1) {
@@ -988,9 +1018,7 @@ socket_connect (transport_t *this)
                 memcpy (&this->peerinfo.sockaddr, &sockaddr, sockaddr_len);
                 this->peerinfo.sockaddr_len = sockaddr_len;
 
-                priv->sock = socket (SA (&sockaddr)->sa_family,
-				     SOCK_STREAM, 0);
-
+                priv->sock = socket (sa_family, SOCK_STREAM, 0);
                 if (priv->sock == -1) {
                         gf_log (this->xl->name, GF_LOG_ERROR,
                                 "socket creation failed (%s)",
@@ -1019,10 +1047,9 @@ socket_connect (transport_t *this)
                                 strerror (errno));
                 }
 
-	
-                if (priv->nodelay) {
-                        ret = setsockopt(priv->sock, IPPROTO_TCP, 
-                                         TCP_NODELAY, &on, sizeof(on));
+
+                if (priv->nodelay && priv->lowlat) {
+                        ret = __socket_nodelay (priv->sock);
                         if (ret == -1) {
                                 gf_log (this->xl->name, GF_LOG_ERROR,
                                         "setsockopt() failed for NODELAY (%s)",
@@ -1030,7 +1057,6 @@ socket_connect (transport_t *this)
                         }
                 }
 
-
                 if (!priv->bio) {
                         ret = __socket_nonblock (priv->sock);
 
@@ -1096,7 +1122,7 @@ socket_listen (transport_t *this)
         socklen_t                sockaddr_len;
         peer_info_t             *myinfo = NULL;
 	glusterfs_ctx_t         *ctx = NULL;
-        int                      on = 1;
+        sa_family_t              sa_family = {0, };
 
 	priv   = this->private;
 	myinfo = &this->myinfo;
@@ -1114,9 +1140,13 @@ socket_listen (transport_t *this)
                 return ret;
         }
 
+        ret = server_fill_address_family (this, &sa_family);
+        if (ret == -1) {
+                return ret;
+        }
+
         ret = socket_server_get_local_sockaddr (this, SA (&sockaddr), 
                                                 &sockaddr_len);
-
         if (ret == -1) {
                 return ret;
         }
@@ -1132,8 +1162,7 @@ socket_listen (transport_t *this)
                 memcpy (&myinfo->sockaddr, &sockaddr, sockaddr_len);
                 myinfo->sockaddr_len = sockaddr_len;
 
-                priv->sock = socket (SA (&sockaddr)->sa_family,
-				     SOCK_STREAM, 0);
+                priv->sock = socket (sa_family, SOCK_STREAM, 0);
 
                 if (priv->sock == -1) {
                         gf_log (this->xl->name, GF_LOG_ERROR,
@@ -1158,16 +1187,13 @@ socket_listen (transport_t *this)
                                 &priv->windowsize,
                                 sizeof (priv->windowsize)) < 0) {
                         gf_log (this->xl->name, GF_LOG_ERROR,
-                                "setting send window size failed: %d: %d: "
+                               "setting send window size failed: %d: %d: "
                                 "%s", priv->sock, priv->windowsize,
                                 strerror (errno));
                 }
 
-
                 if (priv->nodelay) {
-                        ret = setsockopt(priv->sock, IPPROTO_TCP, 
-                                         TCP_NODELAY, &on, sizeof(on));
-
+                        ret = __socket_nodelay (priv->sock);
                         if (ret == -1) {
                                 gf_log (this->xl->name, GF_LOG_ERROR,
                                         "setsockopt() failed for NODELAY (%s)",
@@ -1347,10 +1373,8 @@ socket_init (transport_t *this)
 {
         socket_private_t *priv = NULL;
         gf_boolean_t      tmp_bool = 0;
-        char             *nb_connect = NULL;
-        char             *optstr     = NULL;
         uint64_t          windowsize = GF_DEFAULT_SOCKET_WINDOW_SIZE;
-        char             *wsizestr = NULL;
+        char             *optstr = NULL;
 
         if (this->private) {
                 gf_log (this->xl->name, GF_LOG_DEBUG,
@@ -1375,10 +1399,10 @@ socket_init (transport_t *this)
         INIT_LIST_HEAD (&priv->ioq);
 
         if (dict_get (this->xl->options, "non-blocking-io")) {
-                nb_connect = data_to_str (dict_get (this->xl->options,
+                optstr = data_to_str (dict_get (this->xl->options,
                                                           "non-blocking-io"));
       
-                if (gf_string2boolean (nb_connect, &tmp_bool) == -1) {
+                if (gf_string2boolean (optstr, &tmp_bool) == -1) {
                         gf_log (this->xl->name, GF_LOG_ERROR,
                                 "'non-blocking-io' takes only boolean options,"
 				" not taking any action");
@@ -1392,32 +1416,45 @@ socket_init (transport_t *this)
                 }
         }
 
+        optstr = NULL;
+        
+        // By default, we enable NODELAY
+        priv->nodelay = 1;
         if (dict_get (this->xl->options, "transport.socket.nodelay")) {
                 optstr = data_to_str (dict_get (this->xl->options,
                                                 "transport.socket.nodelay"));
-       
+
                 if (gf_string2boolean (optstr, &tmp_bool) == -1) {
                         gf_log (this->xl->name, GF_LOG_ERROR,
                                 "'transport.socket.nodelay' takes only "
                                  "boolean options, not taking any action");
-                        tmp_bool = 0;
+                        tmp_bool = 1;
                 }
-                // By default we do not enable NODELAY
-                priv->nodelay = 0;
-                if (tmp_bool) {
-                        priv->nodelay = 1;
+                if (!tmp_bool) {
+                        priv->nodelay = 0;
                         gf_log (this->xl->name, GF_LOG_DEBUG,
-                                "enabling nodelay");
+                                "disabling nodelay");
                 }
         }
- 
-        if (dict_get_str (this->xl->options, "window-size", &wsizestr) == 0) {
-                if (gf_string2bytesize (wsizestr, &windowsize) != 0) {
+
+
+        optstr = NULL;
+        if (dict_get_str (this->xl->options, "transport.window-size",
+                          &optstr) == 0) {
+                if (gf_string2bytesize (optstr, &windowsize) != 0) {
                         gf_log (this->xl->name, GF_LOG_ERROR,
-                                "invalid number format: %s", wsizestr);
+                                "invalid number format: %s", optstr);
                         return -1;
                 }
         }
+
+        optstr = NULL;
+
+        if (dict_get_str (this->xl->options, "transport.socket.lowlat",
+                          &optstr) == 0) {
+                priv->lowlat = 1;
+        }
+
         priv->windowsize = (int)windowsize;
         this->private = priv;
 
@@ -1479,15 +1516,21 @@ struct volume_options options[] = {
                     "unix", "inet-sdp" },
           .type  = GF_OPTION_TYPE_STR 
         },
+
+        { .key   = {"non-blocking-io"}, 
+          .type  = GF_OPTION_TYPE_BOOL
+        },
         { .key   = {"transport.window-size"},
           .type  = GF_OPTION_TYPE_SIZET,
           .min   = GF_MIN_SOCKET_WINDOW_SIZE,
           .max   = GF_MAX_SOCKET_WINDOW_SIZE,
         },
-        { .key   = {"transport.socket.nodelay"}, 
+        { .key   = {"transport.socket.nodelay"},
+          .type  = GF_OPTION_TYPE_BOOL
+        },
+        { .key   = {"transport.socket.lowlat"},
           .type  = GF_OPTION_TYPE_BOOL
         },
-
         { .key = {NULL} }
 };
 
