@@ -30,6 +30,7 @@
 #endif
 
 #include "glusterfs.h"
+#include "inode.h"
 #include "afr.h"
 #include "dict.h"
 #include "xlator.h"
@@ -372,9 +373,15 @@ build_child_loc (xlator_t *this, loc_t *child, loc_t *parent, char *name)
 	}
 
 	if (strcmp (parent->path, "/") == 0)
-		asprintf ((char **)&child->path, "/%s", name);
+		ret = asprintf ((char **)&child->path, "/%s", name);
 	else
-		asprintf ((char **)&child->path, "%s/%s", parent->path, name);
+		ret = asprintf ((char **)&child->path, "%s/%s", parent->path, 
+                                name);
+
+        if (-1 == ret) {
+                gf_log (this->name, GF_LOG_ERROR,
+                        "asprintf failed while setting child path");
+        }
 
 	if (!child->path) {
 		gf_log (this->name, GF_LOG_ERROR,
@@ -437,11 +444,47 @@ afr_sh_entry_expunge_entry_done (call_frame_t *frame, xlator_t *this,
 	return 0;
 }
 
+int
+afr_sh_entry_expunge_parent_setattr_cbk (call_frame_t *expunge_frame,
+                                         void *cookie, xlator_t *this,
+                                         int32_t op_ret, int32_t op_errno,
+                                         struct stat *preop, struct stat *postop)
+{
+	afr_private_t   *priv          = NULL;
+	afr_local_t     *expunge_local = NULL;
+	afr_self_heal_t *expunge_sh    = NULL;
+        call_frame_t    *frame         = NULL;
+
+        int active_src = (long) cookie;
+
+        priv          = this->private;
+        expunge_local = expunge_frame->local;
+	expunge_sh    = &expunge_local->self_heal;
+	frame         = expunge_sh->sh_frame;
+
+        if (op_ret != 0) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "setattr on parent directory of %s on subvolume %s failed: %s",
+                        expunge_local->loc.path,
+                        priv->children[active_src]->name, strerror (op_errno));
+        }
+
+	AFR_STACK_DESTROY (expunge_frame);
+	afr_sh_entry_expunge_entry_done (frame, this, active_src);
+
+        return 0;
+}
+
 
 int
-afr_sh_entry_expunge_remove_cbk (call_frame_t *expunge_frame, void *cookie,
+afr_sh_entry_expunge_rename_cbk (call_frame_t *expunge_frame, void *cookie,
 				 xlator_t *this,
-				 int32_t op_ret, int32_t op_errno)
+				 int32_t op_ret, int32_t op_errno,
+                                 struct stat *buf,
+                                 struct stat *preoldparent,
+                                 struct stat *postoldparent,
+                                 struct stat *prenewparent,
+                                 struct stat *postnewparent)
 {
 	afr_private_t   *priv = NULL;
 	afr_local_t     *expunge_local = NULL;
@@ -449,6 +492,7 @@ afr_sh_entry_expunge_remove_cbk (call_frame_t *expunge_frame, void *cookie,
 	int              active_src = 0;
 	call_frame_t    *frame = NULL;
 
+        int32_t valid = 0;
 
 	priv = this->private;
 	expunge_local = expunge_frame->local;
@@ -470,56 +514,230 @@ afr_sh_entry_expunge_remove_cbk (call_frame_t *expunge_frame, void *cookie,
 			strerror (op_errno));
 	}
 
-	AFR_STACK_DESTROY (expunge_frame);
-	afr_sh_entry_expunge_entry_done (frame, this, active_src);
+        valid = GF_SET_ATTR_ATIME | GF_SET_ATTR_MTIME;
+        afr_build_parent_loc (&expunge_sh->parent_loc, &expunge_local->loc);
+
+        STACK_WIND_COOKIE (expunge_frame, afr_sh_entry_expunge_parent_setattr_cbk,
+                           (void *) (long) active_src,
+                           priv->children[active_src],
+                           priv->children[active_src]->fops->setattr,
+                           &expunge_sh->parent_loc,
+                           &expunge_sh->parentbuf,
+                           valid);
+
+        return 0;
+}
+
+
+static void
+init_trash_loc (loc_t *trash_loc, inode_table_t *table)
+{
+        trash_loc->path   = strdup ("/" GF_REPLICATE_TRASH_DIR);
+        trash_loc->name   = GF_REPLICATE_TRASH_DIR;
+        trash_loc->parent = table->root;
+        trash_loc->inode  = inode_new (table);
+}
+
+
+char *
+make_trash_path (const char *path)
+{
+        char *c  = NULL;
+        char *tp = NULL;
+
+        tp = CALLOC (strlen ("/" GF_REPLICATE_TRASH_DIR) + strlen (path) + 1, sizeof (char));
+
+        strcpy (tp, GF_REPLICATE_TRASH_DIR);
+        strcat (tp, path);
+
+        c = strchr (tp, '/') + 1;
+        while (*c++)
+                if (*c == '/')
+                        *c = '-';
+
+        return tp;
+}
+
+
+int
+afr_sh_entry_expunge_rename (call_frame_t *expunge_frame, xlator_t *this,
+                             int active_src, inode_t *trash_inode)
+{
+	afr_private_t   *priv = NULL;
+	afr_local_t     *expunge_local = NULL;
+
+        loc_t rename_loc;
+
+	priv          = this->private;
+	expunge_local = expunge_frame->local;
+
+        rename_loc.inode = inode_ref (expunge_local->loc.inode);
+        rename_loc.path  = make_trash_path (expunge_local->loc.path);
+        rename_loc.name  = strrchr (rename_loc.path, '/') + 1;
+        rename_loc.parent = trash_inode;
+
+	gf_log (this->name, GF_LOG_TRACE,
+		"moving file/directory %s on %s to %s",
+		expunge_local->loc.path, priv->children[active_src]->name,
+                rename_loc.path);
+
+	STACK_WIND_COOKIE (expunge_frame, afr_sh_entry_expunge_rename_cbk,
+			   (void *) (long) active_src,
+			   priv->children[active_src],
+			   priv->children[active_src]->fops->rename,
+			   &expunge_local->loc, &rename_loc);
+
+        loc_wipe (&rename_loc);
 
 	return 0;
 }
 
 
 int
-afr_sh_entry_expunge_rmdir (call_frame_t *expunge_frame, xlator_t *this,
-			     int active_src)
+afr_sh_entry_expunge_mkdir_cbk (call_frame_t *expunge_frame, void *cookie, xlator_t *this,
+                                int32_t op_ret, int32_t op_errno, inode_t *inode,
+                                struct stat *buf, struct stat *preparent,
+                                struct stat *postparent)
 {
-	afr_private_t   *priv = NULL;
-	afr_local_t     *expunge_local = NULL;
+        afr_private_t *priv            = NULL;
+        afr_local_t     *expunge_local = NULL;
+	afr_self_heal_t *expunge_sh    = NULL;
+        call_frame_t    *frame         = NULL;
 
-	priv = this->private;
-	expunge_local = expunge_frame->local;
+        int active_src = (long) cookie;
 
-	gf_log (this->name, GF_LOG_TRACE,
-		"removing directory %s on %s",
-		expunge_local->loc.path, priv->children[active_src]->name);
+        inode_t *trash_inode = NULL;
 
-	STACK_WIND_COOKIE (expunge_frame, afr_sh_entry_expunge_remove_cbk,
-			   (void *) (long) active_src,
-			   priv->children[active_src],
-			   priv->children[active_src]->fops->rmdir,
-			   &expunge_local->loc);
+        priv          = this->private;
+        expunge_local = expunge_frame->local;
+	expunge_sh    = &expunge_local->self_heal;
+	frame         = expunge_sh->sh_frame;
 
-	return 0;
+        if (op_ret != 0) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "mkdir of /" GF_REPLICATE_TRASH_DIR " failed on %s",
+                        priv->children[active_src]->name);
+
+                goto out;
+        }
+
+        /* mkdir successful */
+
+        trash_inode = inode_link (inode, expunge_local->loc.inode->table->root,
+                                  GF_REPLICATE_TRASH_DIR, buf);
+
+        afr_sh_entry_expunge_rename (expunge_frame, this, active_src,
+                                     trash_inode);
+        return 0;
+out:
+        AFR_STACK_DESTROY (expunge_frame);
+        afr_sh_entry_expunge_entry_done (frame, this, active_src);
+        return 0;
 }
 
 
 int
-afr_sh_entry_expunge_unlink (call_frame_t *expunge_frame, xlator_t *this,
-			     int active_src)
+afr_sh_entry_expunge_lookup_trash_cbk (call_frame_t *expunge_frame, void *cookie,
+                                       xlator_t *this,
+                                       int32_t op_ret, int32_t op_errno,
+                                       inode_t *inode, struct stat *buf,
+                                       dict_t *xattr, struct stat *postparent)
+{
+        afr_private_t *priv            = NULL;
+        afr_local_t     *expunge_local = NULL;
+	afr_self_heal_t *expunge_sh    = NULL;
+        call_frame_t    *frame         = NULL;
+
+        int active_src = (long) cookie;
+
+        inode_t *trash_inode;
+        loc_t    trash_loc;
+
+        priv          = this->private;
+        expunge_local = expunge_frame->local;
+	expunge_sh    = &expunge_local->self_heal;
+	frame         = expunge_sh->sh_frame;
+
+        if ((op_ret != 0) && (op_errno == ENOENT)) {
+                init_trash_loc (&trash_loc, expunge_local->loc.inode->table);
+
+                gf_log (this->name, GF_LOG_TRACE,
+                        "creating directory " GF_REPLICATE_TRASH_DIR " on subvolume %s",
+                        priv->children[active_src]->name);
+
+                STACK_WIND_COOKIE (expunge_frame, afr_sh_entry_expunge_mkdir_cbk,
+                                   (void *) (long) active_src,
+                                   priv->children[active_src],
+                                   priv->children[active_src]->fops->mkdir,
+                                   &trash_loc, 0777);
+
+                loc_wipe (&trash_loc);
+                return 0;
+        }
+
+        if (op_ret != 0) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "lookup of /" GF_REPLICATE_TRASH_DIR " failed on %s",
+                        priv->children[active_src]->name);
+                goto out;
+        }
+
+        /* lookup successful */
+
+        trash_inode = inode_link (inode, expunge_local->loc.inode->table->root,
+                                  GF_REPLICATE_TRASH_DIR, buf);
+
+        afr_sh_entry_expunge_rename (expunge_frame, this, active_src,
+                                     trash_inode);
+        return 0;
+out:
+        AFR_STACK_DESTROY (expunge_frame);
+        afr_sh_entry_expunge_entry_done (frame, this, active_src);
+        return 0;
+}
+
+
+int
+afr_sh_entry_expunge_lookup_trash (call_frame_t *expunge_frame, xlator_t *this,
+                                   int active_src)
 {
 	afr_private_t   *priv = NULL;
 	afr_local_t     *expunge_local = NULL;
 
-	priv = this->private;
+        inode_t *root  = NULL;
+        inode_t *trash = NULL;
+        loc_t trash_loc;
+
+	priv          = this->private;
 	expunge_local = expunge_frame->local;
 
+        root = expunge_local->loc.inode->table->root;
+
+        trash = inode_grep (root->table, root, GF_REPLICATE_TRASH_DIR);
+
+        if (trash) {
+                /* inode is in cache, so no need to mkdir */
+
+                afr_sh_entry_expunge_rename (expunge_frame, this, active_src,
+                                             trash);
+                return 0;
+        }
+
+        /* Not in cache, so look it up */
+
+        init_trash_loc (&trash_loc, expunge_local->loc.inode->table);
+
 	gf_log (this->name, GF_LOG_TRACE,
-		"unlinking file %s on %s",
-		expunge_local->loc.path, priv->children[active_src]->name);
-	
-	STACK_WIND_COOKIE (expunge_frame, afr_sh_entry_expunge_remove_cbk,
+		"looking up /" GF_REPLICATE_TRASH_DIR " on %s",
+                priv->children[active_src]->name);
+
+	STACK_WIND_COOKIE (expunge_frame, afr_sh_entry_expunge_lookup_trash_cbk,
 			   (void *) (long) active_src,
 			   priv->children[active_src],
-			   priv->children[active_src]->fops->unlink,
-			   &expunge_local->loc);
+			   priv->children[active_src]->fops->lookup,
+			   &trash_loc, NULL);
+
+        loc_wipe (&trash_loc);
 
 	return 0;
 }
@@ -551,11 +769,8 @@ afr_sh_entry_expunge_remove (call_frame_t *expunge_frame, xlator_t *this,
 	case S_IFCHR:
 	case S_IFIFO:
 	case S_IFLNK:
-		afr_sh_entry_expunge_unlink (expunge_frame, this, active_src);
-
-		break;
 	case S_IFDIR:
-		afr_sh_entry_expunge_rmdir (expunge_frame, this, active_src);
+		afr_sh_entry_expunge_lookup_trash (expunge_frame, this, active_src);
 		break;
 	default:
 		gf_log (this->name, GF_LOG_ERROR,
@@ -579,7 +794,8 @@ int
 afr_sh_entry_expunge_lookup_cbk (call_frame_t *expunge_frame, void *cookie,
 				xlator_t *this,
 				int32_t op_ret,	int32_t op_errno,
-				inode_t *inode, struct stat *buf, dict_t *x)
+                                inode_t *inode, struct stat *buf, dict_t *x,
+                                struct stat *postparent)
 {
 	afr_private_t   *priv = NULL;
 	afr_local_t     *expunge_local = NULL;
@@ -641,7 +857,8 @@ int
 afr_sh_entry_expunge_entry_cbk (call_frame_t *expunge_frame, void *cookie,
 				xlator_t *this,
 				int32_t op_ret,	int32_t op_errno,
-				inode_t *inode, struct stat *buf, dict_t *x)
+                                inode_t *inode, struct stat *buf, dict_t *x,
+                                struct stat *postparent)
 {
 	afr_private_t   *priv = NULL;
 	afr_local_t     *expunge_local = NULL;
@@ -665,6 +882,8 @@ afr_sh_entry_expunge_entry_cbk (call_frame_t *expunge_frame, void *cookie,
 			expunge_local->loc.path,
 			priv->children[source]->name);
 
+                expunge_sh->parentbuf = *postparent;
+
 		afr_sh_entry_expunge_purge (expunge_frame, this, active_src);
 
 		return 0;
@@ -713,8 +932,11 @@ afr_sh_entry_expunge_entry (call_frame_t *frame, xlator_t *this,
 	source = sh->source;
 
 	if ((strcmp (name, ".") == 0)
-	    || (strcmp (name, "..") == 0)) {
-		gf_log (this->name, GF_LOG_TRACE,
+	    || (strcmp (name, "..") == 0)
+            || ((strcmp (local->loc.path, "/") == 0)
+                && (strcmp (name, GF_REPLICATE_TRASH_DIR) == 0))) {
+
+                gf_log (this->name, GF_LOG_TRACE,
 			"skipping inspection of %s under %s",
 			name, local->loc.path);
 		goto out;
@@ -803,7 +1025,7 @@ afr_sh_entry_expunge_readdir_cbk (call_frame_t *frame, void *cookie,
 
 	list_for_each_entry (entry, &entries->list, list) {
 		last_offset = entry->d_off;
-		entry_count++;
+                entry_count++;
 	}
 
 	gf_log (this->name, GF_LOG_TRACE,
@@ -814,7 +1036,7 @@ afr_sh_entry_expunge_readdir_cbk (call_frame_t *frame, void *cookie,
 	local->call_count = entry_count;
 
 	list_for_each_entry (entry, &entries->list, list) {
-		afr_sh_entry_expunge_entry (frame, this, entry->d_name);
+                afr_sh_entry_expunge_entry (frame, this, entry->d_name);
 	}
 
 	return 0;
@@ -923,9 +1145,10 @@ afr_sh_entry_impunge_entry_done (call_frame_t *frame, xlator_t *this,
 
 
 int
-afr_sh_entry_impunge_utimens_cbk (call_frame_t *impunge_frame, void *cookie,
-				  xlator_t *this, int32_t op_ret,
-				  int32_t op_errno, struct stat *stbuf)
+afr_sh_entry_impunge_setattr_cbk (call_frame_t *impunge_frame, void *cookie,
+				  xlator_t *this,
+                                  int32_t op_ret, int32_t op_errno,
+                                  struct stat *preop, struct stat *postop)
 {
 	int              call_count = 0;
 	afr_private_t   *priv = NULL;
@@ -948,12 +1171,12 @@ afr_sh_entry_impunge_utimens_cbk (call_frame_t *impunge_frame, void *cookie,
 
 	if (op_ret == 0) {
 		gf_log (this->name, GF_LOG_TRACE,
-			"utimes set for %s on %s",
+			"setattr done for %s on %s",
 			impunge_local->loc.path,
 			priv->children[child_index]->name);
 	} else {
 		gf_log (this->name, GF_LOG_DEBUG,
-			"setting utimes of %s on %s failed (%s)",
+			"setattr (%s) on %s failed (%s)",
 			impunge_local->loc.path,
 			priv->children[child_index]->name,
 			strerror (op_errno));
@@ -974,76 +1197,6 @@ afr_sh_entry_impunge_utimens_cbk (call_frame_t *impunge_frame, void *cookie,
 }
 
 
-int
-afr_sh_entry_impunge_chown_cbk (call_frame_t *impunge_frame, void *cookie,
-				xlator_t *this, int32_t op_ret,
-				int32_t op_errno, struct stat *stbuf)
-{
-	int              call_count = 0;
-	afr_private_t   *priv = NULL;
-	afr_local_t     *impunge_local = NULL;
-	afr_self_heal_t *impunge_sh = NULL;
-	call_frame_t    *frame = NULL;
-	int              active_src = 0;
-	int              child_index = 0;
-	struct timespec  ts[2];
-
-
-	priv = this->private;
-	impunge_local = impunge_frame->local;
-	impunge_sh = &impunge_local->self_heal;
-	frame = impunge_sh->sh_frame;
-	child_index = (long) cookie;
-
-	if (op_ret == 0) {
-		gf_log (this->name, GF_LOG_TRACE,
-			"ownership of %s on %s changed",
-			impunge_local->loc.path,
-			priv->children[child_index]->name);
-	} else {
-		gf_log (this->name, GF_LOG_DEBUG,
-			"setting ownership of %s on %s failed (%s)",
-			impunge_local->loc.path,
-			priv->children[child_index]->name,
-			strerror (op_errno));
-		goto out;
-	}
-
-#ifdef HAVE_STRUCT_STAT_ST_ATIM_TV_NSEC
-	ts[0] = impunge_local->cont.lookup.buf.st_atim;
-	ts[1] = impunge_local->cont.lookup.buf.st_mtim;
-#elif HAVE_STRUCT_STAT_ST_ATIMESPEC_TV_NSEC
-	ts[0] = impunge_local->cont.lookup.buf.st_atimespec;
-	ts[1] = impunge_local->cont.lookup.buf.st_mtimespec;
-#else
-	ts[0].tv_sec = impunge_local->cont.lookup.buf.st_atime;
-	ts[1].tv_sec = impunge_local->cont.lookup.buf.st_mtime;
-#endif
-	STACK_WIND_COOKIE (impunge_frame,
-			   afr_sh_entry_impunge_utimens_cbk,
-			   (void *) (long) child_index,
-			   priv->children[child_index],
-			   priv->children[child_index]->fops->utimens,
-			   &impunge_local->loc, ts);
-
-	return 0;
-
-out:
-	LOCK (&impunge_frame->lock);
-	{
-		call_count = --impunge_local->call_count;
-	}
-	UNLOCK (&impunge_frame->lock);
-
-	if (call_count == 0) {
-		AFR_STACK_DESTROY (impunge_frame);
-		afr_sh_entry_impunge_entry_done (frame, this, active_src);
-	}
-
-	return 0;
-}
-
-
 int
 afr_sh_entry_impunge_xattrop_cbk (call_frame_t *impunge_frame, void *cookie,
                                   xlator_t *this,
@@ -1056,11 +1209,13 @@ afr_sh_entry_impunge_xattrop_cbk (call_frame_t *impunge_frame, void *cookie,
 	call_frame_t    *frame = NULL;
 	int              child_index = 0;
 
+        struct stat stbuf;
+        int32_t     valid = 0;
 
-	priv = this->private;
+	priv          = this->private;
 	impunge_local = impunge_frame->local;
-	impunge_sh = &impunge_local->self_heal;
-	frame = impunge_sh->sh_frame;
+	impunge_sh    = &impunge_local->self_heal;
+	frame         = impunge_sh->sh_frame;
 
 	child_index = (long) cookie;
 
@@ -1071,13 +1226,53 @@ afr_sh_entry_impunge_xattrop_cbk (call_frame_t *impunge_frame, void *cookie,
 		impunge_local->cont.lookup.buf.st_uid,
 		impunge_local->cont.lookup.buf.st_gid);
 
-	STACK_WIND_COOKIE (impunge_frame, afr_sh_entry_impunge_chown_cbk,
+#ifdef HAVE_STRUCT_STAT_ST_ATIM_TV_NSEC
+	stbuf.st_atim = impunge_local->cont.lookup.buf.st_atim;
+	stbuf.st_mtim = impunge_local->cont.lookup.buf.st_mtim;
+
+#elif HAVE_STRUCT_STAT_ST_ATIMESPEC_TV_NSEC
+	stbuf.st_atimespec = impunge_local->cont.lookup.buf.st_atimespec;
+	stbuf.st_mtimespec = impunge_local->cont.lookup.buf.st_mtimespec;
+#else
+	stbuf.st_atime = impunge_local->cont.lookup.buf.st_atime;
+	stbuf.st_mtime = impunge_local->cont.lookup.buf.st_mtime;
+#endif
+
+        stbuf.st_uid = impunge_local->cont.lookup.buf.st_uid;
+        stbuf.st_gid = impunge_local->cont.lookup.buf.st_gid;
+
+        valid = GF_SET_ATTR_UID   | GF_SET_ATTR_GID |
+                GF_SET_ATTR_ATIME | GF_SET_ATTR_MTIME;
+
+	STACK_WIND_COOKIE (impunge_frame, afr_sh_entry_impunge_setattr_cbk,
 			   (void *) (long) child_index,
 			   priv->children[child_index],
-			   priv->children[child_index]->fops->chown,
+			   priv->children[child_index]->fops->setattr,
 			   &impunge_local->loc,
-			   impunge_local->cont.lookup.buf.st_uid,
-			   impunge_local->cont.lookup.buf.st_gid);
+                           &stbuf, valid);
+        return 0;
+}
+
+
+int
+afr_sh_entry_impunge_parent_setattr_cbk (call_frame_t *setattr_frame,
+                                         void *cookie, xlator_t *this,
+                                         int32_t op_ret, int32_t op_errno,
+                                         struct stat *preop, struct stat *postop)
+{
+        loc_t *parent_loc = cookie;
+
+        if (op_ret != 0) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "setattr on parent directory failed: %s",
+                        strerror (op_errno));
+        }
+
+        loc_wipe (parent_loc);
+
+        FREE (parent_loc);
+
+        AFR_STACK_DESTROY (setattr_frame);
         return 0;
 }
 
@@ -1086,7 +1281,9 @@ int
 afr_sh_entry_impunge_newfile_cbk (call_frame_t *impunge_frame, void *cookie,
 				  xlator_t *this,
 				  int32_t op_ret, int32_t op_errno,
-				  inode_t *inode, struct stat *stbuf)
+                                  inode_t *inode, struct stat *stbuf,
+                                  struct stat *preparent,
+                                  struct stat *postparent)
 {
 	int              call_count = 0;
 	afr_private_t   *priv = NULL;
@@ -1102,6 +1299,11 @@ afr_sh_entry_impunge_newfile_cbk (call_frame_t *impunge_frame, void *cookie,
         afr_local_t     *local = NULL;
         afr_self_heal_t *sh = NULL;
 
+        call_frame_t *setattr_frame = NULL;
+        int32_t valid = 0;
+        loc_t *parent_loc = NULL;
+        struct stat parentbuf;
+
 	priv = this->private;
 	impunge_local = impunge_frame->local;
 	impunge_sh = &impunge_local->self_heal;
@@ -1137,12 +1339,25 @@ afr_sh_entry_impunge_newfile_cbk (call_frame_t *impunge_frame, void *cookie,
         ret = dict_set_static_bin (xattr, priv->pending_key[child_index],
                                    pending_array, sizeof (pending_array));
 
+        valid         = GF_SET_ATTR_ATIME | GF_SET_ATTR_MTIME;
+        parentbuf     = impunge_sh->parentbuf;
+        setattr_frame = copy_frame (impunge_frame);
+
+        parent_loc = CALLOC (1, sizeof (*parent_loc));
+        afr_build_parent_loc (parent_loc, &impunge_local->loc);
+
 	STACK_WIND_COOKIE (impunge_frame, afr_sh_entry_impunge_xattrop_cbk,
 			   (void *) (long) child_index,
 			   priv->children[active_src],
 			   priv->children[active_src]->fops->xattrop,
 			   &impunge_local->loc, GF_XATTROP_ADD_ARRAY, xattr);
 
+        STACK_WIND_COOKIE (setattr_frame, afr_sh_entry_impunge_parent_setattr_cbk,
+                           (void *) (long) parent_loc,
+                           priv->children[child_index],
+                           priv->children[child_index]->fops->setattr,
+                           parent_loc, &parentbuf, valid);
+
         dict_unref (xattr);
 
 	return 0;
@@ -1249,11 +1464,189 @@ afr_sh_entry_impunge_symlink (call_frame_t *impunge_frame, xlator_t *this,
 }
 
 
+int
+afr_sh_entry_impunge_symlink_unlink_cbk (call_frame_t *impunge_frame,
+                                         void *cookie, xlator_t *this,
+                                         int32_t op_ret, int32_t op_errno,
+                                         struct stat *preparent,
+                                         struct stat *postparent)
+{
+        afr_private_t   *priv = NULL;
+	afr_local_t     *impunge_local = NULL;
+	afr_self_heal_t *impunge_sh = NULL;
+	int              child_index = -1;
+	call_frame_t    *frame = NULL;
+	int              call_count = -1;
+	int              active_src = -1;
+
+	priv          = this->private;
+	impunge_local = impunge_frame->local;
+	impunge_sh    = &impunge_local->self_heal;
+	frame         = impunge_sh->sh_frame;
+	active_src    = impunge_sh->active_source;
+
+	child_index = (long) cookie;
+
+	if (op_ret == -1) {
+		gf_log (this->name, GF_LOG_DEBUG,
+			"unlink of %s on %s failed (%s)",
+			impunge_local->loc.path,
+			priv->children[child_index]->name,
+			strerror (op_errno));
+		goto out;
+	}
+
+        afr_sh_entry_impunge_symlink (impunge_frame, this, child_index,
+                                      impunge_sh->linkname);
+
+        return 0;
+out:
+	LOCK (&impunge_frame->lock);
+	{
+		call_count = --impunge_local->call_count;
+	}
+	UNLOCK (&impunge_frame->lock);
+
+	if (call_count == 0) {
+		AFR_STACK_DESTROY (impunge_frame);
+		afr_sh_entry_impunge_entry_done (frame, this, active_src);
+	}
+
+	return 0;
+}
+
+
+int
+afr_sh_entry_impunge_symlink_unlink (call_frame_t *impunge_frame, xlator_t *this,
+                                     int child_index)
+{
+	afr_private_t   *priv          = NULL;
+	afr_local_t     *impunge_local = NULL;
+	afr_self_heal_t *impunge_sh    = NULL;
+
+	priv          = this->private;
+	impunge_local = impunge_frame->local;
+	impunge_sh    = &impunge_local->self_heal;
+
+	gf_log (this->name, GF_LOG_DEBUG,
+		"unlinking symlink %s with wrong target on %s",
+		impunge_local->loc.path,
+		priv->children[child_index]->name);
+
+        STACK_WIND_COOKIE (impunge_frame, afr_sh_entry_impunge_symlink_unlink_cbk,
+                           (void *) (long) child_index,
+                           priv->children[child_index],
+                           priv->children[child_index]->fops->unlink,
+                           &impunge_local->loc);
+
+        return 0;
+}
+
+
+int
+afr_sh_entry_impunge_readlink_sink_cbk (call_frame_t *impunge_frame, void *cookie,
+                                        xlator_t *this,
+                                        int32_t op_ret, int32_t op_errno,
+                                        const char *linkname, struct stat *sbuf)
+{
+        afr_private_t   *priv = NULL;
+	afr_local_t     *impunge_local = NULL;
+	afr_self_heal_t *impunge_sh = NULL;
+	int              child_index = -1;
+	call_frame_t    *frame = NULL;
+	int              call_count = -1;
+	int              active_src = -1;
+
+	priv          = this->private;
+	impunge_local = impunge_frame->local;
+	impunge_sh    = &impunge_local->self_heal;
+	frame         = impunge_sh->sh_frame;
+	active_src    = impunge_sh->active_source;
+
+	child_index = (long) cookie;
+
+	if ((op_ret == -1) && (op_errno != ENOENT)) {
+		gf_log (this->name, GF_LOG_DEBUG,
+			"readlink of %s on %s failed (%s)",
+			impunge_local->loc.path,
+			priv->children[active_src]->name,
+			strerror (op_errno));
+		goto out;
+	}
+
+        /* symlink doesn't exist on the sink */
+
+        if ((op_ret == -1) && (op_errno == ENOENT)) {
+                afr_sh_entry_impunge_symlink (impunge_frame, this,
+                                              child_index, impunge_sh->linkname);
+                return 0;
+        }
+
+
+        /* symlink exists on the sink, so check if targets match */
+
+        if (strcmp (linkname, impunge_sh->linkname) == 0) {
+                /* targets match, nothing to do */
+
+                goto out;
+        } else {
+                /*
+                 * Hah! Sneaky wolf in sheep's clothing!
+                 */
+
+                afr_sh_entry_impunge_symlink_unlink (impunge_frame, this,
+                                                     child_index);
+                return 0;
+        }
+
+out:
+	LOCK (&impunge_frame->lock);
+	{
+		call_count = --impunge_local->call_count;
+	}
+	UNLOCK (&impunge_frame->lock);
+
+	if (call_count == 0) {
+		AFR_STACK_DESTROY (impunge_frame);
+		afr_sh_entry_impunge_entry_done (frame, this, active_src);
+	}
+
+	return 0;
+}
+
+
+int
+afr_sh_entry_impunge_readlink_sink (call_frame_t *impunge_frame, xlator_t *this,
+                                    int child_index)
+{
+	afr_private_t   *priv = NULL;
+	afr_local_t     *impunge_local = NULL;
+	afr_self_heal_t *impunge_sh = NULL;
+
+
+	priv = this->private;
+	impunge_local = impunge_frame->local;
+	impunge_sh = &impunge_local->self_heal;
+
+	gf_log (this->name, GF_LOG_DEBUG,
+		"checking symlink target of %s on %s",
+		impunge_local->loc.path, priv->children[child_index]->name);
+
+	STACK_WIND_COOKIE (impunge_frame, afr_sh_entry_impunge_readlink_sink_cbk,
+			   (void *) (long) child_index,
+			   priv->children[child_index],
+			   priv->children[child_index]->fops->readlink,
+			   &impunge_local->loc, 4096);
+
+	return 0;
+}
+
+
 int
 afr_sh_entry_impunge_readlink_cbk (call_frame_t *impunge_frame, void *cookie,
 				   xlator_t *this,
 				   int32_t op_ret, int32_t op_errno,
-				   const char *linkname)
+				   const char *linkname, struct stat *sbuf)
 {
 	afr_private_t   *priv = NULL;
 	afr_local_t     *impunge_local = NULL;
@@ -1280,8 +1673,10 @@ afr_sh_entry_impunge_readlink_cbk (call_frame_t *impunge_frame, void *cookie,
 		goto out;
 	}
 
-	afr_sh_entry_impunge_symlink (impunge_frame, this, child_index,
-				      linkname);
+        impunge_sh->linkname = strdup (linkname);
+
+	afr_sh_entry_impunge_readlink_sink (impunge_frame, this, child_index);
+
 	return 0;
 
 out:
@@ -1329,7 +1724,7 @@ afr_sh_entry_impunge_recreate_lookup_cbk (call_frame_t *impunge_frame,
 					  void *cookie, xlator_t *this,
 					  int32_t op_ret, int32_t op_errno,
 					  inode_t *inode, struct stat *buf,
-					  dict_t *xattr)
+					  dict_t *xattr,struct stat *postparent)
 {
 	afr_private_t   *priv = NULL;
 	afr_local_t     *impunge_local = NULL;
@@ -1359,6 +1754,8 @@ afr_sh_entry_impunge_recreate_lookup_cbk (call_frame_t *impunge_frame,
 		goto out;
 	}
 
+        impunge_sh->parentbuf = *postparent;
+
 	impunge_local->cont.lookup.buf = *buf;
 	type = (buf->st_mode & S_IFMT);
 
@@ -1437,7 +1834,8 @@ int
 afr_sh_entry_impunge_entry_cbk (call_frame_t *impunge_frame, void *cookie,
 				xlator_t *this,
 				int32_t op_ret,	int32_t op_errno,
-				inode_t *inode, struct stat *buf, dict_t *x)
+                                inode_t *inode, struct stat *buf, dict_t *x,
+                                struct stat *postparent)
 {
 	afr_private_t   *priv = NULL;
 	afr_local_t     *impunge_local = NULL;
@@ -1454,8 +1852,16 @@ afr_sh_entry_impunge_entry_cbk (call_frame_t *impunge_frame, void *cookie,
 	child_index = (long) cookie;
 	active_src = impunge_sh->active_source;
 
-	if (op_ret == -1 && op_errno == ENOENT) {
+	if ((op_ret == -1 && op_errno == ENOENT)
+            || (S_ISLNK (impunge_sh->impunging_entry_mode))) {
+
+                /*
+                 * A symlink's target might have changed, so
+                 * always go down the recreate path for them.
+                 */
+
 		/* decrease call_count in recreate-callback */
+
 		gf_log (this->name, GF_LOG_TRACE,
 			"missing entry %s on %s",
 			impunge_local->loc.path,
@@ -1471,6 +1877,8 @@ afr_sh_entry_impunge_entry_cbk (call_frame_t *impunge_frame, void *cookie,
 			"%s exists under %s",
 			impunge_local->loc.path,
 			priv->children[child_index]->name);
+
+                impunge_sh->parentbuf = *postparent;
 	} else {
 		gf_log (this->name, GF_LOG_TRACE,
 			"looking up %s under %s failed (%s)",
@@ -1496,7 +1904,7 @@ afr_sh_entry_impunge_entry_cbk (call_frame_t *impunge_frame, void *cookie,
 
 int
 afr_sh_entry_impunge_entry (call_frame_t *frame, xlator_t *this,
-			    char *name)
+			    gf_dirent_t *entry)
 {
 	afr_private_t   *priv = NULL;
 	afr_local_t     *local  = NULL;
@@ -1516,17 +1924,20 @@ afr_sh_entry_impunge_entry (call_frame_t *frame, xlator_t *this,
 
 	active_src = sh->active_source;
 
-	if ((strcmp (name, ".") == 0)
-	    || (strcmp (name, "..") == 0)) {
+	if ((strcmp (entry->d_name, ".") == 0)
+	    || (strcmp (entry->d_name, "..") == 0)
+            || ((strcmp (local->loc.path, "/") == 0)
+                && (strcmp (entry->d_name, GF_REPLICATE_TRASH_DIR) == 0))) {
+
 		gf_log (this->name, GF_LOG_TRACE,
 			"skipping inspection of %s under %s",
-			name, local->loc.path);
+			entry->d_name, local->loc.path);
 		goto out;
 	}
 
 	gf_log (this->name, GF_LOG_TRACE,
 		"inspecting existance of %s under %s",
-		name, local->loc.path);
+		entry->d_name, local->loc.path);
 
 	impunge_frame = copy_frame (frame);
 	if (!impunge_frame) {
@@ -1542,7 +1953,9 @@ afr_sh_entry_impunge_entry (call_frame_t *frame, xlator_t *this,
 	impunge_sh->sh_frame = frame;
 	impunge_sh->active_source = active_src;
 
-	ret = build_child_loc (this, &impunge_local->loc, &local->loc, name);
+        impunge_sh->impunging_entry_mode = entry->d_stat.st_mode;
+
+	ret = build_child_loc (this, &impunge_local->loc, &local->loc, entry->d_name);
 	if (ret != 0) {
 		goto out;
 	}
@@ -1586,7 +1999,7 @@ afr_sh_entry_impunge_entry (call_frame_t *frame, xlator_t *this,
 out:
 	if (ret == -1)
 		afr_sh_entry_impunge_entry_done (frame, this, active_src);
-	
+
 	return 0;
 }
 
@@ -1631,7 +2044,7 @@ afr_sh_entry_impunge_readdir_cbk (call_frame_t *frame, void *cookie,
 
 	list_for_each_entry (entry, &entries->list, list) {
 		last_offset = entry->d_off;
-		entry_count++;
+                entry_count++;
 	}
 
 	gf_log (this->name, GF_LOG_TRACE,
@@ -1642,7 +2055,7 @@ afr_sh_entry_impunge_readdir_cbk (call_frame_t *frame, void *cookie,
 	local->call_count = entry_count;
 
 	list_for_each_entry (entry, &entries->list, list) {
-		afr_sh_entry_impunge_entry (frame, this, entry->d_name);
+                afr_sh_entry_impunge_entry (frame, this, entry);
 	}
 
 	return 0;
@@ -1663,7 +2076,7 @@ afr_sh_entry_impunge_subvol (call_frame_t *frame, xlator_t *this,
 
 	STACK_WIND (frame, afr_sh_entry_impunge_readdir_cbk,
 		    priv->children[active_src],
-		    priv->children[active_src]->fops->readdir,
+		    priv->children[active_src]->fops->readdirp,
 		    sh->healing_fd, sh->block_size, sh->offset);
 
 	return 0;
@@ -1904,6 +2317,11 @@ afr_sh_entry_fix (call_frame_t *frame, xlator_t *this)
 	sh = &local->self_heal;
 	priv = this->private;
 
+        if (sh->forced_merge) {
+                sh->source = -1;
+                goto heal;
+        }
+
 	afr_sh_build_pending_matrix (priv, sh->pending_matrix, sh->xattr, 
 				     priv->child_count, AFR_ENTRY_TRANSACTION);
 
@@ -1925,8 +2343,10 @@ afr_sh_entry_fix (call_frame_t *frame, xlator_t *this)
 					  priv->child_count);
 
 	source = afr_sh_select_source (sh->sources, priv->child_count);
-	sh->source = source;
 
+        sh->source = source;
+
+heal:
 	afr_sh_entry_sync_prepare (frame, this);
 
 	return 0;
@@ -1937,7 +2357,8 @@ afr_sh_entry_fix (call_frame_t *frame, xlator_t *this)
 int
 afr_sh_entry_lookup_cbk (call_frame_t *frame, void *cookie,
 			 xlator_t *this, int32_t op_ret, int32_t op_errno,
-			 inode_t *inode, struct stat *buf, dict_t *xattr)
+                         inode_t *inode, struct stat *buf, dict_t *xattr,
+                         struct stat *postparent)
 {
 	afr_private_t   *priv  = NULL;
 	afr_local_t     *local = NULL;
@@ -1985,7 +2406,8 @@ afr_sh_entry_lookup (call_frame_t *frame, xlator_t *this)
 	local = frame->local;
 	sh    = &local->self_heal;
 
-	call_count = local->child_count;
+	call_count = afr_up_children_count (priv->child_count,
+                                            local->child_up);
 
 	local->call_count = call_count;
 	
@@ -2082,7 +2504,8 @@ afr_sh_entry_lock (call_frame_t *frame, xlator_t *this)
 	sh = &local->self_heal;
 	priv = this->private;
 
-	call_count = local->child_count;
+	call_count = afr_up_children_count (priv->child_count,
+                                            local->child_up);
 
 	local->call_count = call_count;		
 
@@ -2120,7 +2543,7 @@ afr_self_heal_entry (call_frame_t *frame, xlator_t *this)
 	local = frame->local;
 	sh = &local->self_heal;
 
-	if (local->need_entry_self_heal && priv->entry_self_heal) {
+	if (local->self_heal.need_entry_self_heal && priv->entry_self_heal) {
 		afr_sh_entry_lock (frame, this);
 	} else {
 		gf_log (this->name, GF_LOG_TRACE,
