@@ -45,7 +45,13 @@ typedef struct _afr_private {
 
         char **pending_key;
 
-	gf_boolean_t data_self_heal;       /* on/off */
+	gf_boolean_t data_self_heal;              /* on/off */
+        char *       data_self_heal_algorithm;    /* name of algorithm */
+        unsigned int data_self_heal_window_size;  /* max number of pipelined
+                                                     read/writes */
+
+        unsigned int background_self_heal_count;
+        unsigned int background_self_heals_started;
 	gf_boolean_t metadata_self_heal;   /* on/off */
 	gf_boolean_t entry_self_heal;      /* on/off */
 
@@ -62,11 +68,47 @@ typedef struct _afr_private {
 	unsigned int entry_lock_server_count;
 
 	unsigned int wait_count;      /* # of servers to wait for success */
+
+        uint64_t up_count;      /* number of CHILD_UPs we have seen */
+        uint64_t down_count;    /* number of CHILD_DOWNs we have seen */
 } afr_private_t;
 
 typedef struct {
+        /* External interface: These are variables (some optional) that
+           are set by whoever has triggered self-heal */
+
+        gf_boolean_t need_data_self_heal;
+        gf_boolean_t need_metadata_self_heal;
+        gf_boolean_t need_entry_self_heal;
+
+        gf_boolean_t forced_merge;        /* Is this a self-heal triggered to
+                                             forcibly merge the directories? */
+
+        gf_boolean_t healing_fd_opened;   /* true if caller has already
+                                             opened fd */
+
+        gf_boolean_t data_lock_held;      /* true if caller has already
+                                             acquired 0-0 lock */
+
+	fd_t *healing_fd;                 /* set if callers has opened fd */
+
+        gf_boolean_t background;          /* do self-heal in background
+                                             if possible */
+
+        mode_t mode;                      /* st_mode of the entry we're doing
+                                             self-heal on */
+
+        /* Function to call to unwind. If self-heal is being done in the
+           background, this function will be called as soon as possible. */
+
+        int (*unwind) (call_frame_t *frame, xlator_t *this);
+
+        /* End of external interface members */
+
+
 	/* array of stat's, one for each child */
 	struct stat *buf;
+        struct stat parentbuf;
 
 	/* array of xattr's, one for each child */
 	dict_t **xattr;
@@ -83,8 +125,11 @@ typedef struct {
 	int active_sinks;
 	int *success;
 	int *locked_nodes;
+        int lock_count;
+
+        mode_t impunging_entry_mode;
+        const char *linkname;
 
-	fd_t *healing_fd;
 	int   op_failed;
 
 	int   file_has_holes;
@@ -93,7 +138,19 @@ typedef struct {
 	off_t offset;
 
 	loc_t parent_loc;
+
+        call_frame_t *orig_frame;
+        gf_boolean_t unwound;
+
+        /* private data for the particular self-heal algorithm */
+        void *private;
+
+        int (*flush_self_heal_cbk) (call_frame_t *frame, xlator_t *this);
+
 	int (*completion_cbk) (call_frame_t *frame, xlator_t *this);
+        int (*algo_completion_cbk) (call_frame_t *frame, xlator_t *this);
+        int (*algo_abort_cbk) (call_frame_t *frame, xlator_t *this);
+
 	call_frame_t *sh_frame;
 } afr_self_heal_t;
 
@@ -135,14 +192,17 @@ afr_index_for_transaction_type (afr_transaction_type type)
 }
 
 
+typedef enum {
+        AFR_CHILD_UP_FLUSH,
+        AFR_CHILD_DOWN_FLUSH,
+} afr_flush_type;
+
+
 typedef struct _afr_local {
 	unsigned int call_count;
 	unsigned int success_count;
 	unsigned int enoent_count;
 
-	unsigned int need_metadata_self_heal;
-	unsigned int need_entry_self_heal;
-	unsigned int need_data_self_heal;
 	unsigned int govinda_gOvinda;
 
 	unsigned int read_child_index;
@@ -164,15 +224,17 @@ typedef struct _afr_local {
 	glusterfs_fop_t fop;
 
 	unsigned char *child_up; 
-	int            child_count;
 
 	int32_t *child_errno;
 	
 	dict_t  *xattr_req;
 	int      open_fd_count;
+
 	int32_t  inodelk_count;
 	int32_t  entrylk_count;
 
+        int (*up_down_flush_cbk) (call_frame_t *, xlator_t *);
+
 	/* 
 	   This struct contains the arguments for the "continuation"
 	   (scheme-like) of fops
@@ -189,12 +251,18 @@ typedef struct _afr_local {
 			inode_t *inode;
 			struct stat buf;
                         struct stat read_child_buf;
+                        struct stat postparent;
                         ino_t ino;
+                        uint64_t gen;
+                        ino_t parent_ino;
 			dict_t *xattr;
+			dict_t **xattrs;
+                        gf_boolean_t is_revalidate;
 		} lookup;
 
 		struct {
 			int32_t flags;
+                        int32_t wbflags;
 		} open;
 
 		struct {
@@ -228,6 +296,7 @@ typedef struct _afr_local {
 		struct {
 			size_t size;
 			int last_tried;
+                        ino_t ino;
 		} readlink;
 
 		struct {
@@ -248,6 +317,8 @@ typedef struct _afr_local {
 			int success_count;
 			int32_t op_ret;
 			int32_t op_errno;
+
+                        uint32_t *checksum;
 		} opendir;
 
 		struct {
@@ -274,38 +345,8 @@ typedef struct _afr_local {
 
 		struct {
 			ino_t ino;
-			mode_t mode;
-			struct stat buf;
-			struct stat read_child_buf;
-		} chmod;
-
-		struct {
-			ino_t ino;
-			mode_t mode;
-			struct stat buf;
-                        struct stat read_child_buf;
-		} fchmod;
-
-		struct {
-			ino_t ino;
-			uid_t uid;
-			gid_t gid;
-			struct stat buf;
-                        struct stat read_child_buf;
-		} chown;
-
-		struct {
-			ino_t ino;
-			uid_t uid;
-			gid_t gid;
-			struct stat buf;
-                        struct stat read_child_buf;
-		} fchown;
-		
-		struct {
-			ino_t ino;
-			struct stat buf;
-			struct stat read_child_buf;
+			struct stat prebuf;
+			struct stat postbuf;
 
 			int32_t op_ret;
 
@@ -315,26 +356,41 @@ typedef struct _afr_local {
 			off_t offset;
 		} writev;
 
+                struct {
+                        ino_t ino;
+                        struct stat prebuf;
+                        struct stat postbuf;
+                } fsync;
+
 		struct {
 			ino_t ino;
 			off_t offset;
-			struct stat buf;
-                        struct stat read_child_buf;
+			struct stat prebuf;
+                        struct stat postbuf;
 		} truncate;
 
 		struct {
 			ino_t ino;
 			off_t offset;
-			struct stat buf;
-                        struct stat read_child_buf;
+			struct stat prebuf;
+                        struct stat postbuf;
 		} ftruncate;
 
 		struct {
 			ino_t ino;
-			struct timespec tv[2];
-			struct stat buf;
-                        struct stat read_child_buf;
-		} utimens;
+			struct stat in_buf;
+                        int32_t valid;
+                        struct stat preop_buf;
+                        struct stat postop_buf;
+		} setattr;
+
+		struct {
+			ino_t ino;
+			struct stat in_buf;
+                        int32_t valid;
+                        struct stat preop_buf;
+                        struct stat postop_buf;
+		} fsetattr;
 
 		struct {
 			dict_t *dict;
@@ -349,60 +405,92 @@ typedef struct _afr_local {
 		
 		struct {
 			ino_t ino;
+                        uint64_t gen;
+                        ino_t parent_ino;
 			fd_t *fd;
 			int32_t flags;
 			mode_t mode;
 			inode_t *inode;
 			struct stat buf;
+                        struct stat preparent;
+                        struct stat postparent;
                         struct stat read_child_buf;
 		} create;
 
 		struct {
 			ino_t ino;
+                        uint64_t gen;
+                        ino_t parent_ino;
 			dev_t dev;
 			mode_t mode;
 			inode_t *inode;
 			struct stat buf;
+                        struct stat preparent;
+                        struct stat postparent;
                         struct stat read_child_buf;
 		} mknod;
 
 		struct {
 			ino_t ino;
+                        uint64_t gen;
+                        ino_t parent_ino;
 			int32_t mode;
 			inode_t *inode;
 			struct stat buf;
                         struct stat read_child_buf;
+                        struct stat preparent;
+                        struct stat postparent;
 		} mkdir;
 
 		struct {
+                        ino_t parent_ino;
 			int32_t op_ret;
 			int32_t op_errno;
+                        struct stat preparent;
+                        struct stat postparent;
 		} unlink;
 
 		struct {
+                        ino_t parent_ino;
 			int32_t op_ret;
 			int32_t op_errno;
+                        struct stat preparent;
+                        struct stat postparent;
 		} rmdir;
 
 		struct {
+                        ino_t oldparent_ino;
+                        ino_t newparent_ino;
 			ino_t ino;
 			struct stat buf;
                         struct stat read_child_buf;
+                        struct stat preoldparent;
+                        struct stat prenewparent;
+                        struct stat postoldparent;
+                        struct stat postnewparent;
 		} rename;
 
 		struct {
 			ino_t ino;
+                        uint64_t gen;
+                        ino_t parent_ino;
 			inode_t *inode;
 			struct stat buf;
                         struct stat read_child_buf;
+                        struct stat preparent;
+                        struct stat postparent;
 		} link;
 
 		struct {
 			ino_t ino;
+                        uint64_t gen;
+                        ino_t parent_ino;
 			inode_t *inode;
 			struct stat buf;
                         struct stat read_child_buf;
 			char *linkpath;
+                        struct stat preparent;
+                        struct stat postparent;
 		} symlink;
 
 		struct {
@@ -442,6 +530,9 @@ typedef struct _afr_local {
 		int (*resume) (call_frame_t *frame, xlator_t *this);
 
 		int (*unwind) (call_frame_t *frame, xlator_t *this);
+
+                /* post-op hook */
+                int (*post_post_op) (call_frame_t *frame, xlator_t *this);
 	} transaction;
 
 	afr_self_heal_t self_heal;
@@ -449,8 +540,13 @@ typedef struct _afr_local {
 
 
 typedef struct {
-        unsigned char pre_op_done;
+        unsigned char *pre_op_done;
+        unsigned char *opened_on;     /* which subvolumes the fd is open on */
         unsigned char *child_failed;
+        int flags;
+        int32_t wbflags;
+        uint64_t up_count;   /* number of CHILD_UPs this fd has seen */
+        uint64_t down_count; /* number of CHILD_DOWNs this fd has seen */
 } afr_fd_ctx_t;
 
 
@@ -506,14 +602,36 @@ afr_local_cleanup (afr_local_t *local, xlator_t *this);
 int
 afr_frame_return (call_frame_t *frame);
 
-#define AFR_STACK_UNWIND(frame, params ...)		\
+uint64_t
+afr_is_split_brain (xlator_t *this, inode_t *inode);
+
+void
+afr_set_split_brain (xlator_t *this, inode_t *inode);
+
+int
+afr_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
+          fd_t *fd, int32_t wbflags);
+
+int
+afr_up_down_flush (call_frame_t *frame, xlator_t *this, fd_t *fd, afr_flush_type type);
+
+void
+afr_set_opendir_done (xlator_t *this, inode_t *inode);
+
+uint64_t
+afr_is_opendir_done (xlator_t *this, inode_t *inode);
+
+void
+afr_local_transaction_cleanup (afr_local_t *local, xlator_t *this);
+
+#define AFR_STACK_UNWIND(fop, frame, params ...)        \
 	do {						\
 		afr_local_t *__local = NULL;		\
 		xlator_t    *__this = NULL;		\
 		__local = frame->local;			\
 		__this = frame->this;			\
 		frame->local = NULL;                    \
-		STACK_UNWIND (frame, params);		\
+		STACK_UNWIND_STRICT (fop, frame, params);       \
 		afr_local_cleanup (__local, __this);	\
 		free (__local);				\
 } while (0);					
