@@ -36,6 +36,7 @@ struct dht_layout {
 	int               preset;
         int               gen;
 	int               type;
+        int               ref;   /* use with dht_conf_t->layout_lock */
         struct {
 		int       err;   /* 0 = normal
 				   -1 = dir exists and no xattr
@@ -49,6 +50,11 @@ struct dht_layout {
 typedef struct dht_layout dht_layout_t;
 
 
+typedef enum {
+	DHT_HASH_TYPE_DM,
+} dht_hashfn_type_t;
+
+
 struct dht_local {
 	int                      call_cnt;
 	loc_t                    loc;
@@ -56,7 +62,14 @@ struct dht_local {
 	int                      op_ret;
 	int                      op_errno;
 	int                      layout_mismatch;
+        /* Use stbuf as the postbuf, when we require both
+         * pre and post attrs */
 	struct stat              stbuf;
+        struct stat              prebuf;
+        struct stat              preoldparent;
+        struct stat              postoldparent; 
+        struct stat              preparent;
+        struct stat              postparent;
 	struct statvfs           statvfs;
 	fd_t                    *fd;
 	inode_t                 *inode;
@@ -65,6 +78,7 @@ struct dht_local {
 	dht_layout_t            *layout;
 	size_t                   size;
 	ino_t                    st_ino;
+        ino_t                    st_dev;
 	xlator_t                *src_hashed, *src_cached;
 	xlator_t                *dst_hashed, *dst_cached;
 	xlator_t                *cached_subvol;
@@ -109,7 +123,6 @@ struct dht_conf {
 	gf_lock_t      subvolume_lock;
         int            subvolume_cnt;
         xlator_t     **subvolumes;
-	xlator_t      *local_volume;     /* Needed by NUFA */
 	char          *subvolume_status;
 	dht_layout_t **file_layouts;
 	dht_layout_t **dir_layouts;
@@ -117,10 +130,14 @@ struct dht_conf {
 	gf_boolean_t   search_unhashed;
 	int            gen;
         dht_du_t      *du_stats;
-        uint32_t       min_free_disk;
+        uint64_t       min_free_disk;
+        char           disk_unit;
         int32_t        refresh_interval;
         gf_boolean_t   unhashed_sticky_bit;
 	struct timeval last_stat_fetch;
+        gf_lock_t      layout_lock;
+        void          *private;     /* Can be used by wrapper xlators over
+                                       dht */
 };
 typedef struct dht_conf dht_conf_t;
 
@@ -144,26 +161,32 @@ typedef struct dht_disk_layout dht_disk_layout_t;
 #define is_last_call(cnt) (cnt == 0)
 
 #define DHT_LINKFILE_MODE (S_ISVTX)
-#define check_is_linkfile(i,s,x) ((s->st_mode & ~S_IFMT) == DHT_LINKFILE_MODE)
+#define check_is_linkfile(i,s,x) (                              \
+                ((s->st_mode & ~S_IFMT) == DHT_LINKFILE_MODE) && \
+                (s->st_size == 0))
 
 #define check_is_dir(i,s,x) (S_ISDIR(s->st_mode))
 
 #define layout_is_sane(layout) ((layout) && (layout->cnt > 0))
 
-#define DHT_STACK_UNWIND(frame, params ...) do {       \
-		dht_local_t *__local = NULL;           \
-		__local = frame->local;                \
-		frame->local = NULL;		       \
-		STACK_UNWIND (frame, params);          \
-		dht_local_wipe (__local);	       \
+#define DHT_STACK_UNWIND(fop, frame, params ...) do {           \
+		dht_local_t *__local = NULL;                    \
+                xlator_t *__xl = NULL;                          \
+                __xl = frame->this;                             \
+		__local = frame->local;                         \
+		frame->local = NULL;                            \
+		STACK_UNWIND_STRICT (fop, frame, params);       \
+		dht_local_wipe (__xl, __local);                 \
 	} while (0)
 
 #define DHT_STACK_DESTROY(frame) do {		       \
 		dht_local_t *__local = NULL;           \
+                xlator_t *__xl = NULL;                 \
+                __xl = frame->this;                    \
 		__local = frame->local;                \
 		frame->local = NULL;		       \
 		STACK_DESTROY (frame->root);	       \
-		dht_local_wipe (__local);	       \
+		dht_local_wipe (__xl, __local);	       \
 	} while (0)
 
 dht_layout_t *dht_layout_new (xlator_t *this, int cnt);
@@ -200,7 +223,7 @@ int dht_itransform (xlator_t *this, xlator_t *subvol, uint64_t x, uint64_t *y);
 int dht_deitransform (xlator_t *this, uint64_t y, xlator_t **subvol,
 		      uint64_t *x);
 
-void dht_local_wipe (dht_local_t *local);
+void dht_local_wipe (xlator_t *this, dht_local_t *local);
 dht_local_t *dht_local_init (call_frame_t *frame);
 int dht_stat_merge (xlator_t *this, struct stat *to, struct stat *from,
 		    xlator_t *subvol);
@@ -237,7 +260,10 @@ int dht_is_subvol_filled (xlator_t *this, xlator_t *subvol);
 xlator_t *dht_free_disk_available_subvol (xlator_t *this, xlator_t *subvol);
 int dht_get_du_info_for_subvol (xlator_t *this, int subvol_idx);
 
-int dht_layout_inode_set (xlator_t *this, xlator_t *subvol, inode_t *inode);
+int dht_layout_preset (xlator_t *this, xlator_t *subvol, inode_t *inode);
+int dht_layout_set (xlator_t *this, inode_t *inode, dht_layout_t *layout);
+void dht_layout_unref (xlator_t *this, dht_layout_t *layout);
+dht_layout_t *dht_layout_ref (xlator_t *this, dht_layout_t *layout);
 xlator_t *dht_first_up_subvol (xlator_t *this);
 
 #endif /* _DHT_H */
