@@ -196,7 +196,8 @@ dht_selfheal_dir_xattr (call_frame_t *frame, loc_t *loc, dht_layout_t *layout)
 int
 dht_selfheal_dir_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			    int op_ret, int op_errno,
-			    inode_t *inode, struct stat *stbuf)
+                            inode_t *inode, struct stat *stbuf,
+                            struct stat *preparent, struct stat *postparent)
 {
 	dht_local_t   *local = NULL;
 	dht_layout_t  *layout = NULL;
@@ -211,6 +212,13 @@ dht_selfheal_dir_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	prev   = cookie;
 	subvol = prev->this;
 
+        dht_stat_merge (this, &local->stbuf, stbuf, prev->this);
+        if (prev->this == local->hashed_subvol)
+                local->st_ino = local->stbuf.st_ino;
+
+        dht_stat_merge (this, &local->preparent, preparent, prev->this);
+        dht_stat_merge (this, &local->postparent, postparent, prev->this);
+
 	if ((op_ret == 0) || (op_errno == EEXIST)) {
 		for (i = 0; i < layout->cnt; i++) {
 			if (layout->list[i].xlator == subvol) {
@@ -431,7 +439,7 @@ dht_selfheal_new_directory (call_frame_t *frame,
 	local = frame->local;
 
 	local->selfheal.dir_cbk = dir_cbk;
-	local->selfheal.layout = layout;
+	local->selfheal.layout = dht_layout_ref (frame->this, layout);
 
 	dht_layout_sort_volname (layout);
 	dht_selfheal_layout_new_directory (frame, &local->loc, layout);	
@@ -470,7 +478,7 @@ dht_selfheal_directory (call_frame_t *frame, dht_selfheal_dir_cbk_t dir_cbk,
 	misc     = local->selfheal.misc;
 
 	local->selfheal.dir_cbk = dir_cbk;
-	local->selfheal.layout = layout;
+	local->selfheal.layout = dht_layout_ref (this, layout);
 
 	if (down) {
 		gf_log (this->name, GF_LOG_DEBUG,
@@ -518,7 +526,7 @@ dht_selfheal_restore (call_frame_t *frame, dht_selfheal_dir_cbk_t dir_cbk,
 	local = frame->local;
 
 	local->selfheal.dir_cbk = dir_cbk;
-	local->selfheal.layout = layout;
+	local->selfheal.layout = dht_layout_ref (frame->this, layout);
 
 	ret = dht_selfheal_dir_mkdir (frame, loc, layout, 1);
 
