@@ -27,12 +27,12 @@
 
 /* TODO: all 'TODO's in dht.c holds good */
 
-int 
+int
 nufa_local_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		       int op_ret, int op_errno,
-		       inode_t *inode, struct stat *stbuf, dict_t *xattr)
+                       inode_t *inode, struct stat *stbuf, dict_t *xattr,
+                       struct stat *postparent)
 {
-	dht_layout_t *layout      = NULL;
         xlator_t     *subvol      = NULL;
         char          is_linkfile = 0;
         char          is_dir      = 0;
@@ -42,6 +42,7 @@ nufa_local_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         int           i           = 0;
         call_frame_t *prev        = NULL;
 	int           call_cnt    = 0;
+        int           ret         = 0;
 
 
         conf  = this->private;
@@ -70,17 +71,16 @@ nufa_local_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		dht_itransform (this, prev->this, stbuf->st_ino,
 				&stbuf->st_ino);
 
-		layout = dht_layout_for_subvol (this, prev->this);
-		if (!layout) {
+		ret = dht_layout_preset (this, prev->this, inode);
+		if (ret < 0) {
 			gf_log (this->name, GF_LOG_DEBUG,
-				"no pre-set layout for subvolume %s",
+				"could not set pre-set layout for subvol %s",
 				prev->this->name);
 			op_ret   = -1;
 			op_errno = EINVAL;
 			goto err;
 		}
 
-                inode_ctx_put (inode, this, (uint64_t)(long)layout);
                 goto out;
         }
 
@@ -137,7 +137,7 @@ out:
 		op_errno = EINVAL;
 		goto err;
 	}
-		
+
 	STACK_WIND (frame, dht_lookup_cbk,
 		    local->hashed_subvol, local->hashed_subvol->fops->lookup,
 		    &local->loc, local->xattr_req);
@@ -145,7 +145,8 @@ out:
 	return 0;
 
  err:
-        DHT_STACK_UNWIND (frame, op_ret, op_errno, inode, stbuf, xattr);
+        DHT_STACK_UNWIND (lookup, frame, op_ret, op_errno,
+                          inode, stbuf, xattr, NULL);
         return 0;
 }
 
@@ -198,12 +199,12 @@ nufa_lookup (call_frame_t *frame, xlator_t *this,
 
 	hashed_subvol = dht_subvol_get_hashed (this, &local->loc);
 	cached_subvol = dht_subvol_get_cached (this, local->loc.inode);
-	
+
 	local->cached_subvol = cached_subvol;
 	local->hashed_subvol = hashed_subvol;
 
         if (is_revalidate (loc)) {
-		layout = dht_layout_get (this, loc->inode);
+		local->layout = layout = dht_layout_get (this, loc->inode);
 
                 if (!layout) {
                         gf_log (this->name, GF_LOG_DEBUG,
@@ -217,8 +218,8 @@ nufa_lookup (call_frame_t *frame, xlator_t *this,
 			gf_log (this->name, GF_LOG_DEBUG,
 				"incomplete layout failure for path=%s",
 				loc->path);
-			op_errno = ESTALE;
-			goto err;
+                        dht_layout_unref (this, local->layout);
+			goto do_fresh_lookup;
 		}
 
 		local->inode    = inode_ref (loc->inode);
@@ -226,16 +227,16 @@ nufa_lookup (call_frame_t *frame, xlator_t *this,
 
 		local->call_cnt = layout->cnt;
 		call_cnt = local->call_cnt;
-		
+
 		/* NOTE: we don't require 'trusted.glusterfs.dht.linkto' attribute,
 		 *       revalidates directly go to the cached-subvolume.
 		 */
-		ret = dict_set_uint32 (local->xattr_req, 
+		ret = dict_set_uint32 (local->xattr_req,
 				       "trusted.glusterfs.dht", 4 * 4);
 
 		for (i = 0; i < layout->cnt; i++) {
 			subvol = layout->list[i].xlator;
-			
+
 			STACK_WIND (frame, dht_revalidate_cbk,
 				    subvol, subvol->fops->lookup,
 				    loc, local->xattr_req);
@@ -244,16 +245,17 @@ nufa_lookup (call_frame_t *frame, xlator_t *this,
 				break;
 		}
 	} else {
-		ret = dict_set_uint32 (local->xattr_req, 
+        do_fresh_lookup:
+		ret = dict_set_uint32 (local->xattr_req,
 				       "trusted.glusterfs.dht", 4 * 4);
 
-		ret = dict_set_uint32 (local->xattr_req, 
+		ret = dict_set_uint32 (local->xattr_req,
 				       "trusted.glusterfs.dht.linkto", 256);
 
 		/* Send it to only local volume */
 		STACK_WIND (frame, nufa_local_lookup_cbk,
-			    conf->local_volume, 
-			    conf->local_volume->fops->lookup,
+			    (xlator_t *)conf->private,
+			    ((xlator_t *)conf->private)->fops->lookup,
 			    loc, local->xattr_req);
 	}
 
@@ -261,42 +263,45 @@ nufa_lookup (call_frame_t *frame, xlator_t *this,
 
 err:
 	op_errno = (op_errno == -1) ? errno : op_errno;
-        DHT_STACK_UNWIND (frame, -1, op_errno, NULL, NULL, NULL);
+        DHT_STACK_UNWIND (lookup, frame, -1, op_errno, NULL, NULL, NULL, NULL);
 	return 0;
 }
 
 int
-nufa_create_linkfile_create_cbk (call_frame_t *frame, void *cookie, 
+nufa_create_linkfile_create_cbk (call_frame_t *frame, void *cookie,
 				 xlator_t *this, int op_ret, int op_errno,
-				 inode_t *inode, struct stat *stbuf)
+                                 inode_t *inode, struct stat *stbuf,
+                                 struct stat *preparent,
+                                 struct stat *postparent)
 {
- 	dht_local_t  *local = NULL;
- 	call_frame_t *prev = NULL;
+	dht_local_t  *local = NULL;
+	call_frame_t *prev = NULL;
 	dht_conf_t   *conf  = NULL;
-	
- 	local = frame->local;
- 	prev  = cookie;
- 	conf  = this->private;
-	
- 	if (op_ret == -1)
- 		goto err;
-	
- 	STACK_WIND (frame, dht_create_cbk,
- 		    local->cached_subvol, local->cached_subvol->fops->create,
- 		    &local->loc, local->flags, local->mode, local->fd);
-	
- 	return 0;
-	
+
+	local = frame->local;
+	prev  = cookie;
+	conf  = this->private;
+
+	if (op_ret == -1)
+		goto err;
+
+	STACK_WIND (frame, dht_create_cbk,
+		    local->cached_subvol, local->cached_subvol->fops->create,
+		    &local->loc, local->flags, local->mode, local->fd);
+
+	return 0;
+
  err:
- 	DHT_STACK_UNWIND (frame, -1, op_errno, NULL, NULL, NULL);	
- 	return 0;
+	DHT_STACK_UNWIND (create, frame, -1, op_errno,
+                          NULL, NULL, NULL, NULL, NULL);
+	return 0;
 }
 
 int
 nufa_create (call_frame_t *frame, xlator_t *this,
 	     loc_t *loc, int32_t flags, mode_t mode, fd_t *fd)
 {
- 	dht_local_t *local = NULL;
+	dht_local_t *local = NULL;
 	dht_conf_t  *conf  = NULL;
 	xlator_t    *subvol = NULL;
         xlator_t    *avail_subvol = NULL;
@@ -307,7 +312,7 @@ nufa_create (call_frame_t *frame, xlator_t *this,
 	VALIDATE_OR_GOTO (this, err);
 	VALIDATE_OR_GOTO (loc, err);
 
- 	conf  = this->private; 	
+	conf  = this->private;
 
         dht_get_du_info (frame, this, loc);
 
@@ -328,13 +333,13 @@ nufa_create (call_frame_t *frame, xlator_t *this,
 		goto err;
 	}
 
-        avail_subvol = conf->local_volume;
-        if (dht_is_subvol_filled (this, conf->local_volume)) {
-                avail_subvol = 
-                        dht_free_disk_available_subvol (this, 
-                                                        conf->local_volume);
+        avail_subvol = conf->private;
+        if (dht_is_subvol_filled (this, (xlator_t *)conf->private)) {
+                avail_subvol =
+                        dht_free_disk_available_subvol (this,
+                                                        (xlator_t *)conf->private);
         }
-                
+
         if (subvol != avail_subvol) {
                 /* create a link file instead of actual file */
                 ret = loc_copy (&local->loc, loc);
@@ -344,13 +349,13 @@ nufa_create (call_frame_t *frame, xlator_t *this,
                                 op_errno = ENOMEM;
                                 goto err;
                 }
-                
+
                 local->fd = fd_ref (fd);
                 local->mode = mode;
                 local->flags = flags;
 
                 local->cached_subvol = avail_subvol;
-                dht_linkfile_create (frame, 
+                dht_linkfile_create (frame,
                                      nufa_create_linkfile_create_cbk,
                                      avail_subvol, subvol, loc);
                 return 0;
@@ -358,44 +363,47 @@ nufa_create (call_frame_t *frame, xlator_t *this,
 
         gf_log (this->name, GF_LOG_TRACE,
                 "creating %s on %s", loc->path, subvol->name);
-        
+
         STACK_WIND (frame, dht_create_cbk,
                     subvol, subvol->fops->create,
                     loc, flags, mode, fd);
-        
+
         return 0;
 
 err:
 	op_errno = (op_errno == -1) ? errno : op_errno;
-	DHT_STACK_UNWIND (frame, -1, op_errno, NULL, NULL, NULL);
+	DHT_STACK_UNWIND (create, frame, -1, op_errno,
+                          NULL, NULL, NULL, NULL, NULL);
 
 	return 0;
 }
 
 int
 nufa_mknod_linkfile_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-			 int op_ret, int op_errno,
-			 inode_t *inode, struct stat *stbuf)
+                         int op_ret, int op_errno, inode_t *inode,
+                         struct stat *stbuf, struct stat *preparent,
+                         struct stat *postparent)
 {
- 	dht_local_t  *local = NULL;
- 	call_frame_t *prev = NULL;
+	dht_local_t  *local = NULL;
+	call_frame_t *prev = NULL;
 	dht_conf_t   *conf  = NULL;
-	
- 	local = frame->local;
- 	prev  = cookie;
- 	conf  = this->private;
- 	
- 	if (op_ret >= 0) {
- 		STACK_WIND (frame, dht_newfile_cbk,
- 			    local->cached_subvol, 
- 			    local->cached_subvol->fops->mknod,
- 			    &local->loc, local->mode, local->rdev);
-		
- 		return 0;
- 	}
-	
- 	DHT_STACK_UNWIND (frame, op_ret, op_errno, inode, stbuf);
- 	return 0;
+
+	local = frame->local;
+	prev  = cookie;
+	conf  = this->private;
+
+	if (op_ret >= 0) {
+		STACK_WIND (frame, dht_newfile_cbk,
+			    local->cached_subvol,
+			    local->cached_subvol->fops->mknod,
+			    &local->loc, local->mode, local->rdev);
+
+		return 0;
+	}
+
+	DHT_STACK_UNWIND (link, frame, op_ret, op_errno,
+                          inode, stbuf, preparent, postparent);
+	return 0;
 }
 
 
@@ -403,7 +411,7 @@ int
 nufa_mknod (call_frame_t *frame, xlator_t *this,
 	    loc_t *loc, mode_t mode, dev_t rdev)
 {
- 	dht_local_t *local = NULL;
+	dht_local_t *local = NULL;
 	dht_conf_t  *conf  = NULL;
 	xlator_t    *subvol = NULL;
         xlator_t    *avail_subvol = NULL;
@@ -414,7 +422,7 @@ nufa_mknod (call_frame_t *frame, xlator_t *this,
 	VALIDATE_OR_GOTO (this, err);
 	VALIDATE_OR_GOTO (loc, err);
 
- 	conf  = this->private; 	
+	conf  = this->private;
 
         dht_get_du_info (frame, this, loc);
 
@@ -436,31 +444,31 @@ nufa_mknod (call_frame_t *frame, xlator_t *this,
 	}
 
         /* Consider the disksize in consideration */
-        avail_subvol = conf->local_volume;
-        if (dht_is_subvol_filled (this, conf->local_volume)) {
-                avail_subvol = 
-                        dht_free_disk_available_subvol (this, 
-                                                        conf->local_volume);
+        avail_subvol = conf->private;
+        if (dht_is_subvol_filled (this, (xlator_t *)conf->private)) {
+                avail_subvol =
+                        dht_free_disk_available_subvol (this,
+                                                        (xlator_t *)conf->private);
         }
 
- 	if (avail_subvol != subvol) {
- 		/* Create linkfile first */
- 		ret = loc_copy (&local->loc, loc);
- 		if (ret == -1) {
- 			gf_log (this->name, GF_LOG_ERROR,
- 				"Out of memory");
- 			op_errno = ENOMEM;
- 			goto err;
- 		}
- 
-		local->mode = mode;
- 		local->rdev = rdev;
- 		local->cached_subvol = avail_subvol;
+	if (avail_subvol != subvol) {
+		/* Create linkfile first */
+		ret = loc_copy (&local->loc, loc);
+		if (ret == -1) {
+			gf_log (this->name, GF_LOG_ERROR,
+				"Out of memory");
+			op_errno = ENOMEM;
+			goto err;
+		}
 
- 		dht_linkfile_create (frame, nufa_mknod_linkfile_cbk,
+		local->mode = mode;
+		local->rdev = rdev;
+		local->cached_subvol = avail_subvol;
+
+		dht_linkfile_create (frame, nufa_mknod_linkfile_cbk,
                                      avail_subvol, subvol, loc);
- 		return 0;
- 	}
+		return 0;
+	}
 
 	gf_log (this->name, GF_LOG_TRACE,
 		"creating %s on %s", loc->path, subvol->name);
@@ -473,7 +481,8 @@ nufa_mknod (call_frame_t *frame, xlator_t *this,
 
 err:
 	op_errno = (op_errno == -1) ? errno : op_errno;
-	DHT_STACK_UNWIND (frame, -1, op_errno, NULL, NULL);
+	DHT_STACK_UNWIND (mknod, frame, -1, op_errno,
+                          NULL, NULL, NULL, NULL);
 
 	return 0;
 }
@@ -527,17 +536,18 @@ init (xlator_t *this)
 	xlator_list_t *trav = NULL;
 	data_t        *data = NULL;
 	char          *local_volname = NULL;
-	char          *lookup_unhashed_str = NULL;
+	char          *temp_str = NULL;
         int            ret = -1;
         int            i = 0;
 	char           my_hostname[256];
+	uint32_t       temp_free_disk = 0;
 
 	if (!this->children) {
 		gf_log (this->name, GF_LOG_CRITICAL,
 			"NUFA needs more than one subvolume");
 		return -1;
 	}
-  
+
 	if (!this->parents) {
 		gf_log (this->name, GF_LOG_WARNING,
 			"dangling volume. check volfile");
@@ -553,8 +563,8 @@ init (xlator_t *this)
 	conf->search_unhashed = 0;
 
 	if (dict_get_str (this->options, "lookup-unhashed",
-			  &lookup_unhashed_str) == 0) {
-		gf_string2boolean (lookup_unhashed_str,
+			  &temp_str) == 0) {
+		gf_string2boolean (temp_str,
 				   &conf->search_unhashed);
 	}
 
@@ -569,6 +579,7 @@ init (xlator_t *this)
         }
 
 	LOCK_INIT (&conf->subvolume_lock);
+	LOCK_INIT (&conf->layout_lock);
 
 	conf->gen = 1;
 
@@ -596,7 +607,7 @@ init (xlator_t *this)
 	}
 
 	if (!trav) {
-		gf_log (this->name, GF_LOG_ERROR, 
+		gf_log (this->name, GF_LOG_ERROR,
 			"Could not find subvolume named '%s'. "
 			"Please define volume with the name as the hostname "
 			"or override it with 'option local-volume-name'",
@@ -604,14 +615,29 @@ init (xlator_t *this)
 		goto err;
 	}
 	/* The volume specified exists */
-	conf->local_volume = trav->xlator;
+	conf->private = trav->xlator;
 
         conf->min_free_disk = 10;
+        conf->disk_unit = 'p';
 
-	data = dict_get (this->options, "min-free-disk");
-	if (data) {
-		gf_string2percent (data->data, &conf->min_free_disk);
-	}
+        if (dict_get_str (this->options, "min-free-disk",
+                          &temp_str) == 0) {
+                if (gf_string2percent (temp_str,
+                                       &temp_free_disk) == 0) {
+                        if (temp_free_disk > 100) {
+                                gf_string2bytesize (temp_str,
+                                                        &conf->min_free_disk);
+                                conf->disk_unit = 'b';
+                        } else {
+                                conf->min_free_disk = (uint64_t)temp_free_disk;
+                                conf->disk_unit = 'p';
+                        }
+                } else {
+                        gf_string2bytesize (temp_str,
+                                                &conf->min_free_disk);
+                        conf->disk_unit = 'b';
+                }
+        }
 
         conf->du_stats = CALLOC (conf->subvolume_cnt, sizeof (dht_du_t));
         if (!conf->du_stats) {
@@ -658,12 +684,7 @@ struct xlator_fops fops = {
 	.mknod       = nufa_mknod,
 
 	.stat        = dht_stat,
-	.chmod       = dht_chmod,
-	.chown       = dht_chown,
-	.fchown      = dht_fchown,
-	.fchmod      = dht_fchmod,
 	.fstat       = dht_fstat,
-	.utimens     = dht_utimens,
 	.truncate    = dht_truncate,
 	.ftruncate   = dht_ftruncate,
 	.access      = dht_access,
@@ -680,6 +701,7 @@ struct xlator_fops fops = {
 	.lk          = dht_lk,
 	.opendir     = dht_opendir,
 	.readdir     = dht_readdir,
+	.readdirp    = dht_readdirp,
 	.fsyncdir    = dht_fsyncdir,
 	.symlink     = dht_symlink,
 	.unlink      = dht_unlink,
@@ -693,6 +715,7 @@ struct xlator_fops fops = {
 	.fentrylk    = dht_fentrylk,
 	.xattrop     = dht_xattrop,
 	.fxattrop    = dht_fxattrop,
+        .setattr     = dht_setattr,
 #if 0
 	.setdents    = dht_setdents,
 	.getdents    = dht_getdents,
@@ -706,21 +729,19 @@ struct xlator_mops mops = {
 
 
 struct xlator_cbks cbks = {
-//	.release    = dht_release,
-//      .releasedir = dht_releasedir,
 	.forget     = dht_forget
 };
 
 
 struct volume_options options[] = {
-	{ .key  = {"local-volume-name"}, 
-	  .type = GF_OPTION_TYPE_XLATOR 
+	{ .key  = {"local-volume-name"},
+	  .type = GF_OPTION_TYPE_XLATOR
 	},
-        { .key  = {"lookup-unhashed"}, 
-	  .type = GF_OPTION_TYPE_BOOL 
+        { .key  = {"lookup-unhashed"},
+	  .type = GF_OPTION_TYPE_BOOL
 	},
         { .key  = {"min-free-disk"},
-          .type = GF_OPTION_TYPE_PERCENT
+          .type = GF_OPTION_TYPE_PERCENT_OR_SIZET,
         },
 	{ .key  = {NULL} },
 };
