@@ -30,12 +30,25 @@
 #include <assert.h>
 #include <sys/time.h>
 
+int ioc_log2_page_size;
+
 uint32_t
 ioc_get_priority (ioc_table_t *table, const char *path);
 
 uint32_t
 ioc_get_priority (ioc_table_t *table, const char *path);
 
+
+inline uint32_t
+ioc_hashfn (void *data, int len)
+{
+        off_t offset;
+
+        offset = *(off_t *) data;
+
+        return (offset >> ioc_log2_page_size);
+}
+
 inline ioc_inode_t *
 ioc_inode_reupdate (ioc_inode_t *ioc_inode)
 {
@@ -80,7 +93,7 @@ ioc_inode_need_revalidate (ioc_inode_t *ioc_inode)
 
 	ret = gettimeofday (&tv, NULL);
 
-	if (time_elapsed (&tv, &ioc_inode->tv) >= table->cache_timeout)
+	if (time_elapsed (&tv, &ioc_inode->cache.tv) >= table->cache_timeout)
 		need_revalidate = 1;
 
 	return need_revalidate;
@@ -93,14 +106,15 @@ ioc_inode_need_revalidate (ioc_inode_t *ioc_inode)
  *
  * assumes lock is held
  */
-int32_t
+int64_t
 __ioc_inode_flush (ioc_inode_t *ioc_inode)
 {
 	ioc_page_t *curr = NULL, *next = NULL;
-	int32_t    destroy_size = 0;
-	int32_t    ret = 0;
+	int64_t    destroy_size = 0;
+	int64_t    ret = 0;
 
-	list_for_each_entry_safe (curr, next, &ioc_inode->pages, pages) {
+	list_for_each_entry_safe (curr, next, &ioc_inode->cache.page_lru,
+                                  page_lru) {
 		ret = ioc_page_destroy (curr);
     
 		if (ret != -1) 
@@ -113,7 +127,7 @@ __ioc_inode_flush (ioc_inode_t *ioc_inode)
 void
 ioc_inode_flush (ioc_inode_t *ioc_inode)
 {
-	int32_t destroy_size = 0;    
+	int64_t destroy_size = 0;    
 
 	ioc_inode_lock (ioc_inode);
 	{
@@ -132,54 +146,38 @@ ioc_inode_flush (ioc_inode_t *ioc_inode)
 	return;
 }
 
-/* 
- * ioc_utimens_cbk -
- * 
- * @frame:
- * @cookie:
- * @this:
- * @op_ret:
- * @op_errno:
- * @stbuf:
- *
- */
 int32_t
-ioc_utimens_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                 int32_t op_ret, int32_t op_errno, struct stat *stbuf)
+ioc_setattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                 int32_t op_ret, int32_t op_errno,
+                 struct stat *preop, struct stat *postop)
 {
-	STACK_UNWIND (frame, op_ret, op_errno, stbuf);
-	return 0;
+ 	STACK_UNWIND_STRICT (setattr, frame, op_ret, op_errno, preop, postop);
+ 	return 0;
 }
 
-/* 
- * ioc_utimens -
- * 
- * @frame:
- * @this:
- * @loc:
- * @tv:
- *
- */
 int32_t
-ioc_utimens (call_frame_t *frame, xlator_t *this, loc_t *loc,
-             struct timespec *tv)
+ioc_setattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
+             struct stat *stbuf, int32_t valid)
 {
-	uint64_t ioc_inode = 0;
-	inode_ctx_get (loc->inode, this, &ioc_inode);
+ 	uint64_t ioc_inode = 0;
 
-	if (ioc_inode)
-		ioc_inode_flush ((ioc_inode_t *)(long)ioc_inode);
+ 	inode_ctx_get (loc->inode, this, &ioc_inode);
 
-	STACK_WIND (frame, ioc_utimens_cbk, FIRST_CHILD (this),
-                    FIRST_CHILD (this)->fops->utimens, loc, tv);
+ 	if (ioc_inode
+            && ((valid & GF_SET_ATTR_ATIME)
+                || (valid & GF_SET_ATTR_MTIME)))
+ 		ioc_inode_flush ((ioc_inode_t *)(long)ioc_inode);
 
-	return 0;
+ 	STACK_WIND (frame, ioc_setattr_cbk, FIRST_CHILD (this),
+                    FIRST_CHILD (this)->fops->setattr, loc, stbuf, valid);
+
+ 	return 0;
 }
 
 int32_t
 ioc_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		int32_t op_ret,	int32_t op_errno, inode_t *inode,
-		struct stat *stbuf, dict_t *dict)
+		struct stat *stbuf, dict_t *dict, struct stat *postparent)
 {
 	ioc_inode_t   *ioc_inode = NULL;
 	ioc_table_t   *table = this->private;
@@ -194,8 +192,8 @@ ioc_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	if (ioc_inode) {
 		ioc_inode_lock (ioc_inode);
 		{
-                        if (ioc_inode->mtime == 0) {
-                                ioc_inode->mtime = stbuf->st_mtime;
+                        if (ioc_inode->cache.mtime == 0) {
+                                ioc_inode->cache.mtime = stbuf->st_mtime;
                         }
 		}
 		ioc_inode_unlock (ioc_inode);
@@ -206,7 +204,7 @@ ioc_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		if (!cache_still_valid) {
 			ioc_inode_flush (ioc_inode);
 		} 
-
+		
 		ioc_table_lock (ioc_inode->table);
 		{
 			list_move_tail (&ioc_inode->inode_lru,
@@ -214,9 +212,10 @@ ioc_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		}
 		ioc_table_unlock (ioc_inode->table);
 	}
-
+	
 out:
-	STACK_UNWIND (frame, op_ret, op_errno, inode, stbuf, dict);
+	STACK_UNWIND_STRICT (lookup, frame, op_ret, op_errno, inode, stbuf, dict,
+                      postparent);
 	return 0;
 }
 
@@ -289,7 +288,7 @@ ioc_cache_validate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		{
 			destroy_size = __ioc_inode_flush (ioc_inode);
 			if (op_ret >= 0)
-				ioc_inode->mtime = stbuf->st_mtime;
+				ioc_inode->cache.mtime = stbuf->st_mtime;
 		}
 		ioc_inode_unlock (ioc_inode);
 		local_stbuf = NULL;
@@ -308,7 +307,7 @@ ioc_cache_validate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
   
 	ioc_inode_lock (ioc_inode);
 	{
-		gettimeofday (&ioc_inode->tv, NULL);
+		gettimeofday (&ioc_inode->cache.tv, NULL);
 	}
 	ioc_inode_unlock (ioc_inode);
 
@@ -329,6 +328,7 @@ ioc_wait_on_inode (ioc_inode_t *ioc_inode, ioc_page_t *page)
 {
 	ioc_waitq_t *waiter = NULL, *trav = NULL;
 	uint32_t    page_found = 0;
+        int32_t     ret = 0;
 
 	trav = ioc_inode->waitq;
 
@@ -342,13 +342,20 @@ ioc_wait_on_inode (ioc_inode_t *ioc_inode, ioc_page_t *page)
   
 	if (!page_found) {
 		waiter = CALLOC (1, sizeof (ioc_waitq_t));
-		ERR_ABORT (waiter);
+                if (waiter == NULL) {
+                        gf_log (ioc_inode->table->xl->name, GF_LOG_ERROR,
+                                "out of memory");
+                        ret = -ENOMEM;
+                        goto out;
+                }
+
 		waiter->data = page;
 		waiter->next = ioc_inode->waitq;
 		ioc_inode->waitq = waiter;
 	}
-  
-	return 0;
+
+out:  
+	return ret;
 }
 
 /*
@@ -365,10 +372,31 @@ ioc_cache_validate (call_frame_t *frame, ioc_inode_t *ioc_inode, fd_t *fd,
 {
 	call_frame_t *validate_frame = NULL;
 	ioc_local_t  *validate_local = NULL;
+        ioc_local_t  *local = NULL;
+        int32_t      ret = 0;
 
+        local = frame->local;
 	validate_local = CALLOC (1, sizeof (ioc_local_t));
-	ERR_ABORT (validate_local);
+        if (validate_local == NULL) {
+                ret = -1;
+                local->op_ret = -1;
+                local->op_errno = ENOMEM;
+                gf_log (ioc_inode->table->xl->name, GF_LOG_ERROR,
+                        "out of memory");
+                goto out;
+        }
+
 	validate_frame = copy_frame (frame);
+        if (validate_frame == NULL) {
+                ret = -1;
+                local->op_ret = -1;
+                local->op_errno = ENOMEM;
+                FREE (validate_local);
+                gf_log (ioc_inode->table->xl->name, GF_LOG_ERROR,
+                        "out of memory");
+                goto out;
+        }
+
 	validate_local->fd = fd_ref (fd);
 	validate_local->inode = ioc_inode;
 	validate_frame->local = validate_local;
@@ -377,20 +405,17 @@ ioc_cache_validate (call_frame_t *frame, ioc_inode_t *ioc_inode, fd_t *fd,
                     FIRST_CHILD (frame->this),
                     FIRST_CHILD (frame->this)->fops->fstat, fd);
 
-	return 0;
+out:
+	return ret;
 }
 
 inline uint32_t
 is_match (const char *path, const char *pattern)
 {
-	char    *pathname = NULL;
 	int32_t ret = 0;
 
-        pathname = strdup (path);
 	ret = fnmatch (pattern, path, FNM_NOESCAPE);
   
-	free (pathname);
-  
 	return (ret == 0);
 }
 
@@ -457,6 +482,7 @@ ioc_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
  
                                 ioc_inode = ioc_inode_update (table, inode,
                                                               weight);
+
                                 __inode_ctx_put (fd->inode, this, 
                                                  (uint64_t)(long)ioc_inode);
                         } else {
@@ -497,7 +523,7 @@ ioc_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
 	FREE (local);
 	frame->local = NULL;
 
-	STACK_UNWIND (frame, op_ret, op_errno, fd);
+	STACK_UNWIND_STRICT (open, frame, op_ret, op_errno, fd);
 
 	return 0;
 }
@@ -518,7 +544,8 @@ ioc_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
 int32_t
 ioc_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 int32_t op_ret,	int32_t op_errno, fd_t *fd,
-		inode_t *inode,	struct stat *buf)
+		inode_t *inode,	struct stat *buf, struct stat *preparent,
+                struct stat *postparent)
 {
 	ioc_local_t *local = NULL;
 	ioc_table_t *table = NULL;
@@ -536,9 +563,10 @@ ioc_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			weight = ioc_get_priority (table, path);
 
 			ioc_inode = ioc_inode_update (table, inode, weight);
+
                         ioc_inode_lock (ioc_inode);
                         {
-                                ioc_inode->mtime = buf->st_mtime;
+                                ioc_inode->cache.mtime = buf->st_mtime;
                         }
                         ioc_inode_unlock (ioc_inode);
 
@@ -574,7 +602,8 @@ ioc_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	frame->local = NULL;
 	FREE (local);
 
-	STACK_UNWIND (frame, op_ret, op_errno, fd, inode, buf);
+        STACK_UNWIND_STRICT (create, frame, op_ret, op_errno, fd, inode, buf,
+                             preparent, postparent);
 
 	return 0;
 }
@@ -589,22 +618,26 @@ ioc_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
  */
 int32_t
 ioc_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
-	  fd_t *fd)
+	  fd_t *fd, int32_t wbflags)
 {
   
 	ioc_local_t *local = NULL;
 
         local = CALLOC (1, sizeof (ioc_local_t));
-	ERR_ABORT (local);
+        if (local == NULL) {
+                gf_log (this->name, GF_LOG_ERROR, "out of memory");
+                STACK_UNWIND_STRICT (open, frame, -1, ENOMEM, NULL);
+                return 0;
+        }
 
 	local->flags = flags;
 	local->file_loc.path = loc->path;
 	local->file_loc.inode = loc->inode;
   
 	frame->local = local;
-  
+        
 	STACK_WIND (frame, ioc_open_cbk, FIRST_CHILD(this),
-                    FIRST_CHILD(this)->fops->open, loc, flags, fd);
+                    FIRST_CHILD(this)->fops->open, loc, flags, fd, wbflags);
 
 	return 0;
 }
@@ -624,9 +657,14 @@ ioc_create (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
 	    mode_t mode, fd_t *fd)
 {
 	ioc_local_t *local = NULL;
-        
+
         local = CALLOC (1, sizeof (ioc_local_t));
-	ERR_ABORT (local);
+        if (local == NULL) {
+                gf_log (this->name, GF_LOG_ERROR, "out of memory");
+                STACK_UNWIND_STRICT (create, frame, -1, ENOMEM, NULL, NULL,
+                                     NULL, NULL, NULL);
+                return 0;
+        }
 
 	local->flags = flags;
 	local->file_loc.path = loc->path;
@@ -672,7 +710,8 @@ ioc_readv_disabled_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			int32_t count, struct stat *stbuf,
                         struct iobref *iobref)
 {
-	STACK_UNWIND (frame, op_ret, op_errno, vector, count, stbuf, iobref);
+	STACK_UNWIND_STRICT (readv, frame, op_ret, op_errno, vector, count,
+                             stbuf, iobref);
 	return 0;
 }
 
@@ -704,7 +743,7 @@ ioc_need_prune (ioc_table_t *table)
  */
 void
 ioc_dispatch_requests (call_frame_t *frame, ioc_inode_t *ioc_inode, fd_t *fd,
-		   off_t offset, size_t size)
+                       off_t offset, size_t size)
 {
 	ioc_local_t *local = NULL;
 	ioc_table_t *table = NULL;
@@ -716,6 +755,7 @@ ioc_dispatch_requests (call_frame_t *frame, ioc_inode_t *ioc_inode, fd_t *fd,
 	int32_t     fault = 0;
         size_t      trav_size = 0;
         off_t       local_offset = 0;
+        int32_t     ret = -1;
 	int8_t      need_validate = 0;
 	int8_t      might_need_validate = 0;  /*
                                                * if a page exists, do we need 
@@ -758,6 +798,9 @@ ioc_dispatch_requests (call_frame_t *frame, ioc_inode_t *ioc_inode, fd_t *fd,
 			if (!trav) {
 				gf_log (frame->this->name, GF_LOG_CRITICAL,
 					"out of memory");
+                                local->op_ret = -1;
+                                local->op_errno = ENOMEM;
+                                goto out;
 			}
 		} 
 
@@ -778,7 +821,20 @@ ioc_dispatch_requests (call_frame_t *frame, ioc_inode_t *ioc_inode, fd_t *fd,
 				if (!ioc_inode->waitq) {
 					need_validate = 1;
 				}
-				ioc_wait_on_inode (ioc_inode, trav);
+
+				ret = ioc_wait_on_inode (ioc_inode, trav);
+                                if (ret < 0) {
+                                        local->op_ret = -1;
+                                        local->op_errno = -ret;
+                                        need_validate = 0;
+
+                                        waitq = ioc_page_wakeup (trav);
+                                        ioc_inode_unlock (ioc_inode);
+
+                                        ioc_waitq_return (waitq);
+                                        waitq = NULL;
+                                        goto out;
+                                }
 			}
 		}
 
@@ -800,12 +856,24 @@ ioc_dispatch_requests (call_frame_t *frame, ioc_inode_t *ioc_inode, fd_t *fd,
 				"sending validate request for "
 				"inode(%"PRId64") at offset=%"PRId64"",
 				fd->inode->ino, trav_offset);
-			ioc_cache_validate (frame, ioc_inode, fd, trav);
+			ret = ioc_cache_validate (frame, ioc_inode, fd, trav);
+                        if (ret == -1) {
+                                ioc_inode_lock (ioc_inode);
+                                {
+                                        waitq = ioc_page_wakeup (trav);
+                                }
+                                ioc_inode_unlock (ioc_inode);
+
+                                ioc_waitq_return (waitq);
+                                waitq = NULL;
+                                goto out;
+                        }
 		}
     
 		trav_offset += table->page_size;
 	}
 
+out:
 	ioc_frame_return (frame);
 
 	if (ioc_need_prune (ioc_inode->table)) {
@@ -834,6 +902,13 @@ ioc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	ioc_inode_t  *ioc_inode = NULL;
 	ioc_local_t  *local = NULL;
 	uint32_t     weight = 0;
+        ioc_table_t  *table = NULL;
+        uint32_t     num_pages = 0;
+        int32_t      op_errno = -1;
+
+        if (!this) {
+                goto out;
+        }
 
 	inode_ctx_get (fd->inode, this, &tmp_ioc_inode);
 	ioc_inode = (ioc_inode_t *)(long)tmp_ioc_inode;
@@ -846,6 +921,56 @@ ioc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd,
 		return 0;
 	}
 
+
+        table = this->private;
+
+        if (!table) {
+                gf_log (this->name, GF_LOG_ERROR, "table is null");
+                op_errno = EINVAL;
+                goto out;
+        }
+
+
+        ioc_table_lock (table);
+        {
+                if (!table->mem_pool) {
+
+                        num_pages = (table->cache_size / table->page_size)
+                                + ((table->cache_size % table->page_size) 
+                                ? 1 : 0);
+
+                        table->mem_pool
+                                =  mem_pool_new (rbthash_entry_t, num_pages);
+
+                        if (!table->mem_pool) {
+                                gf_log (this->name, GF_LOG_ERROR,
+                                        "Unable to allocate mem_pool");
+                                op_errno = ENOMEM;
+                                ioc_table_unlock (table);
+                                goto out;
+                        }
+                }
+        }
+        ioc_table_unlock (table);
+
+        ioc_inode_lock (ioc_inode);
+        {
+                if (!ioc_inode->cache.page_table) {
+                        ioc_inode->cache.page_table
+                                = rbthash_table_init 
+                                                (IOC_PAGE_TABLE_BUCKET_COUNT,
+                                                 ioc_hashfn, NULL, 0,
+                                                 table->mem_pool);
+
+                        if (ioc_inode->cache.page_table == NULL) {
+                                op_errno = ENOMEM;
+                                ioc_inode_unlock (ioc_inode);
+                                goto out;
+                        }
+                }
+        }
+        ioc_inode_unlock (ioc_inode);
+
 	if (!fd_ctx_get (fd, this, NULL)) {
 		/* disable caching for this fd, go ahead with normal readv */
 		STACK_WIND (frame, ioc_readv_disabled_cbk,
@@ -856,7 +981,12 @@ ioc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	}
 
 	local = (ioc_local_t *) CALLOC (1, sizeof (ioc_local_t));
-	ERR_ABORT (local);
+        if (local == NULL) {
+                gf_log (this->name, GF_LOG_ERROR, "out of memory");
+                op_errno = ENOMEM;
+                goto out;
+        }
+
 	INIT_LIST_HEAD (&local->fill_list);
 
 	frame->local = local;  
@@ -880,8 +1010,11 @@ ioc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	ioc_table_unlock (ioc_inode->table);
 
 	ioc_dispatch_requests (frame, ioc_inode, fd, offset, size);
-  
 	return 0;
+
+out:
+        STACK_UNWIND_STRICT (readv, frame, -1, op_errno, NULL, 0, NULL, NULL);
+        return 0;
 }
 
 /*
@@ -896,7 +1029,8 @@ ioc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd,
  */
 int32_t
 ioc_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		int32_t op_ret,	int32_t op_errno, struct stat *stbuf)
+		int32_t op_ret,	int32_t op_errno, struct stat *prebuf,
+                struct stat *postbuf)
 {
 	ioc_local_t *local     = NULL;
 	uint64_t    ioc_inode = 0;
@@ -907,7 +1041,7 @@ ioc_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	if (ioc_inode)
 		ioc_inode_flush ((ioc_inode_t *)(long)ioc_inode);
 
-	STACK_UNWIND (frame, op_ret, op_errno, stbuf);
+	STACK_UNWIND_STRICT (writev, frame, op_ret, op_errno, prebuf, postbuf);
 	return 0;
 }
 
@@ -929,9 +1063,14 @@ ioc_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
 {
 	ioc_local_t *local     = NULL;
 	uint64_t    ioc_inode = 0;
-	
+
 	local = CALLOC (1, sizeof (ioc_local_t));
-	ERR_ABORT (local);
+        if (local == NULL) {
+                gf_log (this->name, GF_LOG_ERROR, "out of memory");
+
+                STACK_UNWIND_STRICT (writev, frame, -1, ENOMEM, NULL, NULL);
+                return 0;
+        }
 
 	/* TODO: why is it not fd_ref'ed */
 	local->fd = fd;
@@ -961,13 +1100,39 @@ ioc_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
  */
 int32_t 
 ioc_truncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                  int32_t op_ret, int32_t op_errno, struct stat *buf)
+                  int32_t op_ret, int32_t op_errno, struct stat *prebuf,
+                  struct stat *postbuf)
 {
 
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
+	STACK_UNWIND_STRICT (truncate, frame, op_ret, op_errno, prebuf,
+                             postbuf);
 	return 0;
 }
 
+
+/*
+ * ioc_ftruncate_cbk -
+ *
+ * @frame:
+ * @cookie:
+ * @this:
+ * @op_ret:
+ * @op_errno:
+ * @buf:
+ *
+ */
+int32_t
+ioc_ftruncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                  int32_t op_ret, int32_t op_errno, struct stat *prebuf,
+                  struct stat *postbuf)
+{
+
+	STACK_UNWIND_STRICT (ftruncate, frame, op_ret, op_errno, prebuf,
+                             postbuf);
+	return 0;
+}
+
+
 /*
  * ioc_truncate -
  * 
@@ -1009,7 +1174,7 @@ ioc_ftruncate (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset)
 	if (ioc_inode)
 		ioc_inode_flush ((ioc_inode_t *)(long)ioc_inode);
 
-	STACK_WIND (frame, ioc_truncate_cbk, FIRST_CHILD(this),
+	STACK_WIND (frame, ioc_ftruncate_cbk, FIRST_CHILD(this),
                     FIRST_CHILD(this)->fops->ftruncate, fd, offset);
 	return 0;
 }
@@ -1018,7 +1183,7 @@ int32_t
 ioc_lk_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
 	    int32_t op_errno, struct flock *lock)
 {
-	STACK_UNWIND (frame, op_ret, op_errno, lock);
+	STACK_UNWIND_STRICT (lk, frame, op_ret, op_errno, lock);
 	return 0;
 }
 
@@ -1034,13 +1199,13 @@ ioc_lk (call_frame_t *frame, xlator_t *this, fd_t *fd, int32_t cmd,
 	if (!ioc_inode) {
 		gf_log (this->name, GF_LOG_DEBUG,
 			"inode context is NULL: returning EBADFD");
-		STACK_UNWIND (frame, -1, EBADFD, NULL);
+		STACK_UNWIND_STRICT (lk, frame, -1, EBADFD, NULL);
 		return 0;
 	}
 
 	ioc_inode_lock (ioc_inode);
 	{
-		gettimeofday (&ioc_inode->tv, NULL);
+		gettimeofday (&ioc_inode->cache.tv, NULL);
 	}
 	ioc_inode_unlock (ioc_inode);
 
@@ -1062,9 +1227,14 @@ ioc_get_priority_list (const char *opt_str, struct list_head *first)
 	char                *pattern = NULL;
 	char                *priority = NULL;
 	char                *string = NULL;
-	struct ioc_priority *curr = NULL;
+	struct ioc_priority *curr = NULL, *tmp = NULL;
 
         string = strdup (opt_str);
+        if (string == NULL) {
+                max_pri = -1;
+                goto out;
+        }
+                
 	/* Get the pattern for cache priority. 
 	 * "option priority *.jpg:1,abc*:2" etc 
 	 */
@@ -1074,28 +1244,71 @@ ioc_get_priority_list (const char *opt_str, struct list_head *first)
 	stripe_str = strtok_r (string, ",", &tmp_str);
 	while (stripe_str) {
 		curr = CALLOC (1, sizeof (struct ioc_priority));
-		ERR_ABORT (curr);
+                if (curr == NULL) {
+                        max_pri = -1;
+                        goto out;
+                }
+
 		list_add_tail (&curr->list, first);
 
 		dup_str = strdup (stripe_str);
+                if (dup_str == NULL) {
+                        max_pri = -1;
+                        goto out;
+                }
+
 		pattern = strtok_r (dup_str, ":", &tmp_str1);
-		if (!pattern)
-			return -1;
+		if (!pattern) {
+                        max_pri = -1;
+                        goto out;
+                }
+
 		priority = strtok_r (NULL, ":", &tmp_str1);
-		if (!priority)
-			return -1;
+		if (!priority) {
+                        max_pri = -1;
+                        goto out;
+                }
+
 		gf_log ("io-cache", GF_LOG_TRACE,
 			"ioc priority : pattern %s : priority %s",
 			pattern,
 			priority);
+
 		curr->pattern = strdup (pattern);
+                if (curr->pattern == NULL) {
+                        max_pri = -1;
+                        goto out;
+                }
+
 		curr->priority = strtol (priority, &tmp_str2, 0);
-		if (tmp_str2 && (*tmp_str2))
-			return -1;
-		else
-			max_pri = max (max_pri, curr->priority);
+		if (tmp_str2 && (*tmp_str2)) {
+                        max_pri = -1;
+                        goto out;
+                } else {
+ 			max_pri = max (max_pri, curr->priority);
+                }
+
+                free (dup_str);
+                dup_str = NULL;
+
 		stripe_str = strtok_r (NULL, ",", &tmp_str);
 	}
+out:
+        if (string != NULL) {
+                free (string);
+        }
+
+        if (dup_str != NULL) {
+                free (dup_str);
+        }
+
+        if (max_pri == -1) {
+                list_for_each_entry_safe (curr, tmp, first, list) {
+                        list_del_init (&curr->list);
+                        free (curr->pattern);
+                        free (curr);
+                }
+        }
 
 	return max_pri;
 }
@@ -1108,16 +1321,18 @@ ioc_get_priority_list (const char *opt_str, struct list_head *first)
 int32_t 
 init (xlator_t *this)
 {
-	ioc_table_t *table;
-	dict_t      *options = this->options;
-	uint32_t    index = 0;
-	char        *cache_size_string = NULL;
+	ioc_table_t     *table = NULL;
+	dict_t          *options = this->options;
+	uint32_t         index = 0;
+	char            *cache_size_string = NULL;
+        int32_t          ret = -1;
+        glusterfs_ctx_t *ctx = NULL;
 
 	if (!this->children || this->children->next) {
 		gf_log (this->name, GF_LOG_ERROR,
 			"FATAL: io-cache not configured with exactly "
 			"one child");
-		return -1;
+                goto out;
 	}
 
 	if (!this->parents) {
@@ -1126,7 +1341,10 @@ init (xlator_t *this)
 	}
 
 	table = (void *) CALLOC (1, sizeof (*table));
-	ERR_ABORT (table);
+        if (table == NULL) {
+                gf_log (this->name, GF_LOG_ERROR, "out of memory");
+                goto out;
+        }
   
 	table->xl = this;
 	table->page_size = this->ctx->page_size;
@@ -1142,7 +1360,7 @@ init (xlator_t *this)
 				"invalid number format \"%s\" of "
 				"\"option cache-size\"", 
 				cache_size_string);
-			return -1;
+                        goto out;
 		}
       
 		gf_log (this->name, GF_LOG_TRACE, 
@@ -1171,20 +1389,37 @@ init (xlator_t *this)
 		table->max_pri = ioc_get_priority_list (option_list, 
 							&table->priority_list);
     
-		if (table->max_pri == -1)
-			return -1;
+		if (table->max_pri == -1) {
+                        goto out;
+                }
 	}
 	table->max_pri ++;
 	INIT_LIST_HEAD (&table->inodes);
   
 	table->inode_lru = CALLOC (table->max_pri, sizeof (struct list_head));
-	ERR_ABORT (table->inode_lru);
+        if (table->inode_lru == NULL) {
+                goto out;
+        }
+
 	for (index = 0; index < (table->max_pri); index++)
 		INIT_LIST_HEAD (&table->inode_lru[index]);
 
 	pthread_mutex_init (&table->table_lock, NULL);
 	this->private = table;
-	return 0;
+        ret = 0;
+
+        ctx = this->ctx;
+        ioc_log2_page_size = gf_log2 (ctx->page_size);
+
+out:
+        if (ret == -1) {
+                if (table != NULL) {
+                        free (table->inode_lru);
+                        free (table);
+                }
+        }
+
+	return ret;
 }
 
 /*
@@ -1198,6 +1433,9 @@ fini (xlator_t *this)
 {
 	ioc_table_t *table = this->private;
 
+        if (table == NULL)
+                return;
+
 	pthread_mutex_destroy (&table->table_lock);
 	FREE (table);
 
@@ -1212,9 +1450,9 @@ struct xlator_fops fops = {
 	.writev      = ioc_writev,
 	.truncate    = ioc_truncate,
 	.ftruncate   = ioc_ftruncate,
-	.utimens     = ioc_utimens,
 	.lookup      = ioc_lookup,
-	.lk          = ioc_lk
+	.lk          = ioc_lk,
+        .setattr     = ioc_setattr
 };
 
 struct xlator_mops mops = {
