@@ -30,35 +30,28 @@
 #include <assert.h>
 #include <sys/time.h>
 
+char
+ioc_empty (struct ioc_cache *cache)
+{
+        return list_empty (&cache->page_lru);
+}
+
 ioc_page_t *
 ioc_page_get (ioc_inode_t *ioc_inode, off_t offset)
 {
-	int8_t       found = 0;
-	ioc_page_t   *page = NULL;
-	ioc_table_t  *table = NULL;
-	off_t        rounded_offset = 0;
+	ioc_page_t   *page           = NULL;
+	ioc_table_t  *table          = NULL;
+	off_t         rounded_offset = 0;
 
         table = ioc_inode->table;
         rounded_offset = floor (offset, table->page_size);
  
-	if (list_empty (&ioc_inode->pages)) {
-		return NULL;
-	}
+        page = rbthash_get (ioc_inode->cache.page_table, &rounded_offset,
+                            sizeof (rounded_offset));
 
-	list_for_each_entry (page, &ioc_inode->pages, pages) {
-		if (page->offset == rounded_offset) {
-			found = 1;
-			break;
-		}
-	}
-
-	/* was previously returning ioc_inode itself.., 
-	 * 1st of its type and found one more downstairs :O */
-	if (!found){
-		page = NULL;
-	} else {
+        if (page != NULL) {
 		/* push the page to the end of the lru list */
-		list_move_tail (&page->page_lru, &ioc_inode->page_lru);
+		list_move_tail (&page->page_lru, &ioc_inode->cache.page_lru);
 	}
 
 	return page;
@@ -74,7 +67,7 @@ ioc_page_get (ioc_inode_t *ioc_inode, off_t offset)
 int64_t
 ioc_page_destroy (ioc_page_t *page)
 {
-	int64_t page_size = 0;
+	int64_t  page_size = 0;
 
 	page_size = iobref_size (page->iobref);
 
@@ -82,8 +75,8 @@ ioc_page_destroy (ioc_page_t *page)
 		/* frames waiting on this page, do not destroy this page */
 		page_size = -1;
 	} else {
-
-		list_del (&page->pages);
+                rbthash_remove (page->inode->cache.page_table, &page->offset,
+                                sizeof (page->offset));
 		list_del (&page->page_lru);
     
 		gf_log (page->inode->table->xl->name, GF_LOG_TRACE,
@@ -98,7 +91,6 @@ ioc_page_destroy (ioc_page_t *page)
 		}
     
 		page->inode = NULL;
-    
 	}
 
 	if (page_size != -1) {
@@ -140,8 +132,8 @@ ioc_prune (ioc_table_t *table)
 				/* { */
 
 				list_for_each_entry_safe (page, next, 
-							  &curr->page_lru, 
-							  page_lru) {
+							  &curr->cache.page_lru,
+                                                          page_lru) {
 					/* done with all pages, and not 
 					 * reached equilibrium yet??
 					 * continue with next inode in 
@@ -163,7 +155,7 @@ ioc_prune (ioc_table_t *table)
 					if (size_pruned >= size_to_prune)
 						break;
 				} /* list_for_each_entry_safe(page...) */
-				if (list_empty (&curr->pages)) {
+				if (ioc_empty (&curr->cache)) {
 					list_del_init (&curr->inode_lru);
 				}
 
@@ -194,34 +186,42 @@ ioc_prune (ioc_table_t *table)
 ioc_page_t *
 ioc_page_create (ioc_inode_t *ioc_inode, off_t offset)
 {
-	ioc_table_t *table = NULL;
-	ioc_page_t  *page = NULL;
-	off_t       rounded_offset = 0;
-	ioc_page_t  *newpage = NULL;
+	ioc_table_t *table          = NULL;
+	ioc_page_t  *page           = NULL;
+	off_t        rounded_offset = 0;
+	ioc_page_t  *newpage        = NULL;
   
         table = ioc_inode->table;
         rounded_offset = floor (offset, table->page_size);
 
         newpage = CALLOC (1, sizeof (*newpage));
-	ERR_ABORT (newpage);
+        if (newpage == NULL) {
+                goto out;
+        }
 
-	if (ioc_inode)
+	if (ioc_inode) {
 		table = ioc_inode->table;
-	else {
-		return NULL;
+        } else {
+                free (newpage);
+                newpage = NULL;
+                goto out;
 	}
    
 	newpage->offset = rounded_offset;
 	newpage->inode = ioc_inode;
 	pthread_mutex_init (&newpage->page_lock, NULL);
 
-	list_add_tail (&newpage->page_lru, &ioc_inode->page_lru);
-	list_add_tail (&newpage->pages, &ioc_inode->pages);
+        rbthash_insert (ioc_inode->cache.page_table, newpage, &rounded_offset,
+                        sizeof (rounded_offset));
+ 
+	list_add_tail (&newpage->page_lru, &ioc_inode->cache.page_lru);
 
 	page = newpage;
 
 	gf_log ("io-cache", GF_LOG_TRACE,
 		"returning new page %p", page);
+
+out:
 	return page;
 }
 
@@ -242,8 +242,13 @@ ioc_wait_on_page (ioc_page_t *page, call_frame_t *frame, off_t offset,
 	ioc_local_t *local = frame->local;
 
 	waitq = CALLOC (1, sizeof (*waitq));
-	ERR_ABORT (waitq);
-  
+        if (waitq == NULL) {
+                local->op_ret = -1;
+                local->op_errno = ENOMEM;
+                gf_log (frame->this->name, GF_LOG_ERROR, "out of memory");
+                goto out;
+        } 
+
 	gf_log (frame->this->name, GF_LOG_TRACE,
 		"frame(%p) waiting on page = %p, offset=%"PRId64", "
 		"size=%"GF_PRI_SIZET"",
@@ -261,6 +266,9 @@ ioc_wait_on_page (ioc_page_t *page, call_frame_t *frame, off_t offset,
 		local->wait_count++;
 	}
 	ioc_local_unlock (local);
+
+out:
+        return;
 }
 
 
@@ -277,14 +285,14 @@ int8_t
 ioc_cache_still_valid (ioc_inode_t *ioc_inode, struct stat *stbuf)
 {
 	int8_t cache_still_valid = 1;
-  
+
 #if 0
-	if (!stbuf || (stbuf->st_mtime != ioc_inode->mtime) || 
+	if (!stbuf || (stbuf->st_mtime != ioc_inode->cache.mtime) || 
 	    (stbuf->st_mtim.tv_nsec != ioc_inode->stbuf.st_mtim.tv_nsec))
 		cache_still_valid = 0;
 
 #else
-	if (!stbuf || (stbuf->st_mtime != ioc_inode->mtime))
+	if (!stbuf || (stbuf->st_mtime != ioc_inode->cache.mtime))
 		cache_still_valid = 0;
 
 #endif
@@ -341,13 +349,12 @@ ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         ioc_inode = local->inode;
         table = ioc_inode->table;
 
-	trav_offset = offset;  
+	trav_offset = offset;
 	payload_size = op_ret;
 
         zero_filled = ((op_ret >=0)
                        && (stbuf->st_mtime == 0));
 
-
 	ioc_inode_lock (ioc_inode);
 	{
 		if (op_ret == -1 ||
@@ -360,9 +367,9 @@ ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		}
 
 		if ((op_ret >= 0) && !zero_filled)
-                        ioc_inode->mtime = stbuf->st_mtime;
+			ioc_inode->cache.mtime = stbuf->st_mtime;
 
-		gettimeofday (&ioc_inode->tv, NULL);
+		gettimeofday (&ioc_inode->cache.tv, NULL);
 
 		if (op_ret < 0) {
 			/* error, readv returned -1 */
@@ -390,6 +397,17 @@ ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	
 				/* keep a copy of the page for our cache */
 				page->vector = iov_dup (vector, count);
+                                if (page->vector == NULL) {
+                                        page = ioc_page_get (ioc_inode, offset);
+                                        if (page != NULL)
+                                                waitq = ioc_page_error (page,
+                                                                        -1, 
+                                                                        ENOMEM);
+                                        op_ret = -1;
+                                        op_errno = ENOMEM;
+                                        goto unlock;
+                                }
+
 				page->count = count;
 				if (iobref) {
 					page->iobref = iobref_ref (iobref);
@@ -422,6 +440,7 @@ ioc_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			} /* if(!page)...else */
 		} /* if(op_ret < 0)...else */
 	} /* ioc_inode locked region end */
+unlock:
 	ioc_inode_unlock (ioc_inode);
 
 	ioc_waitq_return (waitq);
@@ -471,11 +490,29 @@ ioc_page_fault (ioc_inode_t *ioc_inode,	call_frame_t *frame, fd_t *fd,
 	ioc_table_t  *table = NULL;
 	call_frame_t *fault_frame = NULL;
 	ioc_local_t  *fault_local = NULL;
+        int32_t      op_ret = -1, op_errno = -1;
+        ioc_waitq_t  *waitq = NULL;
+        ioc_page_t   *page = NULL;
 
         table = ioc_inode->table;
         fault_frame = copy_frame (frame);
+        if (fault_frame == NULL) {
+                op_ret = -1;
+                op_errno = ENOMEM;
+                gf_log (ioc_inode->table->xl->name, GF_LOG_ERROR,
+                        "out of memory");
+                goto err;
+        }
+
         fault_local = CALLOC (1, sizeof (ioc_local_t));
-        ERR_ABORT (fault_local);
+        if (fault_local == NULL) {
+                op_ret = -1;
+                op_errno = ENOMEM;
+                STACK_DESTROY (fault_frame->root);
+                gf_log (ioc_inode->table->xl->name, GF_LOG_ERROR,
+                        "out of memory");
+                goto err;
+        }
 
 	/* NOTE: copy_frame() means, the frame the fop whose fd_ref we 
 	 * are using till now won't be valid till we get reply from server. 
@@ -498,9 +535,18 @@ ioc_page_fault (ioc_inode_t *ioc_inode,	call_frame_t *frame, fd_t *fd,
 		    FIRST_CHILD(fault_frame->this)->fops->readv, fd,
                     table->page_size, offset);
 	return;
+
+err:
+        page = ioc_page_get (ioc_inode, offset);
+        if (page != NULL) {
+                waitq = ioc_page_error (page, op_ret, op_errno);
+                if (waitq != NULL) {
+                        ioc_waitq_return (waitq);
+                }
+        }
 }
 
-void
+int32_t
 ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
                 size_t size)
 {
@@ -512,6 +558,7 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
 	ioc_inode_t *ioc_inode = NULL;
         ioc_fill_t  *new = NULL;
         int8_t      found = 0;
+        int32_t     ret = 0;
   
         local = frame->local;
         ioc_inode = page->inode;
@@ -522,7 +569,7 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
 		frame, offset, size, page->size, local->wait_count);
 
 	/* immediately move this page to the end of the page_lru list */
-	list_move_tail (&page->page_lru, &ioc_inode->page_lru);
+	list_move_tail (&page->page_lru, &ioc_inode->cache.page_lru);
 	/* fill local->pending_size bytes from local->pending_offset */
 	if (local->op_ret != -1 && page->size) {
 		if (offset > page->offset)
@@ -554,7 +601,15 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
 
 		{
                         new = CALLOC (1, sizeof (*new));
-			ERR_ABORT (new);
+                        if (new == NULL) {
+                                local->op_ret = -1;
+                                local->op_errno = ENOMEM;
+                                ret = -1;
+                                gf_log (page->inode->table->xl->name,
+                                        GF_LOG_ERROR, "out of memory");
+                                goto out;
+                        }
+
 			new->offset = page->offset;
 			new->size = copy_size;
 			new->iobref = iobref_ref (page->iobref);
@@ -563,9 +618,22 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
 						 src_offset,
 						 src_offset + copy_size,
 						 NULL);
+
 			new->vector = CALLOC (new->count, 
 					      sizeof (struct iovec));
-			ERR_ABORT (new->vector);
+                        if (new->vector == NULL) {
+                                local->op_ret = -1;
+                                local->op_errno = ENOMEM;
+
+                                iobref_unref (new->iobref);
+                                FREE (new);
+
+                                ret = -1;
+                                gf_log (page->inode->table->xl->name,
+                                        GF_LOG_ERROR, "out of memory");
+                                goto out;
+                        }
+
 			new->count = iov_subset (page->vector,
 						 page->count,
 						 src_offset,
@@ -604,6 +672,9 @@ ioc_frame_fill (ioc_page_t *page, call_frame_t *frame, off_t offset,
 		}
 		local->op_ret += copy_size;
 	}
+
+out:
+        return ret;
 }
 
 /*
@@ -625,13 +696,17 @@ ioc_frame_unwind (call_frame_t *frame)
 	int32_t       copied = 0;
 	struct iobref *iobref = NULL;
 	struct stat   stbuf = {0,};
-	int32_t       op_ret = 0;
+	int32_t       op_ret = 0, op_errno = 0;
 
         local = frame->local;
 	//  ioc_local_lock (local);
-	iobref = iobref_new ();
-
 	frame->local = NULL;
+	iobref = iobref_new ();
+        if (iobref == NULL) {
+                op_ret = -1;
+                op_errno = ENOMEM;
+                gf_log (frame->this->name, GF_LOG_ERROR, "out of memory");
+        }
 
 	if (list_empty (&local->fill_list)) {
 		gf_log (frame->this->name, GF_LOG_TRACE,
@@ -645,16 +720,23 @@ ioc_frame_unwind (call_frame_t *frame)
 	}
 
 	vector = CALLOC (count, sizeof (*vector));
-	ERR_ABORT (vector);
+        if (vector == NULL) {
+                op_ret = -1;
+                op_errno = ENOMEM;
+                
+                gf_log (frame->this->name, GF_LOG_ERROR, "out of memory");
+        }
   
 	list_for_each_entry_safe (fill, next, &local->fill_list, list) {
-		memcpy (((char *)vector) + copied,
-			fill->vector,
-			fill->count * sizeof (*vector));
+                if ((vector != NULL) &&  (iobref != NULL)) { 
+                        memcpy (((char *)vector) + copied,
+                                fill->vector,
+                                fill->count * sizeof (*vector));
     
-		copied += (fill->count * sizeof (*vector));
+                        copied += (fill->count * sizeof (*vector));
 
-		iobref_merge (iobref, fill->iobref);
+                        iobref_merge (iobref, fill->iobref);
+                }
 
 		list_del (&fill->list);
 		iobref_unref (fill->iobref);
@@ -662,22 +744,31 @@ ioc_frame_unwind (call_frame_t *frame)
 		free (fill);
 	}
   
-	op_ret = iov_length (vector, count);
+        if (op_ret != -1) {
+                op_ret = iov_length (vector, count);
+        }
+
 	gf_log (frame->this->name, GF_LOG_TRACE,
 		"frame(%p) unwinding with op_ret=%d", frame, op_ret);
 
 	//  ioc_local_unlock (local);
 
-	STACK_UNWIND (frame, op_ret, local->op_errno, vector, count,
-		      &stbuf, iobref);
+	STACK_UNWIND_STRICT (readv, frame, op_ret, local->op_errno, vector,
+                             count, &stbuf, iobref);
 
-	iobref_unref (iobref);
+        if (iobref != NULL) {
+                iobref_unref (iobref);
+        }
+        
+        if (vector != NULL) {
+                free (vector);
+                vector = NULL;
+        }
     
 	pthread_mutex_destroy (&local->local_lock);
 	free (local);
-	free (vector);
 
-	return;
+        return;
 }
 
 /*
@@ -719,6 +810,7 @@ ioc_page_wakeup (ioc_page_t *page)
 {
 	ioc_waitq_t  *waitq = NULL, *trav = NULL;
 	call_frame_t *frame = NULL;
+        int32_t      ret = -1; 
 
 	waitq = page->waitq;
 	page->waitq = NULL;
@@ -731,8 +823,11 @@ ioc_page_wakeup (ioc_page_t *page)
   
 	for (trav = waitq; trav; trav = trav->next) {
 		frame = trav->data; 
-		ioc_frame_fill (page, frame, trav->pending_offset, 
-				trav->pending_size);
+		ret = ioc_frame_fill (page, frame, trav->pending_offset, 
+                                      trav->pending_size);
+                if (ret == -1) {
+                        break;
+                }
 	}
 	
 	return waitq;
