@@ -34,6 +34,8 @@
 #include "common-utils.h"
 #include "list.h"
 #include <stdlib.h>
+#include "locking.h"
+#include <semaphore.h>
 
 #define min(a,b) ((a)<(b)?(a):(b))
 #define max(a,b) ((a)>(b)?(a):(b))
@@ -49,17 +51,20 @@ struct iot_request {
         call_stub_t *stub;
 };
 
-#define IOT_STATE_ACTIVE        1
-#define IOT_STATE_DEAD          2
+typedef enum {
+        IOT_STATE_ACTIVE,
+        IOT_STATE_EXIT_REQUEST,
+        IOT_STATE_DEAD
+}iot_state_t;
 #define iot_worker_active(wrk)  ((wrk)->state == IOT_STATE_ACTIVE)
 
-#define MAX_IDLE_SKEW                   1000    /* usecs */
-#define skew_usec_idle_time(usec)       ((usec) + (random () % MAX_IDLE_SKEW))
+#define MAX_IDLE_SKEW                   4       /* In secs */
+#define skew_sec_idle_time(sec)         ((sec) + (random () % MAX_IDLE_SKEW))
 #define IOT_DEFAULT_IDLE                180     /* In secs. */
 
 #define IOT_MIN_THREADS         2
 #define IOT_DEFAULT_THREADS     16
-#define IOT_MAX_THREADS         256
+#define IOT_MAX_THREADS         64
 
 #define IOT_SCALING_OFF                 _gf_false
 #define IOT_SCALING_ON                  _gf_true
@@ -68,15 +73,33 @@ struct iot_request {
 
 #define IOT_THREAD_STACK_SIZE   ((size_t)(1024*1024))
 
+/* This signifies the max number of outstanding request we're expecting
+ * at a point for every worker thread.
+ * For an idea of the memory foot-print, consider at most 16 Bytes per
+ * iot_request_t on a 64-bit system with another 16 bytes per chunk in the
+ * header. For 64 slots in the pool, we'll use up 2 KiB, with 64 threads this
+ * goes up to 128 KiB.
+ *
+ * Note that this size defines the size of the per-worker mem pool. The
+ * advantage is that, we're not only reducing the rate of small iot_request_t
+ * allocations from the heap but also reducing the contention on the libc heap
+ * by having a mem pool, though small, for each worker.
+ */
+#define IOT_REQUEST_MEMPOOL_SIZE        64
+
 struct iot_worker {
         struct list_head rqlist;      /* List of requests assigned to me. */
         struct iot_conf  *conf;
+#ifndef HAVE_SPINLOCK
+        pthread_cond_t   notifier;
+#else
+        sem_t            notifier;
+#endif
         int64_t          q,dq;
-        pthread_cond_t   dq_cond;
-        pthread_mutex_t  qlock;
+        gf_lock_t        qlock;
         int32_t          queue_size;
         pthread_t        thread;
-        int              state;            /* What state is the thread in. */
+        iot_state_t      state;            /* What state is the thread in. */
         int              thread_idx;       /* Thread's index into the worker
                                               array. Since this will be thread
                                               local data, for ensuring that
@@ -88,6 +111,7 @@ struct iot_worker {
                                               would have been required to update
                                               centralized state inside conf.
                                            */
+        struct mem_pool  *req_pool;    /* iot_request_t's come from here. */
 };
 
 struct iot_conf {
