@@ -18,6 +18,7 @@
 */
 
 #include "quick-read.h"
+#include "statedump.h"
 
 int32_t
 qr_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
@@ -149,7 +150,7 @@ out:
 int32_t
 qr_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                int32_t op_ret, int32_t op_errno, inode_t *inode,
-               struct stat *buf, dict_t *dict)
+               struct stat *buf, dict_t *dict, struct stat *postparent)
 {
         data_t    *content = NULL;
         qr_file_t *qr_file = NULL;
@@ -239,7 +240,8 @@ out:
          * FIXME: content size in dict can be greater than the size application 
          * requested for. Applications need to be careful till this is fixed.
          */
-	STACK_UNWIND (frame, op_ret, op_errno, inode, buf, dict);
+	STACK_UNWIND_STRICT (lookup, frame, op_ret, op_errno, inode, buf, dict,
+                             postparent);
         return 0;
 }
 
@@ -320,7 +322,8 @@ qr_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xattr_req)
         return 0;
 
 unwind:
-        STACK_UNWIND (frame, op_ret, op_errno, NULL, NULL, NULL);
+        STACK_UNWIND_STRICT (lookup, frame, op_ret, op_errno, NULL, NULL, NULL,
+                             NULL);
 
         if (new_req_dict) {
                 dict_unref (new_req_dict);
@@ -403,7 +406,7 @@ qr_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
         }
 out: 
         if (is_open) {
-                STACK_UNWIND (frame, op_ret, op_errno, fd);
+                STACK_UNWIND_STRICT (open, frame, op_ret, op_errno, fd);
         }
 
         return 0;
@@ -412,7 +415,7 @@ out:
 
 int32_t
 qr_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
-         fd_t *fd)
+         fd_t *fd, int32_t wbflags)
 {
         qr_file_t   *qr_file = NULL;
         int32_t      ret = -1;
@@ -508,12 +511,12 @@ unwind:
                 FREE (local);
         }
 
-        STACK_UNWIND (frame, op_ret, op_errno, fd);
+        STACK_UNWIND_STRICT (open, frame, op_ret, op_errno, fd);
         return 0;
 
 wind:
         STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
-                    FIRST_CHILD(this)->fops->open, loc, flags, fd);
+                    FIRST_CHILD(this)->fops->open, loc, flags, fd, wbflags);
         return 0;
 }
 
@@ -588,7 +591,7 @@ qr_validate_cache_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         frame->local = NULL;
 
         call_resume (local->stub);
-        
+
         FREE (local);
         return 0;
 
@@ -599,7 +602,8 @@ unwind:
         }
 
         /* this is actually unwind of readv */
-        STACK_UNWIND (frame, op_ret, op_errno, NULL, -1, NULL, NULL);
+        STACK_UNWIND_STRICT (readv, frame, op_ret, op_errno, NULL, -1, NULL,
+                             NULL);
         return 0;
 }
 
@@ -609,7 +613,7 @@ qr_validate_cache_helper (call_frame_t *frame, xlator_t *this, fd_t *fd)
 {
         qr_local_t *local = NULL;
         int32_t     op_ret = -1, op_errno = -1;
-        
+
         local = frame->local;
         if (local == NULL) {
                 op_ret = -1;
@@ -706,7 +710,7 @@ qr_validate_cache (call_frame_t *frame, xlator_t *this, fd_t *fd,
 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
                             FIRST_CHILD(this)->fops->open,
-                            &loc, flags, fd);
+                            &loc, flags, fd, 0);
                         
                 qr_loc_wipe (&loc);
         } else if (can_wind) {
@@ -726,7 +730,8 @@ qr_readv_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
               int32_t op_errno, struct iovec *vector, int32_t count,
               struct stat *stbuf, struct iobref *iobref)
 {
-	STACK_UNWIND (frame, op_ret, op_errno, vector, count, stbuf, iobref);
+	STACK_UNWIND_STRICT (readv, frame, op_ret, op_errno, vector, count,
+                             stbuf, iobref);
 	return 0;
 }
 
@@ -876,8 +881,8 @@ qr_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
 
 out:
         if (content_cached || need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno, vector, count, &stbuf,
-                              iobref);
+                STACK_UNWIND_STRICT (readv, frame, op_ret, op_errno, vector,
+                                     count, &stbuf, iobref);
 
         } else if (need_validation) {
                 stub = fop_readv_stub (frame, qr_readv, fd, size, offset);
@@ -946,7 +951,7 @@ out:
 
                         STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
                                     FIRST_CHILD(this)->fops->open,
-                                    &loc, flags, fd);
+                                    &loc, flags, fd, 0);
                         
                         qr_loc_wipe (&loc);
                 } else if (can_wind) {
@@ -971,10 +976,11 @@ out:
 
 
 int32_t
-qr_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
-               int32_t op_errno, struct stat *stbuf)
+qr_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+               int32_t op_ret, int32_t op_errno, struct stat *prebuf,
+               struct stat *postbuf)
 {
-	STACK_UNWIND (frame, op_ret, op_errno, stbuf);
+	STACK_UNWIND_STRICT (writev, frame, op_ret, op_errno, prebuf, postbuf);
 	return 0;
 }
 
@@ -1065,12 +1071,13 @@ qr_writev (call_frame_t *frame, xlator_t *this, fd_t *fd, struct iovec *vector,
 
 out:
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno, NULL);
+                STACK_UNWIND_STRICT (writev, frame, op_ret, op_errno, NULL,
+                                     NULL);
         } else if (can_wind) {
                 STACK_WIND (frame, qr_writev_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->writev, fd, vector, count,
                             off, iobref);
-        } else if (need_open) { 
+        } else if (need_open) {
                 op_ret = qr_loc_fill (&loc, fd->inode, path);
                 if (op_ret == -1) {
                         qr_resume_pending_ops (qr_fd_ctx);
@@ -1078,7 +1085,7 @@ out:
                 }
 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
-                            FIRST_CHILD(this)->fops->open, &loc, flags, fd);
+                            FIRST_CHILD(this)->fops->open, &loc, flags, fd, 0);
 
                 qr_loc_wipe (&loc);
         }
@@ -1091,7 +1098,7 @@ int32_t
 qr_fstat_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
 		   int32_t op_errno, struct stat *buf)
 {
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
+	STACK_UNWIND_STRICT (fstat, frame, op_ret, op_errno, buf);
 	return 0;
 }
 
@@ -1159,7 +1166,7 @@ qr_fstat (call_frame_t *frame, xlator_t *this, fd_t *fd)
 
 out:
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno, NULL);
+                STACK_UNWIND_STRICT (fstat, frame, op_ret, op_errno, NULL);
         } else if (can_wind) {
                 STACK_WIND (frame, qr_fstat_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->fstat, fd);
@@ -1171,7 +1178,7 @@ out:
                 }
 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
-                            FIRST_CHILD(this)->fops->open, &loc, flags, fd);
+                            FIRST_CHILD(this)->fops->open, &loc, flags, fd, 0);
 
                 qr_loc_wipe (&loc);
         }
@@ -1180,124 +1187,36 @@ out:
 }
 
 
-static int32_t
-qr_fchown_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-               int32_t op_ret, int32_t op_errno, struct stat *buf)
+
+
+int32_t
+qr_fsetattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                 int32_t op_ret, int32_t op_errno,
+                 struct stat *preop, struct stat *postop)
 {
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
+	STACK_UNWIND_STRICT (fsetattr, frame, op_ret, op_errno, preop, postop);
 	return 0;
 }
 
 
 int32_t
-qr_fchown_helper (call_frame_t *frame, xlator_t *this, fd_t *fd, uid_t uid,
-                  gid_t gid)
+qr_fsetattr_helper (call_frame_t *frame, xlator_t *this, fd_t *fd,
+                    struct stat *stbuf, int32_t valid)
 {
-        STACK_WIND (frame, qr_fchown_cbk, FIRST_CHILD (this),
-                    FIRST_CHILD (this)->fops->fchown, fd, uid, gid);
+        STACK_WIND(frame, qr_fsetattr_cbk, FIRST_CHILD(this),
+                   FIRST_CHILD(this)->fops->fsetattr, fd, stbuf,
+                   valid);
         return 0;
 }
 
 
 int32_t
-qr_fchown (call_frame_t *frame, xlator_t *this,	fd_t *fd, uid_t uid, gid_t gid)
+qr_fsetattr (call_frame_t *frame, xlator_t *this, fd_t *fd,
+             struct stat *stbuf, int32_t valid)
 {
         uint64_t     value = 0;
         int          flags = 0;
-        call_stub_t *stub = NULL;  
-        char        *path = NULL;
-        loc_t        loc = {0, };
-        qr_fd_ctx_t *qr_fd_ctx = NULL;
-        int32_t      ret = -1, op_ret = -1, op_errno = -1;
-        char         need_open = 0, can_wind = 0, need_unwind = 0;
-
-        ret = fd_ctx_get (fd, this, &value);
-        if (ret == 0) {
-                qr_fd_ctx = (qr_fd_ctx_t *)(long) value;
-        }
-
-        if (qr_fd_ctx) {
-                LOCK (&qr_fd_ctx->lock);
-                {
-                        path = qr_fd_ctx->path;
-                        flags = qr_fd_ctx->flags;
-
-                        if (!(qr_fd_ctx->opened
-                              || qr_fd_ctx->open_in_transit)) {
-                                need_open = 1;
-                                qr_fd_ctx->open_in_transit = 1;
-                        }
-
-                        if (qr_fd_ctx->opened) {
-                                can_wind = 1;
-                        } else {
-                                stub = fop_fchown_stub (frame, qr_fchown_helper,
-                                                        fd, uid, gid);
-                                if (stub == NULL) {
-                                        op_ret = -1;
-                                        op_errno = ENOMEM;
-                                        need_unwind = 1;
-                                        qr_fd_ctx->open_in_transit = 0;
-                                        goto unlock;
-                                }
-
-                                list_add_tail (&stub->list,
-                                               &qr_fd_ctx->waiting_ops);
-                        }
-                }
-        unlock:
-                UNLOCK (&qr_fd_ctx->lock);
-        } else {
-                can_wind = 1;
-        }
-
-out:
-        if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno, NULL);
-        } else if (can_wind) {
-                STACK_WIND (frame, qr_fchown_cbk, FIRST_CHILD (this),
-                            FIRST_CHILD (this)->fops->fchown, fd, uid, gid);
-        } else if (need_open) {
-                op_ret = qr_loc_fill (&loc, fd->inode, path);
-                if (op_ret == -1) {
-                        qr_resume_pending_ops (qr_fd_ctx);
-                        goto out;
-                }
-
-                STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
-                            FIRST_CHILD(this)->fops->open, &loc, flags, fd);
-
-                qr_loc_wipe (&loc);
-        }
-        
-        return 0;
-}
-
-
-int32_t
-qr_fchmod_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-               int32_t op_ret, int32_t op_errno, struct stat *buf)
-{
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
-	return 0;
-}
-
-
-int32_t
-qr_fchmod_helper (call_frame_t *frame, xlator_t *this, fd_t *fd, mode_t mode)
-{
-        STACK_WIND(frame, qr_fchmod_cbk, FIRST_CHILD(this),
-                   FIRST_CHILD(this)->fops->fchmod, fd, mode);
-        return 0;
-}
-
-
-int32_t
-qr_fchmod (call_frame_t *frame, xlator_t *this,	fd_t *fd, mode_t mode)
-{
-        uint64_t     value = 0;
-        int          flags = 0;
-        call_stub_t *stub = NULL;  
+        call_stub_t *stub = NULL;
         char        *path = NULL;
         loc_t        loc = {0, };
         qr_fd_ctx_t *qr_fd_ctx = NULL;
@@ -1323,8 +1242,9 @@ qr_fchmod (call_frame_t *frame, xlator_t *this,	fd_t *fd, mode_t mode)
                         if (qr_fd_ctx->opened) {
                                 can_wind = 1;
                         } else {
-                                stub = fop_fchmod_stub (frame, qr_fchmod_helper,
-                                                        fd, mode);
+                                stub = fop_fsetattr_stub (frame,
+                                                          qr_fsetattr_helper,
+                                                          fd, stbuf, valid);
                                 if (stub == NULL) {
                                         op_ret = -1;
                                         op_errno = ENOMEM;
@@ -1345,10 +1265,12 @@ qr_fchmod (call_frame_t *frame, xlator_t *this,	fd_t *fd, mode_t mode)
 
 out:
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno, NULL);
+                STACK_UNWIND_STRICT (fsetattr, frame, op_ret, op_errno, NULL,
+                                     NULL);
         } else if (can_wind) {
-                STACK_WIND (frame, qr_fchmod_cbk, FIRST_CHILD (this),
-                            FIRST_CHILD (this)->fops->fchmod, fd, mode);
+                STACK_WIND (frame, qr_fsetattr_cbk, FIRST_CHILD (this),
+                            FIRST_CHILD (this)->fops->fsetattr, fd, stbuf,
+                            valid);
         } else if (need_open) {
                 op_ret = qr_loc_fill (&loc, fd->inode, path);
                 if (op_ret == -1) {
@@ -1357,7 +1279,7 @@ out:
                 }
 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
-                            FIRST_CHILD(this)->fops->open, &loc, flags, fd);
+                            FIRST_CHILD(this)->fops->open, &loc, flags, fd, 0);
 
                 qr_loc_wipe (&loc);
         }
@@ -1370,7 +1292,7 @@ int32_t
 qr_fsetxattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                   int32_t op_ret, int32_t op_errno)
 {
-	STACK_UNWIND (frame, op_ret, op_errno);
+	STACK_UNWIND_STRICT (fsetxattr, frame, op_ret, op_errno);
 	return 0;
 }
 
@@ -1441,7 +1363,7 @@ qr_fsetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *dict,
 
 out:
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno);
+                STACK_UNWIND_STRICT (fsetxattr, frame, op_ret, op_errno);
         } else if (can_wind) {
                 STACK_WIND (frame, qr_fsetxattr_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->fsetxattr, fd, dict,
@@ -1455,7 +1377,7 @@ out:
 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
                             FIRST_CHILD(this)->fops->open, &loc, open_flags,
-                            fd);
+                            fd, 0);
 
                 qr_loc_wipe (&loc);
         } 
@@ -1468,7 +1390,7 @@ int32_t
 qr_fgetxattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  int32_t op_ret, int32_t op_errno, dict_t *dict)
 {
-	STACK_UNWIND (frame, op_ret, op_errno, dict);
+	STACK_UNWIND_STRICT (fgetxattr, frame, op_ret, op_errno, dict);
 	return 0;
 }
 
@@ -1543,7 +1465,7 @@ qr_fgetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd, const char *name)
 
 out:
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno, NULL);
+                STACK_UNWIND_STRICT (open, frame, op_ret, op_errno, NULL);
         } else if (can_wind) {
                 STACK_WIND (frame, qr_fgetxattr_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->fgetxattr, fd, name);
@@ -1555,7 +1477,7 @@ out:
                 }
                 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
-                            FIRST_CHILD(this)->fops->open, &loc, flags, fd);
+                            FIRST_CHILD(this)->fops->open, &loc, flags, fd, 0);
 
                 qr_loc_wipe (&loc);
         }
@@ -1568,7 +1490,7 @@ int32_t
 qr_flush_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
               int32_t op_errno)
 {
-	STACK_UNWIND (frame, op_ret, op_errno);
+	STACK_UNWIND_STRICT (flush, frame, op_ret, op_errno);
 	return 0;
 }
 
@@ -1626,7 +1548,7 @@ qr_flush (call_frame_t *frame, xlator_t *this, fd_t *fd)
         }
 
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno);
+                STACK_UNWIND_STRICT (flush, frame, op_ret, op_errno);
         } else if (can_wind) {
                 STACK_WIND (frame, qr_flush_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->flush, fd);
@@ -1640,7 +1562,7 @@ int32_t
 qr_fentrylk_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  int32_t op_ret, int32_t op_errno)
 {
-	STACK_UNWIND (frame, op_ret, op_errno);
+	STACK_UNWIND_STRICT (fentrylk, frame, op_ret, op_errno);
 	return 0;
 }
 
@@ -1713,7 +1635,7 @@ qr_fentrylk (call_frame_t *frame, xlator_t *this, const char *volume, fd_t *fd,
 
 out:
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno);
+                STACK_UNWIND_STRICT (fentrylk, frame, op_ret, op_errno);
         } else if (can_wind) {
                 STACK_WIND (frame, qr_fentrylk_cbk, FIRST_CHILD(this),
                             FIRST_CHILD(this)->fops->fentrylk, volume, fd,
@@ -1726,7 +1648,7 @@ out:
                 }
 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
-                            FIRST_CHILD(this)->fops->open, &loc, flags, fd);
+                            FIRST_CHILD(this)->fops->open, &loc, flags, fd, 0);
 
                 qr_loc_wipe (&loc);
         }
@@ -1740,7 +1662,7 @@ qr_finodelk_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  int32_t op_ret, int32_t op_errno)
 
 {
-	STACK_UNWIND (frame, op_ret, op_errno);
+	STACK_UNWIND_STRICT (finodelk, frame, op_ret, op_errno);
 	return 0;
 }
 
@@ -1812,7 +1734,7 @@ qr_finodelk (call_frame_t *frame, xlator_t *this, const char *volume, fd_t *fd,
 
 out:
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno);
+                STACK_UNWIND_STRICT (finodelk, frame, op_ret, op_errno);
         } else if (can_wind) {
                 STACK_WIND (frame, qr_finodelk_cbk, FIRST_CHILD(this),
                             FIRST_CHILD(this)->fops->finodelk, volume, fd,
@@ -1825,7 +1747,7 @@ out:
                 }
 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
-                            FIRST_CHILD(this)->fops->open, &loc, flags, fd);
+                            FIRST_CHILD(this)->fops->open, &loc, flags, fd, 0);
                 
                 qr_loc_wipe (&loc);
         }
@@ -1836,9 +1758,9 @@ out:
 
 int32_t
 qr_fsync_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int32_t op_ret,
-              int32_t op_errno)
+              int32_t op_errno, struct stat *prebuf, struct stat *postbuf)
 {
-	STACK_UNWIND (frame, op_ret, op_errno);
+	STACK_UNWIND_STRICT (fsync, frame, op_ret, op_errno, prebuf, postbuf);
 	return 0;
 }
 
@@ -1905,7 +1827,8 @@ qr_fsync (call_frame_t *frame, xlator_t *this, fd_t *fd, int32_t flags)
 
 out:
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno);
+                STACK_UNWIND_STRICT (fsync, frame, op_ret, op_errno, NULL,
+                                     NULL);
         } else if (can_wind) {
                 STACK_WIND (frame, qr_fsync_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->fsync, fd, flags);
@@ -1918,21 +1841,22 @@ out:
 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
                             FIRST_CHILD(this)->fops->open, &loc, open_flags,
-                            fd);
+                            fd, 0);
 
                 qr_loc_wipe (&loc);
         }
-        
+
         return 0;
 }
 
 
 int32_t
 qr_ftruncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                  int32_t op_ret, int32_t op_errno, struct stat *buf)
+                  int32_t op_ret, int32_t op_errno, struct stat *prebuf,
+                  struct stat *postbuf)
 {
         int32_t     ret = 0;
-        uint64_t    value = 0; 
+        uint64_t    value = 0;
         qr_file_t  *qr_file = NULL;
         qr_local_t *local = NULL;
 
@@ -1947,15 +1871,16 @@ qr_ftruncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 op_errno = EINVAL;
                 goto out;
         }
-                
+
         ret = inode_ctx_get (local->fd->inode, this, &value);
         if (ret == 0) {
                 qr_file = (qr_file_t *)(long) value;
-                
+
                 if (qr_file) {
                         LOCK (&qr_file->lock);
                         {
-                                if (qr_file->stbuf.st_size != buf->st_size) {
+                                if (qr_file->stbuf.st_size != postbuf->st_size)
+                                {
                                         dict_unref (qr_file->xattr);
                                         qr_file->xattr = NULL;
                                 }
@@ -1965,7 +1890,8 @@ qr_ftruncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         }
 
 out:
-        STACK_UNWIND (frame, op_ret, op_errno, buf);
+        STACK_UNWIND_STRICT (ftruncate, frame, op_ret, op_errno, prebuf,
+                             postbuf);
         return 0;
 }
 
@@ -1997,7 +1923,7 @@ qr_ftruncate (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset)
         if (ret == 0) {
                 qr_fd_ctx = (qr_fd_ctx_t *)(long)value;
         }
-        
+
         local = CALLOC (1, sizeof (*local));
         if (local == NULL) {
                 op_ret = -1;
@@ -2008,7 +1934,7 @@ qr_ftruncate (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset)
 
         local->fd = fd;
         frame->local = local;
- 
+
         if (qr_fd_ctx) {
                 LOCK (&qr_fd_ctx->lock);
                 {
@@ -2047,7 +1973,8 @@ qr_ftruncate (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset)
 
 out:
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno, NULL);
+                STACK_UNWIND_STRICT (ftruncate, frame, op_ret, op_errno, NULL,
+                                     NULL);
         } else if (can_wind) {
                 STACK_WIND (frame, qr_ftruncate_cbk, FIRST_CHILD(this),
                             FIRST_CHILD(this)->fops->ftruncate, fd, offset);
@@ -2059,11 +1986,11 @@ out:
                 }
 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
-                            FIRST_CHILD(this)->fops->open, &loc, flags, fd);
+                            FIRST_CHILD(this)->fops->open, &loc, flags, fd, 0);
 
                 qr_loc_wipe (&loc);
         }
-        
+
         return 0;
 }
 
@@ -2072,7 +1999,7 @@ int32_t
 qr_lk_cbk (call_frame_t *frame,	void *cookie, xlator_t *this, int32_t op_ret,
            int32_t op_errno, struct flock *lock)
 {
-	STACK_UNWIND (frame, op_ret, op_errno, lock);
+	STACK_UNWIND_STRICT (lk, frame, op_ret, op_errno, lock);
 	return 0;
 }
 
@@ -2143,7 +2070,7 @@ qr_lk (call_frame_t *frame, xlator_t *this, fd_t *fd, int32_t cmd,
 
 out:
         if (need_unwind) {
-                STACK_UNWIND (frame, op_ret, op_errno, NULL);
+                STACK_UNWIND_STRICT (lk, frame, op_ret, op_errno, NULL);
         } else if (can_wind) {
                 STACK_WIND (frame, qr_lk_cbk, FIRST_CHILD(this),
                             FIRST_CHILD(this)->fops->lk, fd, cmd, lock);
@@ -2155,7 +2082,7 @@ out:
                 }
 
                 STACK_WIND (frame, qr_open_cbk, FIRST_CHILD(this),
-                            FIRST_CHILD(this)->fops->open, &loc, flags, fd);
+                            FIRST_CHILD(this)->fops->open, &loc, flags, fd, 0);
 
                 qr_loc_wipe (&loc);
         }
@@ -2210,6 +2137,36 @@ qr_forget (xlator_t *this, inode_t *inode)
         return 0;
 }
 
+int
+qr_priv_dump (xlator_t *this)
+{
+        qr_conf_t       *conf = NULL;
+        char            key[GF_DUMP_MAX_BUF_LEN];
+        char            key_prefix[GF_DUMP_MAX_BUF_LEN];
+
+        if (!this)
+                return -1;
+
+        conf = this->private;
+        if (!conf) {
+                gf_log (this->name, GF_LOG_WARNING,
+                        "conf null in xlator");
+                return -1;
+        }
+
+        gf_proc_dump_build_key (key_prefix,
+                                "xlator.performance.quick-read",
+                                "priv");
+
+        gf_proc_dump_add_section (key_prefix);
+
+        gf_proc_dump_build_key (key, key_prefix, "max_file_size");
+        gf_proc_dump_write (key, "%d", conf->max_file_size);
+        gf_proc_dump_build_key (key, key_prefix, "cache_timeout");
+        gf_proc_dump_write (key, "%d", conf->cache_timeout);
+
+        return 0;
+}
 
 int32_t 
 init (xlator_t *this)
@@ -2291,8 +2248,6 @@ struct xlator_fops fops = {
         .readv       = qr_readv,
         .writev      = qr_writev,
         .fstat       = qr_fstat,
-        .fchown      = qr_fchown,
-        .fchmod      = qr_fchmod,
         .fsetxattr   = qr_fsetxattr,
         .fgetxattr   = qr_fgetxattr,
         .flush       = qr_flush,
@@ -2301,6 +2256,7 @@ struct xlator_fops fops = {
         .fsync       = qr_fsync,
         .ftruncate   = qr_ftruncate,
         .lk          = qr_lk,
+        .fsetattr    = qr_fsetattr,
 };
 
 
@@ -2313,6 +2269,10 @@ struct xlator_cbks cbks = {
         .release = qr_release, 
 };
 
+struct xlator_dumpops dumpops = {
+        .priv      =  qr_priv_dump,
+};
+
 struct volume_options options[] = {
         { .key  = {"cache-timeout"},
           .type = GF_OPTION_TYPE_INT,
